@@ -22,18 +22,17 @@
 
 #include <QCoreApplication>
 #include <QDateTime>
-#include <QDir>
 #include <QFile>
 #include <QSettings>
-#include <QUrl>
+#include <QStorageInfo>
+
+#include <filesystem>
 
 #include <sys/stat.h>
 #include <sys/types.h>
 
 #ifdef Q_OS_WIN
-#include <windows.h>
-#include <windef.h>
-#include <winbase.h>
+#include "utility_win.h"
 #include <fcntl.h>
 #include <io.h>
 #endif
@@ -43,7 +42,7 @@ namespace {
 // https://en.wikipedia.org/wiki/Comparison_of_file_systems#Limits
 // the file name limit appears to be about 255 chars
 // -10 for safety
-constexpr int fileNameMaxC = 255 - 10;
+constexpr qsizetype fileNameMaxC = 255 - 10;
 
 constexpr auto replacementCharC = QLatin1Char('_');
 }
@@ -52,6 +51,16 @@ namespace OCC {
 
 Q_LOGGING_CATEGORY(lcFileSystem, "sync.filesystem", QtInfoMsg)
 
+QByteArray FileSystem::encodeFileName(const QString &fileName)
+{
+    return fileName.toLocal8Bit();
+}
+
+QString FileSystem::decodeFileName(const char *localFileName)
+{
+    return QString::fromLocal8Bit(localFileName);
+}
+
 QString FileSystem::longWinPath(const QString &inpath)
 {
 #ifndef Q_OS_WIN
@@ -163,16 +172,14 @@ bool FileSystem::rename(const QString &originFileName,
     bool success = false;
     QString error;
 #ifdef Q_OS_WIN
-    const QString orig = longWinPath(originFileName);
+    const QString originalFileNameLong = longWinPath(originFileName);
     const QString dest = longWinPath(destinationFileName);
     if (FileSystem::isFileLocked(dest, FileSystem::LockMode::Exclusive)) {
         error = QCoreApplication::translate("FileSystem", "Can't rename %1, the file is currently in use").arg(destinationFileName);
-    } else if (FileSystem::isFileLocked(orig, FileSystem::LockMode::Exclusive)) {
+    } else if (FileSystem::isFileLocked(originalFileNameLong, FileSystem::LockMode::Exclusive)) {
         error = QCoreApplication::translate("FileSystem", "Can't rename %1, the file is currently in use").arg(originFileName);
     } else if (isLnkFile(originFileName) || isLnkFile(destinationFileName)) {
-        success = MoveFileEx((wchar_t *)orig.utf16(),
-            (wchar_t *)dest.utf16(),
-            MOVEFILE_COPY_ALLOWED | MOVEFILE_WRITE_THROUGH);
+        success = MoveFileEx((wchar_t *)originalFileNameLong.utf16(), (wchar_t *)dest.utf16(), MOVEFILE_COPY_ALLOWED | MOVEFILE_WRITE_THROUGH);
         if (!success) {
             error = Utility::formatWinError(GetLastError());
         }
@@ -202,28 +209,15 @@ bool FileSystem::uncheckedRenameReplace(const QString &originFileName,
     QString *errorString)
 {
     Q_ASSERT(errorString);
+
 #ifndef Q_OS_WIN
-    bool success;
-    QFile orig(originFileName);
-    // We want a rename that also overwites.  QFile::rename does not overwite.
-    // Qt 5.1 has QSaveFile::renameOverwrite we could use.
-    // ### FIXME
-    success = true;
-    bool destExists = fileExists(destinationFileName);
-    if (destExists && !QFile::remove(destinationFileName)) {
-        *errorString = orig.errorString();
-        qCWarning(lcFileSystem) << "Target file could not be removed.";
-        success = false;
-    }
-    if (success) {
-        success = orig.rename(destinationFileName);
-    }
-    if (!success) {
-        *errorString = orig.errorString();
+    std::error_code err;
+    std::filesystem::rename(originFileName.toStdString(), destinationFileName.toStdString(), err);
+    if (err) {
+        *errorString = QString::fromStdString(err.message());
         qCWarning(lcFileSystem) << "Renaming temp file to final failed: " << *errorString;
         return false;
     }
-
 #else //Q_OS_WIN
     // You can not overwrite a read-only file on windows.
 
@@ -274,17 +268,10 @@ bool FileSystem::openAndSeekFileSharedRead(QFile *file, QString *errorOrNull, qi
     DWORD creationDisp = OPEN_EXISTING;
 
     // Create the file handle.
-    SECURITY_ATTRIBUTES securityAtts = { sizeof(SECURITY_ATTRIBUTES), NULL, FALSE };
+    SECURITY_ATTRIBUTES securityAtts = {sizeof(SECURITY_ATTRIBUTES), nullptr, FALSE};
     QString fName = longWinPath(file->fileName());
 
-    HANDLE fileHandle = CreateFileW(
-        (const wchar_t *)fName.utf16(),
-        accessRights,
-        shareMode,
-        &securityAtts,
-        creationDisp,
-        FILE_ATTRIBUTE_NORMAL,
-        NULL);
+    HANDLE fileHandle = CreateFileW((const wchar_t *)fName.utf16(), accessRights, shareMode, &securityAtts, creationDisp, FILE_ATTRIBUTE_NORMAL, nullptr);
 
     // Bail out on error.
     if (fileHandle == INVALID_HANDLE_VALUE) {
@@ -355,7 +342,7 @@ bool FileSystem::fileExists(const QString &filename, const QFileInfo &fileInfo)
 #endif
     bool re = fileInfo.exists();
     // if the filename is different from the filename in fileInfo, the fileInfo is
-    // not valid. There needs to be one initialised here. Otherwise the incoming
+    // not valid. There needs to be one initialised here. Otherwise, the incoming
     // fileInfo is re-used.
     if (fileInfo.filePath() != filename) {
         re = QFileInfo::exists(filename);
@@ -363,28 +350,41 @@ bool FileSystem::fileExists(const QString &filename, const QFileInfo &fileInfo)
     return re;
 }
 
-#ifdef Q_OS_WIN
-QString FileSystem::fileSystemForPath(const QString &path)
+bool FileSystem::mkpath(const QString &parent, const QString &newDir)
 {
-    // See also QStorageInfo (Qt >=5.4) and GetVolumeInformationByHandleW (>= Vista)
-    QString drive = path.left(2);
-    if (!drive.endsWith(QLatin1Char(':')))
-        return QString();
-    drive.append(QLatin1Char('\\'));
-
-    const size_t fileSystemBufferSize = 4096;
-    TCHAR fileSystemBuffer[fileSystemBufferSize];
-
-    if (!GetVolumeInformationW(
-            reinterpret_cast<LPCWSTR>(drive.utf16()),
-            NULL, 0,
-            NULL, NULL, NULL,
-            fileSystemBuffer, fileSystemBufferSize)) {
-        return QString();
+#ifdef Q_OS_WIN
+    return QDir(parent).mkpath(newDir);
+#else // POSIX
+    std::error_code err;
+    QString fullPath = parent;
+    if (!fullPath.endsWith(u'/')) {
+        fullPath += u'/';
     }
-    return QString::fromUtf16(reinterpret_cast<const ushort *>(fileSystemBuffer));
+    fullPath += newDir;
+    std::filesystem::create_directories(fullPath.toStdString(), err);
+    return err.value() == 0;
+#endif
 }
 
+QString FileSystem::fileSystemForPath(const QString &path)
+{
+    QString p = path;
+    while (true) {
+        if (!fileExists(p)) {
+            QFileInfo file(p);
+            p = file.absolutePath();
+            continue;
+        }
+        const QStorageInfo storage(p);
+        if (!storage.isValid() || !storage.isReady()) {
+            return {};
+        }
+
+        return QString::fromUtf8(storage.fileSystemType());
+    }
+}
+
+#ifdef Q_OS_WIN
 bool FileSystem::longPathsEnabledOnWindows()
 {
     static std::optional<bool> longPathsEnabledCached = {};
@@ -422,127 +422,52 @@ bool FileSystem::remove(const QString &fileName, QString *errorString)
     return true;
 }
 
-bool FileSystem::moveToTrash(const QString &fileName, QString *errorString)
-{
-    // TODO: Qt 5.15 bool QFile::moveToTrash()
-#if defined Q_OS_UNIX && !defined Q_OS_MAC
-    QString trashPath, trashFilePath, trashInfoPath;
-    QString xdgDataHome = QFile::decodeName(qgetenv("XDG_DATA_HOME"));
-    if (xdgDataHome.isEmpty()) {
-        trashPath = QDir::homePath() + QStringLiteral("/.local/share/Trash/"); // trash path that should exist
-    } else {
-        trashPath = xdgDataHome + QStringLiteral("/Trash/");
-    }
-
-    trashFilePath = trashPath + QStringLiteral("files/"); // trash file path contain delete files
-    trashInfoPath = trashPath + QStringLiteral("info/"); // trash info path contain delete files information
-
-    if (!(QDir().mkpath(trashFilePath) && QDir().mkpath(trashInfoPath))) {
-        *errorString = QCoreApplication::translate("FileSystem", "Could not make directories in trash");
-        return false; //mkpath will return true if path exists
-    }
-
-    QFileInfo f(fileName);
-
-    QDir file;
-    int suffix_number = 1;
-    if (file.exists(trashFilePath + f.fileName())) { //file in trash already exists, move to "filename.1"
-        QString path = trashFilePath + f.fileName() + QLatin1Char('.');
-        while (file.exists(path + QString::number(suffix_number))) { //or to "filename.2" if "filename.1" exists, etc
-            suffix_number++;
-        }
-        if (!file.rename(f.absoluteFilePath(), path + QString::number(suffix_number))) { // rename(file old path, file trash path)
-            *errorString = QCoreApplication::translate("FileSystem", "Could not move '%1' to '%2'")
-                               .arg(f.absoluteFilePath(), path + QString::number(suffix_number));
-            return false;
-        }
-    } else {
-        if (!file.rename(f.absoluteFilePath(), trashFilePath + f.fileName())) { // rename(file old path, file trash path)
-            *errorString = QCoreApplication::translate("FileSystem", "Could not move '%1' to '%2'")
-                               .arg(f.absoluteFilePath(), trashFilePath + f.fileName());
-            return false;
-        }
-    }
-
-    // create file format for trash info file----- START
-    QFile infoFile;
-    if (file.exists(trashInfoPath + f.fileName() + QStringLiteral(".trashinfo"))) { //TrashInfo file already exists, create "filename.1.trashinfo"
-        QString filename = trashInfoPath + f.fileName() + QLatin1Char('.') + QString::number(suffix_number) + QStringLiteral(".trashinfo");
-        infoFile.setFileName(filename); //filename+.trashinfo //  create file information file in /.local/share/Trash/info/ folder
-    } else {
-        QString filename = trashInfoPath + f.fileName() + QStringLiteral(".trashinfo");
-        infoFile.setFileName(filename); //filename+.trashinfo //  create file information file in /.local/share/Trash/info/ folder
-    }
-
-    infoFile.open(QIODevice::ReadWrite);
-
-    QTextStream stream(&infoFile); // for write data on open file
-
-    stream << "[Trash Info]\n"
-           << "Path="
-           << QUrl::toPercentEncoding(f.absoluteFilePath(), "~_-./")
-           << "\n"
-           << "DeletionDate="
-           << QDateTime::currentDateTime().toString(Qt::ISODate)
-           << '\n';
-    infoFile.close();
-
-    // create info file format of trash file----- END
-
-    return true;
-#else
-    Q_UNUSED(fileName)
-    *errorString = QCoreApplication::translate("FileSystem", "Moving to the trash is not implemented on this platform");
-    return false;
-#endif
-}
-
 #ifdef Q_OS_WIN
-Utility::Handle FileSystem::lockFile(const QString &fileName, LockMode mode)
-{
-    const QString fName = longWinPath(fileName);
-    int shareMode = 0;
-    int accessMode = GENERIC_READ | GENERIC_WRITE;
-    switch (mode) {
-    case LockMode::Exclusive:
-        shareMode = 0;
-        break;
-    case LockMode::Shared:
-        shareMode = FILE_SHARE_READ | FILE_SHARE_WRITE | FILE_SHARE_DELETE;
-        break;
-    case LockMode::SharedRead:
-        shareMode = FILE_SHARE_READ | FILE_SHARE_WRITE | FILE_SHARE_DELETE;
-        accessMode = GENERIC_READ;
-        break;
-    }
-    // Check if file exists
-    DWORD attr = GetFileAttributesW(reinterpret_cast<const wchar_t *>(fName.utf16()));
-    if (attr != INVALID_FILE_ATTRIBUTES) {
-        // Try to open the file with as much access as possible..
-        auto out = Utility::Handle { CreateFileW(
-            reinterpret_cast<const wchar_t *>(fName.utf16()),
-            accessMode,
-            shareMode,
-            nullptr,
-            OPEN_EXISTING,
-            FILE_ATTRIBUTE_NORMAL | FILE_FLAG_BACKUP_SEMANTICS,
-            nullptr) };
 
-        if (out) {
-            LARGE_INTEGER start;
-            start.QuadPart = 0;
-            LARGE_INTEGER end;
-            end.QuadPart = -1;
-            if (LockFile(out.handle(), start.LowPart, start.HighPart, end.LowPart, end.HighPart)) {
-                OC_ENSURE(UnlockFile(out.handle(), start.LowPart, start.HighPart, end.LowPart, end.HighPart));
-                return out;
-            } else {
-                return {};
-            }
+namespace {
+    /**
+     * This function creates a file handle with the desired LockMode
+     */
+    Utility::Handle lockFile(const QString &fileName, FileSystem::LockMode mode)
+    {
+        const QString fName = FileSystem::longWinPath(fileName);
+        int shareMode = 0;
+        int accessMode = GENERIC_READ | GENERIC_WRITE;
+        switch (mode) {
+        case FileSystem::LockMode::Exclusive:
+            shareMode = 0;
+            break;
+        case FileSystem::LockMode::Shared:
+            shareMode = FILE_SHARE_READ | FILE_SHARE_WRITE | FILE_SHARE_DELETE;
+            break;
+        case FileSystem::LockMode::SharedRead:
+            shareMode = FILE_SHARE_READ | FILE_SHARE_WRITE | FILE_SHARE_DELETE;
+            accessMode = GENERIC_READ;
+            break;
         }
-        return out;
+        // Check if file exists
+        DWORD attr = GetFileAttributesW(reinterpret_cast<const wchar_t *>(fName.utf16()));
+        if (attr != INVALID_FILE_ATTRIBUTES) {
+            // Try to open the file with as much access as possible..
+            auto out = Utility::Handle{CreateFileW(reinterpret_cast<const wchar_t *>(fName.utf16()), accessMode, shareMode, nullptr, OPEN_EXISTING,
+                FILE_ATTRIBUTE_NORMAL | FILE_FLAG_BACKUP_SEMANTICS, nullptr)};
+
+            if (out) {
+                LARGE_INTEGER start;
+                start.QuadPart = 0;
+                LARGE_INTEGER end;
+                end.QuadPart = -1;
+                if (LockFile(out.handle(), start.LowPart, start.HighPart, end.LowPart, end.HighPart)) {
+                    OC_ENFORCE(UnlockFile(out.handle(), start.LowPart, start.HighPart, end.LowPart, end.HighPart));
+                    return out;
+                } else {
+                    return {};
+                }
+            }
+            return out;
+        }
+        return {};
     }
-    return {};
 }
 #endif
 
@@ -575,7 +500,8 @@ bool FileSystem::isJunction(const QString &filename)
 {
 #ifdef Q_OS_WIN
     WIN32_FIND_DATA findData;
-    HANDLE hFind = FindFirstFileEx(reinterpret_cast<const wchar_t *>(longWinPath(filename).utf16()), FindExInfoBasic, &findData, FindExSearchNameMatch, NULL, 0);
+    HANDLE hFind =
+        FindFirstFileEx(reinterpret_cast<const wchar_t *>(longWinPath(filename).utf16()), FindExInfoBasic, &findData, FindExSearchNameMatch, nullptr, 0);
     if (hFind != INVALID_HANDLE_VALUE) {
         FindClose(hFind);
         return false;
@@ -589,7 +515,7 @@ bool FileSystem::isJunction(const QString &filename)
 #endif
 }
 
-bool FileSystem::isChildPathOf(const QString &child, const QString &parent)
+bool FileSystem::isChildPathOf(QStringView child, QStringView parent)
 {
     // if it is a relative path assume a local file, resolve it based on root
     const auto sensitivity = Utility::fsCaseSensitivity();
@@ -598,7 +524,11 @@ bool FileSystem::isChildPathOf(const QString &child, const QString &parent)
     if (parent.isEmpty()) {
         // The empty parent is often used as the sync root, as (child) items do not start with a `/`
         return true;
-    } else if (parent.endsWith(QLatin1Char('/'))) {
+    }
+
+    const auto isSeparator = [](QChar c) { return c == QLatin1Char('/') || (Utility::isWindows() && c == QLatin1Char('\\')); };
+
+    if (isSeparator(parent.back())) {
         // Here we can do a normal prefix check, because the parent is "terminated" with a slash,
         // and we can't walk into the case in the else below.
         if (child.startsWith(parent, sensitivity)) {
@@ -615,7 +545,7 @@ bool FileSystem::isChildPathOf(const QString &child, const QString &parent)
         // first check if the child string starts with the parent string
         if (child.startsWith(parent, sensitivity)) {
             // ok, now check if the character after the parent is a '/'
-            if (child.length() >= parent.length() + 1 && child.at(parent.length()) == QLatin1Char('/')) {
+            if (child.length() >= parent.length() + 1 && isSeparator(child.at(parent.length()))) {
                 return true;
             }
             // else: do the `cleanPath` version below
@@ -623,22 +553,33 @@ bool FileSystem::isChildPathOf(const QString &child, const QString &parent)
     }
 
     // Slow path (`QDir::cleanPath` does lots of string operations):
-    return QString::compare(QDir::cleanPath(parent), QDir::cleanPath(child), sensitivity) == 0;
+    const QString cleanParent = QDir::cleanPath(parent.toString());
+    const QString cleanChild = QDir::cleanPath(child.toString());
+    // cleanPath removes trailing slashes, add one to parent to be sure we handle a child path
+    // /root/foo/bar is not  a child of /root/fo, therefor the trailing slash is important
+    if (cleanChild.startsWith(cleanParent + QLatin1Char('/'), sensitivity)) {
+        return true;
+    }
+    // both paths are the same /root/foo == /root/foo
+    if (cleanChild.compare(cleanParent, sensitivity) == 0) {
+        return true;
+    }
+    return false;
 }
 
-QString OCSYNC_EXPORT FileSystem::createPortableFileName(const QString &path, const QString &fileName, int reservedSize)
+QString FileSystem::createPortableFileName(const QString &path, const QString &fileName, qsizetype reservedSize)
 {
     // remove leading and trailing whitespace
     QString tmp = pathEscape(fileName);
     // limit size to fileNameMaxC
-    tmp.resize(std::min(tmp.size(), fileNameMaxC - reservedSize));
+    tmp.resize(std::min<qsizetype>(tmp.size(), fileNameMaxC - reservedSize));
     // remove eventual trailing whitespace after the resize
     tmp = tmp.trimmed();
 
     return QDir::cleanPath(path + QLatin1Char('/') + tmp);
 }
 
-QString OCSYNC_EXPORT FileSystem::pathEscape(const QString &s)
+QString FileSystem::pathEscape(const QString &s)
 {
     QString tmp = s;
     tmp.replace(QLatin1String("../"), QString(replacementCharC));
