@@ -18,14 +18,17 @@
 
 #pragma once
 
-#include "config.h"
+#include "constexpr_list.h"
+#include "ocsynclib.h"
+#include "utility.h"
 
-#include <QString>
-#include <ctime>
 #include <QFileInfo>
 #include <QLoggingCategory>
+#include <QString>
+
+#include <cstdint>
+#include <ctime>
 
-#include <ocsynclib.h>
 
 class QFile;
 
@@ -44,6 +47,16 @@ OCSYNC_EXPORT Q_DECLARE_LOGGING_CATEGORY(lcFileSystem)
 namespace FileSystem {
     OCSYNC_EXPORT Q_NAMESPACE;
 
+    QByteArray OCSYNC_EXPORT encodeFileName(const QString &fileName);
+    QString OCSYNC_EXPORT decodeFileName(const char *localFileName);
+
+    /**
+     * List of characters not allowd in filenames on Windows
+     */
+    constexpr_list auto IllegalFilenameCharsWindows = {
+        QLatin1Char('\\'), QLatin1Char(':'), QLatin1Char('?'), QLatin1Char('*'), QLatin1Char('"'), QLatin1Char('>'), QLatin1Char('<'), QLatin1Char('|')
+    };
+
     /**
      * @brief Mark the file as hidden  (only has effects on windows)
      */
@@ -74,7 +87,17 @@ namespace FileSystem {
      */
     void OCSYNC_EXPORT setFolderMinimumPermissions(const QString &filename);
 
-    /** convert a "normal" windows path into a path that can be 32k chars long. */
+    /*
+     * This function takes a path and converts it to a UNC representation of the
+     * string. That means that it prepends a \\?\ (unless already UNC) and converts
+     * all slashes to backslashes.
+     *
+     * Note the following:
+     *  - The string must be absolute.
+     *  - it needs to contain a drive character to be a valid UNC
+     *  - A conversion is only done if the path len is larger than 245. Otherwise
+     *    the windows API functions work with the normal "unixoid" representation too.
+     */
     QString OCSYNC_EXPORT longWinPath(const QString &inpath);
 
     /**
@@ -85,6 +108,8 @@ namespace FileSystem {
      */
     bool OCSYNC_EXPORT fileExists(const QString &filename, const QFileInfo & = QFileInfo());
 
+    bool OCSYNC_EXPORT mkpath(const QString &parent, const QString &newDir);
+
     /**
      * @brief Rename the file \a originFileName to \a destinationFileName.
      *
@@ -110,11 +135,6 @@ namespace FileSystem {
      */
     bool OCSYNC_EXPORT remove(const QString &fileName, QString *errorString = nullptr);
 
-    /**
-     * Move the specified file or folder to the trash. (Only implemented on linux)
-     */
-    bool OCSYNC_EXPORT moveToTrash(const QString &filename, QString *errorString);
-
     /**
      * Replacement for QFile::open(ReadOnly) followed by a seek().
      * This version sets a more permissive sharing mode on Windows.
@@ -122,31 +142,12 @@ namespace FileSystem {
      * Warning: The resulting file may have an empty fileName and be unsuitable for use
      * with QFileInfo! Calling seek() on the QFile with >32bit signed values will fail!
      */
-    bool OCSYNC_EXPORT openAndSeekFileSharedRead(QFile *file, QString *error, qint64 seek);
-
-#ifdef Q_OS_WIN
-    /**
-     * Returns the file system used at the given path.
-     */
-    QString fileSystemForPath(const QString &path);
-
-    /*
-     * This function takes a path and converts it to a UNC representation of the
-     * string. That means that it prepends a \\?\ (unless already UNC) and converts
-     * all slashes to backslashes.
-     *
-     * Note the following:
-     *  - The string must be absolute.
-     *  - it needs to contain a drive character to be a valid UNC
-     *  - A conversion is only done if the path len is larger than 245. Otherwise
-     *    the windows API functions work with the normal "unixoid" representation too.
-     */
-    QString OCSYNC_EXPORT pathtoUNC(const QString &str);
-#endif
+    bool OCSYNC_EXPORT openAndSeekFileSharedRead(QFile * file, QString * error, qint64 seek);
 
     enum class LockMode {
         Shared,
-        Exclusive
+        Exclusive,
+        SharedRead
     };
     Q_ENUM_NS(LockMode);
     /**
@@ -154,6 +155,16 @@ namespace FileSystem {
      */
     bool OCSYNC_EXPORT isFileLocked(const QString &fileName, LockMode mode);
 
+#ifdef Q_OS_WIN
+
+    bool OCSYNC_EXPORT longPathsEnabledOnWindows();
+
+#endif
+    /**
+     * Returns the file system used at the given path.
+     */
+    QString OCSYNC_EXPORT fileSystemForPath(const QString &path);
+
     /**
      * Returns whether the file is a shortcut file (ends with .lnk)
      */
@@ -167,7 +178,58 @@ namespace FileSystem {
     /**
      * Returns whether a Path is a child of another
      */
-    bool OCSYNC_EXPORT isChildPathOf(const QString &child, const QString &parent);
+    bool OCSYNC_EXPORT isChildPathOf(QStringView child, QStringView parent);
+
+    /**
+     * Ensures the file name length is allowed on all platforms and the file name does not contain illegal characters
+     * reservedSize: The resulting path will be reservedSize < MAX and allows appending.
+     */
+    QString OCSYNC_EXPORT createPortableFileName(const QString &path, const QString &fileName, qsizetype reservedSize = 0);
+
+    /*
+     * Replace path navigation elements from the string
+     */
+    QString OCSYNC_EXPORT pathEscape(const QString &s);
+
+    namespace SizeLiterals {
+        constexpr auto BinaryBase = 1024;
+        constexpr auto DecimalBase = 1000;
+
+        constexpr unsigned long long operator"" _B(unsigned long long sz)
+        {
+            return sz;
+        }
+
+        constexpr unsigned long long operator"" _KiB(unsigned long long sz)
+        {
+            return operator"" _B(sz) * BinaryBase;
+        }
+
+        constexpr unsigned long long operator"" _MiB(unsigned long long sz)
+        {
+            return operator"" _KiB(sz) * BinaryBase;
+        }
+
+        constexpr unsigned long long operator"" _GiB(unsigned long long sz)
+        {
+            return operator"" _MiB(sz) * BinaryBase;
+        }
+
+        constexpr unsigned long long operator"" _kB(unsigned long long sz)
+        {
+            return operator"" _B(sz) * DecimalBase;
+        }
+
+        constexpr unsigned long long operator"" _MB(unsigned long long sz)
+        {
+            return operator"" _kB(sz) * DecimalBase;
+        }
+
+        constexpr unsigned long long operator"" _GB(unsigned long long sz)
+        {
+            return operator"" _MB(sz) * DecimalBase;
+        }
+    }
 }
 
 /** @} */
