@@ -17,12 +17,17 @@
 #include "accountmanager.h"
 #include "application.h"
 #include "configfile.h"
-#include "creds/abstractcredentials.h"
-#include "creds/httpcredentials.h"
+#include "fetchserversettings.h"
+
+#include "libsync/creds/abstractcredentials.h"
+#include "libsync/creds/httpcredentials.h"
+
+#include "gui/networkinformation.h"
+#include "gui/quotainfo.h"
 #include "gui/settingsdialog.h"
+#include "gui/spacemigration.h"
 #include "gui/tlserrordialog.h"
-#include "logger.h"
-#include "settingsdialog.h"
+
 #include "socketapi/socketapi.h"
 #include "theme.h"
 
@@ -67,8 +72,8 @@ UpdateUrlDialog *AccountState::updateUrlDialog(const QUrl &newUrl)
         Q_EMIT urlUpdated();
     });
 
-    _updateUrlDialog->show();
-    ocApp()->gui()->raiseDialog(_updateUrlDialog);
+    ownCloudGui::raise();
+    _updateUrlDialog->open();
 
     return _updateUrlDialog;
 }
@@ -105,37 +110,103 @@ AccountState::AccountState(AccountPtr account)
         },
         Qt::QueuedConnection);
 
+    connect(NetworkInformation::instance(), &NetworkInformation::reachabilityChanged, this, [this](NetworkInformation::Reachability reachability) {
+        switch (reachability) {
+        case NetworkInformation::Reachability::Online:
+            [[fallthrough]];
+        case NetworkInformation::Reachability::Site:
+            [[fallthrough]];
+        case NetworkInformation::Reachability::Unknown:
+            // the connection might not yet be established
+            QTimer::singleShot(0, this, [this] { checkConnectivity(false); });
+            break;
+        case NetworkInformation::Reachability::Disconnected:
+            // explicitly set disconnected, this way a successful checkConnectivity call above will trigger a local discover
+            if (state() != State::SignedOut) {
+                setState(State::Disconnected);
+            }
+            [[fallthrough]];
+        case NetworkInformation::Reachability::Local:
+            break;
+        }
+    });
+
+    connect(NetworkInformation::instance(), &NetworkInformation::isMeteredChanged, this, [this](bool isMetered) {
+        if (ConfigFile().pauseSyncWhenMetered()) {
+            if (state() == State::Connected && isMetered) {
+                qCInfo(lcAccountState) << "Network switched to a metered connection, setting account state to PausedDueToMetered";
+                setState(State::Connecting);
+            } else if (state() == State::Connecting && !isMetered) {
+                qCInfo(lcAccountState) << "Network switched to a NON-metered connection, setting account state to Connected";
+                setState(State::Connected);
+            }
+        }
+    });
+
+    connect(NetworkInformation::instance(), &NetworkInformation::isBehindCaptivePortalChanged, this, [this](bool onoff) {
+        if (onoff) {
+            // Block jobs from starting: they will fail because of the captive portal.
+            // Note: this includes the `Drives` jobs started periodically by the `SpacesManager`.
+            _queueGuard.block();
+        } else {
+            // Empty the jobs queue before unblocking it. The client might have been behind a captive
+            // portal for hours, so a whole bunch of jobs might have queued up. If we wouldn't
+            // clear the queue, unleashing all those jobs might look like a DoS attack. Most of them
+            // are also not very useful anymore (e.g. `Drives` jobs), and the important ones (PUT jobs)
+            // will be rescheduled by a directory scan.
+            _account->jobQueue()->clear();
+            _queueGuard.unblock();
+        }
+
+        // A direct connect is not possible, because then the state parameter of `isBehindCaptivePortalChanged`
+        // would become the `verifyServerState` argument to `checkConnectivity`.
+        // The call is also made for when we "go behind" a captive portal. That ensures that not
+        // only the status is set to `Connecting`, but also makes the UI show that syncing is paused.
+        QTimer::singleShot(0, this, [this] { checkConnectivity(false); });
+    });
+    if (NetworkInformation::instance()->isBehindCaptivePortal()) {
+        _queueGuard.block();
+    }
+
+    // as a fallback and to recover after server issues we also poll
+    auto timer = new QTimer(this);
+    timer->setInterval(ConnectionValidator::DefaultCallingInterval);
+    connect(timer, &QTimer::timeout, this, [this] { checkConnectivity(false); });
+    timer->start();
+
     connect(account->credentials(), &AbstractCredentials::requestLogout, this, [this] {
-        _state = State::SignedOut;
+        setState(State::SignedOut);
     });
 
     if (FolderMan::instance()) {
         FolderMan::instance()->socketApi()->registerAccount(account);
     }
+
+    connect(account.data(), &Account::appProviderErrorOccured, this, [](const QString &error) {
+        QMessageBox *msgBox = new QMessageBox(QMessageBox::Information, Theme::instance()->appNameGUI(), error, {}, ocApp()->gui()->settingsDialog());
+        msgBox->setAttribute(Qt::WA_DeleteOnClose);
+        ownCloudGui::raise();
+        msgBox->open();
+    });
 }
 
-AccountState::~AccountState()
-{
-    if (FolderMan::instance()) {
-        FolderMan::instance()->socketApi()->unregisterAccount(account());
-    }
-}
+AccountState::~AccountState() { }
 
-AccountStatePtr AccountState::loadFromSettings(AccountPtr account, const QSettings &settings)
+std::unique_ptr<AccountState> AccountState::loadFromSettings(AccountPtr account, const QSettings &settings)
 {
-    auto accountState = AccountStatePtr(new AccountState(account));
+    auto accountState = std::unique_ptr<AccountState>(new AccountState(account));
     const bool userExplicitlySignedOut = settings.value(userExplicitlySignedOutC(), false).toBool();
     if (userExplicitlySignedOut) {
         // see writeToSettings below
-        accountState->_state = SignedOut;
+        accountState->setState(SignedOut);
     }
     accountState->_supportsSpaces = settings.value(supportsSpacesC(), false).toBool();
     return accountState;
 }
 
-AccountStatePtr AccountState::fromNewAccount(AccountPtr account)
+std::unique_ptr<AccountState> AccountState::fromNewAccount(AccountPtr account)
 {
-    return AccountStatePtr(new AccountState(account));
+    return std::unique_ptr<AccountState>(new AccountState(account));
 }
 
 void AccountState::writeToSettings(QSettings &settings) const
@@ -173,8 +244,7 @@ void AccountState::setState(State state)
 {
     const State oldState = _state;
     if (_state != state) {
-        qCInfo(lcAccountState) << "AccountState state change: "
-                               << stateString(_state) << "->" << stateString(state);
+        qCInfo(lcAccountState) << "AccountState state change: " << _state << "->" << state;
         _state = state;
 
         if (_state == SignedOut) {
@@ -188,49 +258,38 @@ void AccountState::setState(State state)
             // Check if we are actually down for maintenance.
             // To do this we must clear the connection validator that just
             // produced the 503. It's finished anyway and will delete itself.
+            _connectionValidator->deleteLater();
             _connectionValidator.clear();
             checkConnectivity();
-        }
-        if (oldState == Connected || _state == Connected) {
-            emit isConnectedChanged();
+        } else if (_state == Connected) {
+            if ((NetworkInformation::instance()->isMetered() && ConfigFile().pauseSyncWhenMetered())
+                || NetworkInformation::instance()->isBehindCaptivePortal()) {
+                _state = Connecting;
+            }
         }
     }
 
     // might not have changed but the underlying _connectionErrors might have
     if (_state == Connected) {
-        QTimer::singleShot(0, this, [this] {
+        QTimer::singleShot(0, this, [this, oldState] {
             // ensure the connection validator is done
             _queueGuard.unblock();
+            // update capabilites and fetch relevant settings
+            _fetchCapabilitiesJob = new FetchServerSettingsJob(account(), this);
+            connect(_fetchCapabilitiesJob.get(), &FetchServerSettingsJob::finishedSignal, this, [oldState, this] {
+                if (oldState == Connected || _state == Connected) {
+                    _fetchCapabilitiesJob.clear();
+                    Q_EMIT isConnectedChanged();
+                    Q_EMIT supportsSpacesChanged();
+                }
+            });
+            _fetchCapabilitiesJob->start();
         });
     }
-    // don't anounce a state change from connected to connected
-    // https://github.com/owncloud/client/commit/2c6c21d7532f0cbba4b768fde47810f6673ed931
-    if (oldState != state || state != Connected) {
-        emit stateChanged(_state);
-    }
-}
 
-QString AccountState::stateString(State state)
-{
-    switch (state) {
-    case SignedOut:
-        return tr("Signed out");
-    case Disconnected:
-        return tr("Disconnected");
-    case Connected:
-        return tr("Connected");
-    case ServiceUnavailable:
-        return tr("Service unavailable");
-    case MaintenanceMode:
-        return tr("Maintenance mode");
-    case NetworkError:
-        return tr("Network error");
-    case ConfigurationError:
-        return tr("Configuration error");
-    case AskingCredentials:
-        return tr("Asking Credentials");
+    if (oldState != _state) {
+        Q_EMIT stateChanged(_state);
     }
-    return tr("Unknown account state");
 }
 
 bool AccountState::isSignedOut() const
@@ -243,6 +302,8 @@ void AccountState::signOutByUi()
     account()->credentials()->forgetSensitiveData();
     account()->clearCookieJar();
     setState(SignedOut);
+    // persist that we are signed out
+    Q_EMIT account()->wantsAccountSaved(account().data());
 }
 
 void AccountState::freshConnectionAttempt()
@@ -257,6 +318,8 @@ void AccountState::signIn()
     if (_state == SignedOut) {
         _waitingForNewCredentials = false;
         setState(Disconnected);
+        // persist that we are no longer signed out
+        Q_EMIT account()->wantsAccountSaved(account().data());
     }
 }
 
@@ -272,22 +335,27 @@ void AccountState::tagLastSuccessfullETagRequest(const QDateTime &tp)
 
 void AccountState::checkConnectivity(bool blockJobs)
 {
-    qCWarning(lcAccountState) << "checkConnectivity blocking:" << blockJobs;
     if (isSignedOut() || _waitingForNewCredentials) {
         return;
     }
+    qCInfo(lcAccountState) << "checkConnectivity blocking:" << blockJobs << account()->displayNameWithHost();
+    if (_state != Connected) {
+        setState(Connecting);
+    }
     if (_tlsDialog) {
         qCDebug(lcAccountState) << "Skip checkConnectivity, waiting for tls dialog";
         return;
     }
 
+
     if (_connectionValidator && blockJobs && !_queueGuard.queue()->isBlocked()) {
         // abort already running non blocking validator
         _connectionValidator->deleteLater();
         _connectionValidator.clear();
     }
     if (_connectionValidator) {
-        qCWarning(lcAccountState) << "ConnectionValidator already running, ignoring" << account()->displayName();
+        qCWarning(lcAccountState) << "ConnectionValidator already running, ignoring" << account()->displayNameWithHost()
+                                  << "Queue is blocked:" << _queueGuard.queue()->isBlocked();
         return;
     }
 
@@ -300,12 +368,14 @@ void AccountState::checkConnectivity(bool blockJobs)
     if (account()->hasCapabilities()) {
         // IF the account is connected the connection check can be skipped
         // if the last successful etag check job is not so long ago.
+        // TODO: https://github.com/owncloud/client/issues/10935
         const auto pta = account()->capabilities().remotePollInterval();
         const auto polltime = duration_cast<seconds>(ConfigFile().remotePollInterval(pta));
         const auto elapsed = _timeOfLastETagCheck.secsTo(QDateTime::currentDateTimeUtc());
         if (!blockJobs && isConnected() && _timeOfLastETagCheck.isValid()
             && elapsed <= polltime.count()) {
-            qCDebug(lcAccountState) << account()->displayName() << "The last ETag check succeeded within the last " << polltime.count() << "s (" << elapsed << "s). No connection check needed!";
+            qCDebug(lcAccountState) << account()->displayNameWithHost() << "The last ETag check succeeded within the last " << polltime.count() << "s ("
+                                    << elapsed << "s). No connection check needed!";
             return;
         }
     }
@@ -318,6 +388,9 @@ void AccountState::checkConnectivity(bool blockJobs)
         this, &AccountState::slotConnectionValidatorResult);
 
     connect(_connectionValidator, &ConnectionValidator::sslErrors, this, [blockJobs, this](const QList<QSslError> &errors) {
+        if (NetworkInformation::instance()->isBehindCaptivePortal()) {
+            return;
+        }
         if (!_tlsDialog) {
             // ignore errors for already accepted certificates
             auto filteredErrors = _account->accessManager()->filterSslErrors(errors);
@@ -326,25 +399,27 @@ void AccountState::checkConnectivity(bool blockJobs)
                 _tlsDialog->setAttribute(Qt::WA_DeleteOnClose);
                 QSet<QSslCertificate> certs;
                 certs.reserve(filteredErrors.size());
-                for (const auto &error : qAsConst(filteredErrors)) {
+                for (const auto &error : std::as_const(filteredErrors)) {
                     certs << error.certificate();
                 }
                 connect(_tlsDialog, &TlsErrorDialog::accepted, _tlsDialog, [certs, blockJobs, this]() {
                     _account->addApprovedCerts(certs);
                     _tlsDialog.clear();
-                    _waitingForNewCredentials = false;
+                    // force a new _connectionValidator
+                    if (_connectionValidator) {
+                        _connectionValidator->deleteLater();
+                        _connectionValidator.clear();
+                    }
                     checkConnectivity(blockJobs);
                 });
                 connect(_tlsDialog, &TlsErrorDialog::rejected, this, [certs, this]() {
                     setState(SignedOut);
                 });
 
-                _tlsDialog->show();
+                ownCloudGui::raise();
+                _tlsDialog->open();
             }
         }
-        if (_tlsDialog) {
-            ocApp()->gui()->raiseDialog(_tlsDialog);
-        }
     });
     ConnectionValidator::ValidationMode mode = ConnectionValidator::ValidationMode::ValidateAuthAndUpdate;
     if (isConnected()) {
@@ -380,6 +455,7 @@ void AccountState::slotConnectionValidatorResult(ConnectionValidator::Status sta
         // this code should only be needed when upgrading from a < 3.0 release where capabilities where not cached
         // The last check was _waitingForNewCredentials = true so we only checked ValidateServer
         // now check again and fetch capabilities
+        _connectionValidator->deleteLater();
         _connectionValidator.clear();
         checkConnectivity();
         return;
@@ -410,14 +486,32 @@ void AccountState::slotConnectionValidatorResult(ConnectionValidator::Status sta
     }
     _connectionErrors = errors;
 
+    if (Q_UNLIKELY(Theme::instance()->enableCernBranding())) {
+        if (status == ConnectionValidator::Connected) {
+            Q_ASSERT(_account->hasCapabilities());
+            if (_account->capabilities().migration().space_migration.enabled) {
+                auto statePtr = AccountManager::instance()->accountState(_account->uuid());
+                auto migration = new SpaceMigration(statePtr, _account->capabilities().migration().space_migration.endpoint, this);
+                connect(migration, &SpaceMigration::finished, this, [migration, this] {
+                    migration->deleteLater();
+                    setState(Connected);
+                });
+                migration->start();
+                return;
+            }
+        }
+    }
     switch (status) {
     case ConnectionValidator::Connected:
         setState(Connected);
         break;
     case ConnectionValidator::Undefined:
+        [[fallthrough]];
     case ConnectionValidator::NotConfigured:
         setState(Disconnected);
         break;
+    case ConnectionValidator::ClientUnsupported:
+        [[fallthrough]];
     case ConnectionValidator::ServerVersionMismatch:
         setState(ConfigurationError);
         break;
@@ -428,6 +522,7 @@ void AccountState::slotConnectionValidatorResult(ConnectionValidator::Status sta
         setState(NetworkError);
         break;
     case ConnectionValidator::CredentialsWrong:
+        [[fallthrough]];
     case ConnectionValidator::CredentialsNotReady:
         slotInvalidCredentials();
         break;
@@ -445,6 +540,9 @@ void AccountState::slotConnectionValidatorResult(ConnectionValidator::Status sta
     case ConnectionValidator::Timeout:
         setState(NetworkError);
         break;
+    case ConnectionValidator::CaptivePortal:
+        setState(Connecting);
+        break;
     }
 }
 
@@ -470,7 +568,7 @@ void AccountState::slotInvalidCredentials()
     }
 }
 
-void AccountState::slotCredentialsFetched(AbstractCredentials *)
+void AccountState::slotCredentialsFetched()
 {
     // Make a connection attempt, no matter whether the credentials are
     // ready or not - we want to check whether we can get an SSL connection
@@ -481,14 +579,13 @@ void AccountState::slotCredentialsFetched(AbstractCredentials *)
     checkConnectivity();
 }
 
-void AccountState::slotCredentialsAsked(AbstractCredentials *credentials)
+void AccountState::slotCredentialsAsked()
 {
-    qCInfo(lcAccountState) << "Credentials asked for" << _account->url().toString()
-                           << "are they ready?" << credentials->ready();
+    qCInfo(lcAccountState) << "Credentials asked for" << _account->url().toString() << "are they ready?" << _account->credentials()->ready();
 
     _waitingForNewCredentials = false;
 
-    if (!credentials->ready()) {
+    if (!_account->credentials()->ready()) {
         // User canceled the connection or did not give a password
         setState(SignedOut);
         return;
@@ -498,12 +595,17 @@ void AccountState::slotCredentialsAsked(AbstractCredentials *credentials)
         // When new credentials become available we always want to restart the
         // connection validation, even if it's currently running.
         _connectionValidator->deleteLater();
-        _connectionValidator = nullptr;
+        _connectionValidator.clear();
     }
 
     checkConnectivity();
 }
 
+Account *AccountState::accountForQml() const
+{
+    return _account.data();
+}
+
 std::unique_ptr<QSettings> AccountState::settings()
 {
     auto s = ConfigFile::settingsWithGroup(QStringLiteral("Accounts"));
@@ -516,4 +618,31 @@ bool AccountState::supportsSpaces() const
     return _supportsSpaces && _account->hasCapabilities() && _account->capabilities().spacesSupport().enabled;
 }
 
+QuotaInfo *AccountState::quotaInfo()
+{
+    // QuotaInfo should not be used with spaces
+    Q_ASSERT(!supportsSpaces());
+    if (!_quotaInfo) {
+        _quotaInfo = new QuotaInfo(this);
+    }
+    return _quotaInfo;
+}
+
+bool AccountState::isSettingUp() const
+{
+    return _settingUp;
+}
+
+void AccountState::setSettingUp(bool settingUp)
+{
+    if (_settingUp != settingUp) {
+        _settingUp = settingUp;
+        Q_EMIT isSettingUpChanged();
+    }
+}
+bool AccountState::readyForSync() const
+{
+    return !_fetchCapabilitiesJob && isConnected();
+}
+
 } // namespace OCC
