@@ -11,25 +11,21 @@
  * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
  * for more details.
  */
+#include "gui/connectionvalidator.h"
+#include "gui/clientproxy.h"
+#include "gui/fetchserversettings.h"
+#include "gui/networkinformation.h"
+#include "gui/tlserrordialog.h"
+#include "libsync/account.h"
+#include "libsync/creds/abstractcredentials.h"
+#include "libsync/networkjobs.h"
+#include "libsync/networkjobs/checkserverjobfactory.h"
+#include "libsync/theme.h"
 
-#include <QJsonDocument>
 #include <QJsonObject>
 #include <QLoggingCategory>
-#include <QNetworkReply>
 #include <QNetworkProxyFactory>
-#include <QXmlStreamReader>
-
-#include "account.h"
-#include "clientproxy.h"
-#include "connectionvalidator.h"
-#include "cookiejar.h"
-#include "creds/abstractcredentials.h"
-#include "networkjobs.h"
-#include "networkjobs/checkserverjobfactory.h"
-#include "networkjobs/jobgroup.h"
-#include "networkjobs/jsonjob.h"
-#include "theme.h"
-#include "tlserrordialog.h"
+#include <QNetworkReply>
 
 using namespace std::chrono_literals;
 
@@ -40,13 +36,22 @@ Q_LOGGING_CATEGORY(lcConnectionValidator, "sync.connectionvalidator", QtInfoMsg)
 // Make sure the timeout for this job is less than how often we get called
 // This makes sure we get tried often enough without "ConnectionValidator already running"
 namespace {
-    const auto timeoutToUse = ConnectionValidator::DefaultCallingInterval - 5s;
+    auto timeoutToUse()
+    {
+        return std::min(ConnectionValidator::DefaultCallingInterval - 5s, AbstractNetworkJob::httpTimeout);
+    };
 }
 
 ConnectionValidator::ConnectionValidator(AccountPtr account, QObject *parent)
     : QObject(parent)
     , _account(account)
 {
+    // TODO: 6.0 abort validator on 5min timeout
+    auto timer = new QTimer(this);
+    timer->setInterval(30s);
+    connect(timer, &QTimer::timeout, this,
+        [this] { qCInfo(lcConnectionValidator) << "ConnectionValidator" << _account->displayNameWithHost() << "still running after" << _duration.duration(); });
+    timer->start();
 }
 
 void ConnectionValidator::setClearCookies(bool clearCookies)
@@ -85,7 +90,7 @@ void ConnectionValidator::systemProxyLookupDone(const QNetworkProxy &proxy)
     }
 
     if (proxy.type() != QNetworkProxy::NoProxy) {
-        qCInfo(lcConnectionValidator) << "Setting QNAM proxy to be system proxy" << printQNetworkProxy(proxy);
+        qCInfo(lcConnectionValidator) << "Setting QNAM proxy to be system proxy" << ClientProxy::printQNetworkProxy(proxy);
     } else {
         qCInfo(lcConnectionValidator) << "No system proxy set by OS";
     }
@@ -123,11 +128,14 @@ void ConnectionValidator::slotCheckServerAndAuth()
                 reportResult(Timeout);
                 return;
             case QNetworkReply::SslHandshakeFailedError:
-                reportResult(SslError);
+                reportResult(NetworkInformation::instance()->isBehindCaptivePortal() ? CaptivePortal : SslError);
                 return;
             case QNetworkReply::TooManyRedirectsError:
                 reportResult(MaintenanceMode);
                 return;
+            case QNetworkReply::ContentAccessDenied:
+                reportResult(ClientUnsupported);
+                return;
             default:
                 break;
             }
@@ -151,7 +159,7 @@ void ConnectionValidator::slotStatusFound(const QUrl &url, const QJsonObject &in
                                   << url << " with version "
                                   << info.value(QLatin1String("versionstring")).toString();
 
-    // Update server url in case of redirection
+    // Update server URL in case of redirection
     if (_account->url() != url) {
         qCInfo(lcConnectionValidator()) << "status.php was redirected to" << url.toString() << "asking user to accept and abort for now";
         Q_EMIT _account->requestUrlUpdate(url);
@@ -181,41 +189,40 @@ void ConnectionValidator::slotStatusFound(const QUrl &url, const QJsonObject &in
 
 void ConnectionValidator::checkAuthentication()
 {
-    // simply GET the webdav root, will fail if credentials are wrong.
+    // simply GET the WebDAV root, will fail if credentials are wrong.
     // continue in slotAuthCheck here :-)
     qCDebug(lcConnectionValidator) << "# Check whether authenticated propfind works.";
 
     // we explicitly use a legacy dav path here
     auto *job = new PropfindJob(_account, _account->url(), Theme::instance()->webDavPath(), PropfindJob::Depth::Zero, this);
     job->setAuthenticationJob(true); // don't retry
-    job->setTimeout(timeoutToUse);
+    job->setTimeout(timeoutToUse());
     job->setProperties({ QByteArrayLiteral("getlastmodified") });
     connect(job, &PropfindJob::finishedWithoutError, this, &ConnectionValidator::slotAuthSuccess);
     connect(job, &PropfindJob::finishedWithError, this, &ConnectionValidator::slotAuthFailed);
     job->start();
 }
 
-void ConnectionValidator::slotAuthFailed(QNetworkReply *reply)
+void ConnectionValidator::slotAuthFailed()
 {
     auto job = qobject_cast<PropfindJob *>(sender());
     Status stat = Timeout;
 
-    if (reply->error() == QNetworkReply::SslHandshakeFailedError) {
+    if (job->reply()->error() == QNetworkReply::SslHandshakeFailedError) {
         _errors << job->errorStringParsingBody();
-        stat = SslError;
+        stat = NetworkInformation::instance()->isBehindCaptivePortal() ? CaptivePortal : SslError;
 
-    } else if (reply->error() == QNetworkReply::AuthenticationRequiredError
-        || !_account->credentials()->stillValid(reply)) {
-        qCWarning(lcConnectionValidator) << "******** Password is wrong!" << reply->error() << job;
+    } else if (job->reply()->error() == QNetworkReply::AuthenticationRequiredError || !_account->credentials()->stillValid(job->reply())) {
+        qCWarning(lcConnectionValidator) << "******** Password is wrong!" << job->reply()->error() << job;
         _errors << tr("The provided credentials are not correct");
         stat = CredentialsWrong;
-
-    } else if (reply->error() != QNetworkReply::NoError) {
+    } else if (job->reply()->error() == QNetworkReply::ContentAccessDenied) {
+        stat = ClientUnsupported;
+        _errors << extractErrorMessage(job->reply()->readAll());
+    } else if (job->reply()->error() != QNetworkReply::NoError) {
         _errors << job->errorStringParsingBody();
 
-        const int httpStatus =
-            reply->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt();
-        if (httpStatus == 503) {
+        if (job->httpStatusCode() == 503) {
             _errors.clear();
             stat = ServiceUnavailable;
         }
@@ -228,120 +235,48 @@ void ConnectionValidator::slotAuthSuccess()
 {
     _errors.clear();
     if (_mode != ConnectionValidator::ValidationMode::ValidateAuth) {
-        checkServerCapabilities();
+        auto *fetchSetting = new FetchServerSettingsJob(_account, this);
+        const auto unsupportedServerError = [this] {
+            _errors.append({tr("The configured server for this client is too old."), tr("Please update to the latest server and restart the client.")});
+        };
+        connect(fetchSetting, &FetchServerSettingsJob::finishedSignal, this, [unsupportedServerError, this](FetchServerSettingsJob::Result result) {
+            switch (result) {
+            case FetchServerSettingsJob::Result::UnsupportedServer:
+                unsupportedServerError();
+                reportResult(ServerVersionMismatch);
+                break;
+            case FetchServerSettingsJob::Result::InvalidCredentials:
+                reportResult(CredentialsWrong);
+                break;
+            case FetchServerSettingsJob::Result::TimeOut:
+                reportResult(Timeout);
+                break;
+            case FetchServerSettingsJob::Result::Success:
+                if (_account->serverSupportLevel() == Account::ServerSupportLevel::Unknown) {
+                    unsupportedServerError();
+                }
+                reportResult(Connected);
+                break;
+            case FetchServerSettingsJob::Result::Undefined:
+                reportResult(Undefined);
+                break;
+            }
+        });
+
+        fetchSetting->start();
         return;
     }
     reportResult(Connected);
 }
 
-void ConnectionValidator::checkServerCapabilities()
-{
-    // The main flow now needs the capabilities
-    auto *job = new JsonApiJob(_account, QStringLiteral("ocs/v2.php/cloud/capabilities"), {}, {}, this);
-    job->setAuthenticationJob(true);
-    job->setTimeout(timeoutToUse);
-
-    QObject::connect(job, &JsonApiJob::finishedSignal, this, [job, this] {
-        auto caps = job->data().value(QStringLiteral("ocs")).toObject().value(QStringLiteral("data")).toObject().value(QStringLiteral("capabilities")).toObject();
-        qCInfo(lcConnectionValidator) << "Server capabilities" << caps;
-        _account->setCapabilities(caps.toVariantMap());
-        if (!checkServerInfo()) {
-            return;
-        }
-
-        fetchUser();
-    });
-    job->start();
-}
-
-void ConnectionValidator::fetchUser()
-{
-    auto *job = new JsonApiJob(_account, QStringLiteral("ocs/v2.php/cloud/user"), {}, {}, this);
-    job->setTimeout(timeoutToUse);
-    job->setAuthenticationJob(true);
-    QObject::connect(job, &JsonApiJob::finishedSignal, this, [job, this] {
-        const QString user = job->data().value(QStringLiteral("ocs")).toObject().value(QStringLiteral("data")).toObject().value(QStringLiteral("id")).toString();
-        if (!user.isEmpty()) {
-            _account->setDavUser(user);
-        }
-        const QString displayName = job->data().value(QStringLiteral("ocs")).toObject().value(QStringLiteral("data")).toObject().value(QStringLiteral("display-name")).toString();
-        if (!displayName.isEmpty()) {
-            _account->setDavDisplayName(displayName);
-        }
-
-        auto capabilities = _account->capabilities();
-        // We should have received the capabilities by now. Check that assumption in a debug build. If
-        // it's not the case, the code below will assume that they are not available.
-        Q_ASSERT(capabilities.isValid());
-
-        auto *group = new JobGroup(this);
-        group->setJobHook([](auto *job) {
-            job->setAuthenticationJob(true);
-            job->setTimeout(20s);
-        });
-        if (capabilities.isValid()) {
-            if (capabilities.avatarsAvailable()) {
-                auto *avatarJob = group->createJob<AvatarJob>(_account, _account->davUser(), 128, this);
-                connect(avatarJob, &AvatarJob::avatarPixmap, this, [this](const QPixmap &img) {
-                    _account->setAvatar(img);
-                });
-                avatarJob->start();
-            }
-            if (capabilities.appProviders().enabled) {
-                auto *jsonJob = group->createJob<JsonJob>(_account, _account->url(), capabilities.appProviders().appsUrl, "GET");
-                connect(jsonJob, &JsonApiJob::finishedSignal, this, [jsonJob, this] {
-                    _account->setAppProvider(AppProvider { jsonJob->data() });
-                });
-                jsonJob->start();
-            }
-        }
-        connect(group, &JobGroup::finishedSignal, this, [group, this] {
-            group->deleteLater();
-            reportResult(Connected);
-        });
-
-        if (group->isEmpty()) {
-            Q_EMIT group->finishedSignal();
-        }
-    });
-    job->start();
-}
-
-bool ConnectionValidator::checkServerInfo()
-{
-    // We cannot deal with servers < 10.0.0
-    switch (_account->serverSupportLevel()) {
-    case Account::ServerSupportLevel::Supported:
-        break;
-    case Account::ServerSupportLevel::Unknown:
-        [[fallthrough]];
-    case Account::ServerSupportLevel::Unsupported:
-        _errors.append(tr("The configured server for this client is too old."));
-        _errors.append(tr("Please update to the latest server and restart the client."));
-        if (_account->serverSupportLevel() == Account::ServerSupportLevel::Unsupported) {
-            reportResult(ServerVersionMismatch);
-            return false;
-        }
-    }
-    // We attempt to work with servers >= 7.0.0 but warn users.
-    // Check usages of Account::serverVersionUnsupported() for details.
-
-    // Record that the server supports HTTP/2
-    // Actual decision if we should use HTTP/2 is done in AccessManager::createRequest
-    if (auto job = qobject_cast<AbstractNetworkJob *>(sender())) {
-        if (auto reply = job->reply()) {
-            _account->setHttp2Supported(
-                reply->attribute(QNetworkRequest::Http2WasUsedAttribute).toBool());
-        }
-    }
-    return true;
-}
-
 void ConnectionValidator::reportResult(Status status)
 {
-    qCDebug(lcConnectionValidator) << status;
-    emit connectionResult(status, _errors);
-    deleteLater();
+    if (OC_ENSURE(!_finished)) {
+        _finished = true;
+        qCDebug(lcConnectionValidator) << status << _duration.duration();
+        Q_EMIT connectionResult(status, _errors);
+        deleteLater();
+    }
 }
 
 } // namespace OCC
