@@ -30,6 +30,7 @@
 #include "filesystem.h"
 #include "folderman.h"
 #include "folderwatcher.h"
+#include "gui/accountsettings.h"
 #include "libsync/graphapi/spacesmanager.h"
 #include "localdiscoverytracker.h"
 #include "scheduling/syncscheduler.h"
@@ -49,13 +50,16 @@
 #include <QDir>
 #include <QSettings>
 
-#include <QMessageBox>
-#include <QPushButton>
 #include <QApplication>
+#include <QMessageBox>
 
 using namespace std::chrono_literals;
 
 namespace {
+/* How often to retry a sync
+ * Either due to _engine->isAnotherSyncNeeded or a sync error
+ */
+constexpr int retrySyncLimitC = 3;
 
 /*
  * [Accounts]
@@ -99,15 +103,10 @@ using namespace FileSystem::SizeLiterals;
 
 Q_LOGGING_CATEGORY(lcFolder, "gui.folder", QtInfoMsg)
 
-Folder::Folder(const FolderDefinition &definition,
-    const AccountStatePtr &accountState, std::unique_ptr<Vfs> &&vfs,
-    QObject *parent)
+Folder::Folder(const FolderDefinition &definition, const AccountStatePtr &accountState, std::unique_ptr<Vfs> &&vfs, QObject *parent)
     : QObject(parent)
     , _accountState(accountState)
     , _definition(definition)
-    , _lastSyncDuration(0)
-    , _consecutiveFailingSyncs(0)
-    , _consecutiveFollowUpSyncs(0)
     , _journal(_definition.absoluteJournalPath())
     , _fileLog(new SyncRunFileLog)
     , _vfs(vfs.release())
@@ -138,15 +137,9 @@ Folder::Folder(const FolderDefinition &definition,
         connect(_engine.data(), &SyncEngine::started, this, &Folder::slotSyncStarted, Qt::QueuedConnection);
         connect(_engine.data(), &SyncEngine::finished, this, &Folder::slotSyncFinished, Qt::QueuedConnection);
 
-        connect(_engine.data(), &SyncEngine::aboutToRemoveAllFiles,
-            this, &Folder::slotAboutToRemoveAllFiles);
-        connect(_engine.data(), &SyncEngine::transmissionProgress, this, [this](const ProgressInfo &pi) {
-            emit ProgressDispatcher::instance()->progressInfo(this, pi);
-        });
-        connect(_engine.data(), &SyncEngine::itemCompleted,
-            this, &Folder::slotItemCompleted);
-        connect(_engine.data(), &SyncEngine::newBigFolder,
-            this, &Folder::slotNewBigFolderDiscovered);
+        connect(_engine.data(), &SyncEngine::transmissionProgress, this,
+            [this](const ProgressInfo &pi) { Q_EMIT ProgressDispatcher::instance()->progressInfo(this, pi); });
+        connect(_engine.data(), &SyncEngine::itemCompleted, this, &Folder::slotItemCompleted);
         connect(_engine.data(), &SyncEngine::seenLockedFile, FolderMan::instance(), &FolderMan::slotSyncOnceFileUnlocks);
         connect(_engine.data(), &SyncEngine::aboutToPropagate,
             this, &Folder::slotLogPropagationStart);
@@ -162,6 +155,12 @@ Folder::Folder(const FolderDefinition &definition,
         connect(_engine.data(), &SyncEngine::itemCompleted,
             _localDiscoveryTracker.data(), &LocalDiscoveryTracker::slotItemCompleted);
 
+        connect(_accountState->account()->spacesManager(), &GraphApi::SpacesManager::spaceChanged, this, [this](GraphApi::Space *changedSpace) {
+            if (_definition.spaceId() == changedSpace->id()) {
+                Q_EMIT spaceChanged();
+            }
+        });
+
         // Potentially upgrade suffix vfs to windows vfs
         OC_ENFORCE(_vfs);
         // Initialize the vfs plugin. Do this after the UI is running, so we can show a dialog when something goes wrong.
@@ -187,10 +186,20 @@ Result<void, QString> Folder::checkPathLength(const QString &path)
             return tr("The path '%1' is too long. Please enable long paths in the Windows settings or choose a different folder.").arg(path);
         }
     }
+#else
+    Q_UNUSED(path)
 #endif
     return {};
 }
 
+GraphApi::Space *Folder::space() const
+{
+    if (_accountState->supportsSpaces() && _accountState->account()->spacesManager()) {
+        return _accountState->account()->spacesManager()->space(_definition.spaceId());
+    }
+    return nullptr;
+}
+
 bool Folder::checkLocalPath()
 {
 #ifdef Q_OS_WIN
@@ -248,9 +257,6 @@ SyncOptions Folder::loadSyncOptions()
     SyncOptions opt(_vfs);
     ConfigFile cfgFile;
 
-    auto newFolderLimit = cfgFile.newBigFolderSizeLimit();
-    opt._newBigFolderSizeLimit = newFolderLimit.first ? newFolderLimit.second * 1000LL * 1000LL : -1; // convert from MB to B
-    opt._confirmExternalStorage = cfgFile.confirmExternalStorage();
     opt._moveFilesToTrash = cfgFile.moveToTrash();
     opt._vfs = _vfs;
     opt._parallelNetworkJobs = _accountState->account()->isHttp2Supported() ? 20 : 6;
@@ -297,6 +303,8 @@ void Folder::prepareFolder(const QString &path)
         const auto error = GetLastError();
         qCWarning(lcFolder) << "SetFileAttributesW failed on" << longDesktopIniPath << Utility::formatWinError(error);
     }
+#else
+    Q_UNUSED(path)
 #endif
 }
 
@@ -307,6 +315,9 @@ QByteArray Folder::id() const
 
 QString Folder::displayName() const
 {
+    if (auto *s = space()) {
+        return s->displayName();
+    }
     return _definition.displayName();
 }
 
@@ -396,7 +407,7 @@ bool Folder::syncPaused() const
 
 bool Folder::canSync() const
 {
-    return !syncPaused() && accountState()->readyForSync() && isReady() && _accountState->account()->hasCapabilities() && _folderWatcher;
+    return _engine && !syncPaused() && accountState()->readyForSync() && isReady() && _accountState->account()->hasCapabilities() && _folderWatcher;
 }
 
 bool Folder::isReady() const
@@ -416,13 +427,13 @@ void Folder::setSyncPaused(bool paused)
     _definition.paused = paused;
     saveToSettings();
 
-    emit syncPausedChanged(this, paused);
+    Q_EMIT syncPausedChanged(this, paused);
     if (!paused) {
         setSyncState(SyncResult::NotYetStarted);
     } else {
         setSyncState(SyncResult::Paused);
     }
-    emit canSyncChanged();
+    Q_EMIT canSyncChanged();
 }
 
 void Folder::setSyncState(SyncResult::Status state)
@@ -522,7 +533,7 @@ void Folder::createGuiLog(const QString &filename, LogStatus status, int count,
             break;
         case LogStatusConflict:
             if (count > 1) {
-                text = tr("%1 has and %n other file(s) have sync conflicts.", "", count - 1).arg(file);
+                text = tr("%1 and %n other file(s) have sync conflicts.", "", count - 1).arg(file);
             } else {
                 text = tr("%1 has a sync conflict. Please check the conflict file!").arg(file);
             }
@@ -597,10 +608,11 @@ void Folder::startVfs()
         _vfsIsReady = false;
     });
 
+    slotNextSyncFullLocalDiscovery();
     _vfs->start(vfsParams);
 }
 
-int Folder::slotDiscardDownloadProgress()
+void Folder::slotDiscardDownloadProgress()
 {
     // Delete from journal and from filesystem.
     QDir folderpath(_definition.localPath());
@@ -612,17 +624,6 @@ int Folder::slotDiscardDownloadProgress()
         qCInfo(lcFolder) << "Deleting temporary file: " << tmppath;
         FileSystem::remove(tmppath);
     }
-    return deleted_infos.size();
-}
-
-int Folder::downloadInfoCount()
-{
-    return _journal.downloadInfoCount();
-}
-
-int Folder::errorBlackListEntryCount()
-{
-    return _journal.errorBlackListEntryCount();
 }
 
 int Folder::slotWipeErrorBlacklist()
@@ -632,7 +633,10 @@ int Folder::slotWipeErrorBlacklist()
 
 void Folder::slotWatchedPathsChanged(const QSet<QString> &paths, ChangeReason reason)
 {
-    Q_ASSERT(isReady());
+    if (!isReady()) {
+        // we might be switching backend
+        return;
+    }
     bool needSync = false;
     for (const auto &path : paths) {
         Q_ASSERT(FileSystem::isChildPathOf(path, this->path()));
@@ -643,7 +647,7 @@ void Folder::slotWatchedPathsChanged(const QSet<QString> &paths, ChangeReason re
 
             {
                 // horrible hack to compensate that we don't handle folder deletes on a per file basis
-                int index = 0;
+                qsizetype index = 0;
                 QString p = relativePath;
                 while ((index = p.lastIndexOf(QLatin1Char('/'))) != -1) {
                     p = p.left(index);
@@ -681,33 +685,12 @@ void Folder::slotWatchedPathsChanged(const QSet<QString> &paths, ChangeReason re
             }
             if (spurious) {
                 qCInfo(lcFolder) << "Ignoring spurious notification for file" << relativePath;
-                Q_ASSERT([&] {
-                    Q_ASSERT(record.isValid());
-                    // we don't intend to burn to many cpu cycles so limit this check on small files
-                    if (!record.isVirtualFile() && record._fileSize < static_cast<qint64>(1_mb)) {
-                        const auto header = ChecksumHeader::parseChecksumHeader(record._checksumHeader);
-                        auto *compute = new ComputeChecksum(this);
-                        compute->setChecksumType(header.type());
-                        quint64 inode = 0;
-                        FileSystem::getInode(path, &inode);
-                        connect(compute, &ComputeChecksum::done, this, [=](CheckSums::Algorithm, const QByteArray &checksum) {
-                            compute->deleteLater();
-                            qWarning() << "Spurious notification:" << path << (checksum == header.checksum()) << checksum << header.checksum()
-                                       << "Inode:" << record._inode << inode;
-                            Q_ASSERT(inode == record._inode);
-                            Q_ASSERT(checksum == header.checksum());
-                        });
-                        compute->start(path);
-                    }
-                    return true;
-                }());
-
                 continue; // probably a spurious notification
             }
         }
         warnOnNewExcludedItem(record, relativePath);
 
-        emit watchedFileChangedExternally(path);
+        Q_EMIT watchedFileChangedExternally(path);
         needSync = true;
     }
     if (needSync && canSync()) {
@@ -756,34 +739,59 @@ void Folder::setVirtualFilesEnabled(bool enabled)
 
     if (newMode != _definition.virtualFilesMode) {
         // This is tested in TestSyncVirtualFiles::testWipeVirtualSuffixFiles, so for changes here, have them reflected in that test.
+        const bool isPaused = _definition.paused;
+        if (!isPaused) {
+            setSyncPaused(true);
+        }
+        auto finalizeVfsSwitch = [newMode, enabled, isPaused, this] {
+            // Wipe selective sync blacklist
+            bool ok = false;
+            const auto oldBlacklist = journalDb()->getSelectiveSyncList(SyncJournalDb::SelectiveSyncBlackList, &ok);
+            journalDb()->setSelectiveSyncList(SyncJournalDb::SelectiveSyncBlackList, {});
 
-        // TODO: Must wait for current sync to finish!
-        OC_ENFORCE(!isSyncRunning());
+            // Wipe the dehydrated files from the DB, they will get downloaded on the next sync. We need to do this, otherwise the files
+            // are in the DB but not on disk, so the client assumes they are deleted, and removes them from the remote.
+            _vfs->wipeDehydratedVirtualFiles();
 
-        // Wipe the dehydrated files from the DB, they will get downloaded on the next sync. We need to do this, otherwise the files
-        // are in the DB but not on disk, so the client assumes they are deleted, and removes them from the remote.
-        _vfs->wipeDehydratedVirtualFiles();
+            // Tear down the VFS
+            _vfsIsReady = false;
+            _vfs->stop();
+            _vfs->unregisterFolder();
 
-        // Tear down the VFS
-        _vfs->stop();
-        _vfs->unregisterFolder();
+            disconnect(_vfs.data(), nullptr, this, nullptr);
+            disconnect(&_engine->syncFileStatusTracker(), nullptr, _vfs.data(), nullptr);
 
-        disconnect(_vfs.data(), nullptr, this, nullptr);
-        disconnect(&_engine->syncFileStatusTracker(), nullptr, _vfs.data(), nullptr);
+            // _vfs is a shared pointer...
+            _vfs.reset(VfsPluginManager::instance().createVfsFromPlugin(newMode).release());
 
-        _vfsIsReady = false;
-        _vfs.reset(VfsPluginManager::instance().createVfsFromPlugin(newMode).release());
-
-        // Restart VFS.
-        _definition.virtualFilesMode = newMode;
-        startVfs();
-        saveToSettings();
+            // Restart VFS.
+            _definition.virtualFilesMode = newMode;
+            if (enabled) {
+                connect(_vfs.data(), &Vfs::started, this, [oldBlacklist, this] {
+                    for (const auto &entry : oldBlacklist) {
+                        journalDb()->schedulePathForRemoteDiscovery(entry);
+                        std::ignore = vfs().setPinState(entry, PinState::OnlineOnly);
+                    }
+                });
+            }
+            saveToSettings();
+            if (!isPaused) {
+                setSyncPaused(isPaused);
+            }
+            startVfs();
+        };
+        if (isSyncRunning()) {
+            connect(this, &Folder::syncFinished, this, finalizeVfsSwitch, Qt::SingleShotConnection);
+            slotTerminateSync(tr("Switching VFS mode on folder '%1'").arg(displayName()));
+        } else {
+            finalizeVfsSwitch();
+        }
     }
 }
 
 bool Folder::supportsSelectiveSync() const
 {
-    return !virtualFilesEnabled() && !isVfsOnOffSwitchPending();
+    return !virtualFilesEnabled() && isReady();
 }
 
 bool Folder::isDeployed() const
@@ -804,7 +812,7 @@ void Folder::saveToSettings() const
         }
     }
     // with spaces we rely on the space id
-    // we save the dav url nevertheless to have it available during startup
+    // we save the dav URL nevertheless to have it available during startup
     definitionToSave.setWebDavUrl(webDavUrl());
 
     // keep the scope of the interaction with the settings limited to prevent possible loss of settings
@@ -847,12 +855,12 @@ bool Folder::isFileExcludedRelative(const QString &relativePath) const
     return isFileExcludedAbsolute(path() + relativePath);
 }
 
-void Folder::slotTerminateSync()
+void Folder::slotTerminateSync(const QString &reason)
 {
     if (isReady()) {
         qCInfo(lcFolder) << "folder " << path() << " Terminating!";
         if (_engine->isSyncRunning()) {
-            _engine->abort();
+            _engine->abort(reason);
             setSyncState(SyncResult::SyncAbortRequested);
         }
     }
@@ -944,6 +952,10 @@ void Folder::startSync()
 
     setDirtyNetworkLimits();
 
+    // get the latest touched files
+    // this will enque this folder again, it doesn't matter
+    slotWatchedPathsChanged(_folderWatcher->popChangeSet(), Folder::ChangeReason::Other);
+
     const std::chrono::milliseconds fullLocalDiscoveryInterval = ConfigFile().fullLocalDiscoveryInterval();
     const bool hasDoneFullLocalDiscovery = _timeSinceLastFullLocalDiscovery.isValid();
     // negative fullLocalDiscoveryInterval means we don't require periodic full runs
@@ -964,16 +976,9 @@ void Folder::startSync()
     }
 
     _engine->setIgnoreHiddenFiles(_definition.ignoreHiddenFiles);
-    if (_allowRemoveAllOnce) {
-        _engine->setPromtRemoveAllFiles(false);
-        _allowRemoveAllOnce = false;
-    } else {
-        _engine->setPromtRemoveAllFiles(ConfigFile().promptDeleteFiles());
-    }
-
     QMetaObject::invokeMethod(_engine.data(), &SyncEngine::startSync, Qt::QueuedConnection);
 
-    emit syncStarted();
+    Q_EMIT syncStarted();
 }
 
 void Folder::setDirtyNetworkLimits()
@@ -999,10 +1004,15 @@ void Folder::setDirtyNetworkLimits()
     _engine->setNetworkLimits(uploadLimit, downloadLimit);
 }
 
+void Folder::reloadSyncOptions()
+{
+    _engine->setSyncOptions(loadSyncOptions());
+}
+
 void Folder::slotSyncError(const QString &message, ErrorCategory category)
 {
     _syncResult.appendErrorString(message);
-    emit ProgressDispatcher::instance()->syncError(this, message, category);
+    Q_EMIT ProgressDispatcher::instance()->syncError(this, message, category);
 }
 
 void Folder::slotSyncStarted()
@@ -1028,7 +1038,7 @@ void Folder::slotSyncFinished(bool success)
     _fileLog->finish();
     showSyncResultPopup();
 
-    auto anotherSyncNeeded = _engine->isAnotherSyncNeeded();
+    auto anotherSyncNeeded = false;
 
     auto syncStatus = SyncResult::Status::Undefined;
 
@@ -1048,6 +1058,7 @@ void Folder::slotSyncFinished(bool success)
         _consecutiveFailingSyncs = 0;
     } else {
         _consecutiveFailingSyncs++;
+        anotherSyncNeeded |= _consecutiveFailingSyncs <= retrySyncLimitC;
         qCInfo(lcFolder) << "the last" << _consecutiveFailingSyncs << "syncs failed";
     }
 
@@ -1073,8 +1084,9 @@ void Folder::slotSyncFinished(bool success)
     _timeSinceLastSyncDone.start();
 
     // Increment the follow-up sync counter if necessary.
-    if (anotherSyncNeeded == AnotherSyncNeeded::ImmediateFollowUp) {
+    if (_engine->isAnotherSyncNeeded()) {
         _consecutiveFollowUpSyncs++;
+        anotherSyncNeeded |= _consecutiveFollowUpSyncs <= retrySyncLimitC;
         qCInfo(lcFolder) << "another sync was requested by the finished sync, this has"
                          << "happened" << _consecutiveFollowUpSyncs << "times";
     } else {
@@ -1082,7 +1094,7 @@ void Folder::slotSyncFinished(bool success)
     }
 
     // Maybe force a follow-up sync to take place, but only a couple of times.
-    if (anotherSyncNeeded == AnotherSyncNeeded::ImmediateFollowUp && _consecutiveFollowUpSyncs <= 3) {
+    if (anotherSyncNeeded && canSync()) {
         // Sometimes another sync is requested because a local file is still
         // changing, so wait at least a small amount of time before syncing
         // the folder again.
@@ -1101,42 +1113,7 @@ void Folder::slotItemCompleted(const SyncFileItemPtr &item)
     _syncResult.processCompletedItem(item);
 
     _fileLog->logItem(*item);
-    emit ProgressDispatcher::instance()->itemCompleted(this, item);
-}
-
-void Folder::slotNewBigFolderDiscovered(const QString &newF, bool isExternal)
-{
-    auto newFolder = newF;
-    if (!newFolder.endsWith(QLatin1Char('/'))) {
-        newFolder += QLatin1Char('/');
-    }
-    auto journal = journalDb();
-
-    // Add the entry to the blacklist if it is neither in the blacklist or whitelist already
-    bool ok1, ok2;
-    auto blacklist = journal->getSelectiveSyncList(SyncJournalDb::SelectiveSyncBlackList, &ok1);
-    auto whitelist = journal->getSelectiveSyncList(SyncJournalDb::SelectiveSyncWhiteList, &ok2);
-    if (ok1 && ok2 && !blacklist.contains(newFolder) && !whitelist.contains(newFolder)) {
-        blacklist.insert(newFolder);
-        journal->setSelectiveSyncList(SyncJournalDb::SelectiveSyncBlackList, blacklist);
-    }
-
-    // And add the entry to the undecided list and signal the UI
-    auto undecidedList = journal->getSelectiveSyncList(SyncJournalDb::SelectiveSyncUndecidedList, &ok1);
-    if (ok1) {
-        if (!undecidedList.contains(newFolder)) {
-            undecidedList.insert(newFolder);
-            journal->setSelectiveSyncList(SyncJournalDb::SelectiveSyncUndecidedList, undecidedList);
-            emit newBigFolderDiscovered(newFolder);
-        }
-        QString message = !isExternal ? (tr("A new folder larger than %1 MB has been added: %2.\n")
-                                                .arg(ConfigFile().newBigFolderSizeLimit().second)
-                                                .arg(newF))
-                                      : (tr("A folder from an external storage has been added.\n"));
-        message += tr("Please go in the settings to select it if you wish to download it.");
-
-        ocApp()->gui()->slotShowOptionalTrayMessage(Theme::instance()->appNameGUI(), message);
-    }
+    Q_EMIT ProgressDispatcher::instance()->itemCompleted(this, item);
 }
 
 void Folder::slotLogPropagationStart()
@@ -1213,8 +1190,8 @@ void Folder::slotWatcherUnreliable(const QString &message)
         {}, ocApp()->gui()->settingsDialog());
 
     msgBox->setAttribute(Qt::WA_DeleteOnClose);
+    ownCloudGui::raise();
     msgBox->open();
-    ocApp()->gui()->raiseDialog(msgBox);
 }
 
 void Folder::registerFolderWatcher()
@@ -1236,57 +1213,7 @@ void Folder::registerFolderWatcher()
 
 bool Folder::virtualFilesEnabled() const
 {
-    return _definition.virtualFilesMode != Vfs::Off && !isVfsOnOffSwitchPending();
-}
-
-void Folder::slotAboutToRemoveAllFiles(SyncFileItem::Direction direction)
-{
-    if (_removeAllFilesDialog) {
-        ocApp()->gui()->raiseDialog(_removeAllFilesDialog);
-        return;
-    }
-    const QString msg = [direction] {
-        if (direction == SyncFileItem::Down) {
-            return tr("All files in the sync folder '%1' folder were deleted on the server.\n"
-                      "These deletes will be synchronized to your local sync folder, making such files "
-                      "unavailable unless you have a right to restore. \n"
-                      "If you decide to keep the files, they will be re-synced with the server if you have rights to do so.\n"
-                      "If you decide to delete the files, they will be unavailable to you, unless you are the owner.");
-        } else {
-            return tr("All the files in your local sync folder '%1' were deleted. These deletes will be "
-                      "synchronized with your server, making such files unavailable unless restored.\n"
-                      "Are you sure you want to sync those actions with the server?\n"
-                      "If this was an accident and you decide to keep your files, they will be re-synced from the server.");
-        }
-    }();
-    _removeAllFilesDialog =
-        new QMessageBox(QMessageBox::Warning, tr("Remove All Files?"), msg.arg(shortGuiLocalPath()), QMessageBox::NoButton, ocApp()->gui()->settingsDialog());
-    _removeAllFilesDialog->setAttribute(Qt::WA_DeleteOnClose);
-    _removeAllFilesDialog->setWindowFlags(_removeAllFilesDialog->windowFlags() | Qt::WindowStaysOnTopHint);
-    _removeAllFilesDialog->addButton(tr("Remove all files"), QMessageBox::DestructiveRole);
-    QPushButton *keepBtn = _removeAllFilesDialog->addButton(tr("Keep files"), QMessageBox::AcceptRole);
-    _removeAllFilesDialog->setDefaultButton(keepBtn);
-    setSyncPaused(true);
-    connect(_removeAllFilesDialog, &QMessageBox::finished, this, [keepBtn, this] {
-        if (_removeAllFilesDialog->clickedButton() == keepBtn) {
-            // reset the db upload all local files or download all remote files
-            FileSystem::setFolderMinimumPermissions(path());
-            // will remove all dehydrated placeholders
-            _vfs->wipeDehydratedVirtualFiles();
-            journalDb()->clearFileTable();
-        }
-        // if all local files where placeholders, they might be gone after the next sync
-        // therefor we need to allow removal off all files in the next sync even if we selected keep
-        _allowRemoveAllOnce = true;
-        // the only way we end up in here is that the folder was not paused
-        setSyncPaused(false);
-        if (canSync()) {
-            FolderMan::instance()->scheduler()->enqueueFolder(this);
-        }
-    });
-    connect(this, &Folder::destroyed, _removeAllFilesDialog, &QMessageBox::deleteLater);
-    _removeAllFilesDialog->open();
-    ownCloudGui::raiseDialog(_removeAllFilesDialog);
+    return _definition.virtualFilesMode != Vfs::Off;
 }
 
 FolderDefinition::FolderDefinition(const QByteArray &id, const QUrl &davUrl, const QString &spaceId, const QString &displayName)
@@ -1337,7 +1264,7 @@ FolderDefinition FolderDefinition::load(QSettings &settings, const QByteArray &i
     folder.paused = settings.value(QStringLiteral("paused")).toBool();
     folder.ignoreHiddenFiles = settings.value(QStringLiteral("ignoreHiddenFiles"), QVariant(true)).toBool();
     folder._deployed = settings.value(deployedC(), false).toBool();
-    folder._priority = settings.value(priorityC(), 0).toInt();
+    folder._priority = settings.value(priorityC(), 0).toUInt();
 
     folder.virtualFilesMode = Vfs::Off;
     QString vfsModeString = settings.value(QStringLiteral("virtualFilesMode")).toString();
@@ -1412,11 +1339,6 @@ bool Folder::groupInSidebar() const
     return false;
 }
 
-QString Folder::spaceId() const
-{
-    return _definition.spaceId();
-}
-
 bool FolderDefinition::isDeployed() const
 {
     return _deployed;
