@@ -15,9 +15,9 @@
 #ifndef MIRALL_FOLDERWATCHER_WIN_H
 #define MIRALL_FOLDERWATCHER_WIN_H
 
-#include <QThread>
 #include <QAtomicInt>
-#include <windows.h>
+#include <QThread>
+#include <qt_windows.h>
 
 namespace OCC {
 
@@ -49,7 +49,7 @@ protected:
     void processEntries(FILE_NOTIFY_INFORMATION *curEntry);
     void closeHandle();
 
-signals:
+Q_SIGNALS:
     void changed(const QSet<QString> &path);
     void lostChanges();
 
