@@ -23,23 +23,18 @@
 
 #include "account.h"
 #include "common/asserts.h"
-#include "configfile.h"
-#include "creds/abstractcredentials.h"
+#include "common/vfs.h"
 #include "gui/application.h"
-#include "gui/askexperimentalvirtualfilesfeaturemessagebox.h"
-#include "gui/guiutility.h"
 #include "gui/settingsdialog.h"
-#include "networkjobs.h"
 #include "theme.h"
 
 #include "gui/accountstate.h"
 #include "gui/folderman.h"
-#include "gui/selectivesyncdialog.h"
-#include "gui/spaces/spacesmodel.h"
+
+#include "libsync/graphapi/space.h"
 
 #include <QDesktopServices>
 #include <QDir>
-#include <QEvent>
 #include <QFileInfo>
 #include <QUrl>
 
@@ -69,7 +64,8 @@ QString FolderWizardPrivate::formatWarnings(const QStringList &warnings, bool is
 QString FolderWizardPrivate::defaultSyncRoot() const
 {
     if (!_account->account()->hasDefaultSyncRoot()) {
-        return FolderMan::suggestSyncFolder(_account->account()->url(), _account->account()->davDisplayName());
+        const auto folderType = _account->supportsSpaces() ? FolderMan::NewFolderType::SpacesSyncRoot : FolderMan::NewFolderType::OC10SyncRoot;
+        return FolderMan::suggestSyncFolder(folderType, _account->account()->uuid());
     } else {
         return _account->account()->defaultSyncRoot();
     }
@@ -79,23 +75,24 @@ FolderWizardPrivate::FolderWizardPrivate(FolderWizard *q, const AccountStatePtr
     : q_ptr(q)
     , _account(account)
     , _folderWizardSourcePage(new FolderWizardLocalPath(this))
-    , _folderWizardSelectiveSyncPage(new FolderWizardSelectiveSync(this))
+    , _folderWizardSelectiveSyncPage(nullptr)
 {
     if (account->supportsSpaces()) {
         _spacesPage = new SpacesPage(account->account(), q);
         q->setPage(FolderWizard::Page_Space, _spacesPage);
-        _spacesPage->installEventFilter(q);
     }
+
     q->setPage(FolderWizard::Page_Source, _folderWizardSourcePage);
-    _folderWizardSourcePage->installEventFilter(q);
-    // for now spaces are meant to be synced as a whole
+
     if (!_account->supportsSpaces() && !Theme::instance()->singleSyncFolder()) {
         _folderWizardTargetPage = new FolderWizardRemotePath(this);
         q->setPage(FolderWizard::Page_Target, _folderWizardTargetPage);
-        _folderWizardTargetPage->installEventFilter(q);
     }
-    if (!_account->supportsSpaces()) {
-        // TODO: add spaces support to selective sync
+
+    // When VFS is available (currently only with Windows' CFApi), and it is forced on, Spaces are meant to be synced as a whole.
+    const bool showPage = VfsPluginManager::instance().bestAvailableVfsMode() != Vfs::WindowsCfApi || !Theme::instance()->forceVirtualFilesOption();
+    if (showPage) {
+        _folderWizardSelectiveSyncPage = new FolderWizardSelectiveSync(this);
         q->setPage(FolderWizard::Page_SelectiveSync, _folderWizardSelectiveSyncPage);
     }
 }
@@ -103,10 +100,13 @@ FolderWizardPrivate::FolderWizardPrivate(FolderWizard *q, const AccountStatePtr
 QString FolderWizardPrivate::initialLocalPath() const
 {
     if (_account->supportsSpaces()) {
-        return FolderMan::instance()->findGoodPathForNewSyncFolder(
-            defaultSyncRoot(), _spacesPage->selectedSpace(Spaces::SpacesModel::Columns::Name).toString());
+        return FolderMan::findGoodPathForNewSyncFolder(
+            defaultSyncRoot(), _spacesPage->currentSpace()->displayName(), FolderMan::NewFolderType::SpacesSyncRoot, _account->account()->uuid());
     }
-    return defaultSyncRoot();
+
+    // Split default sync root:
+    const QFileInfo path(defaultSyncRoot());
+    return FolderMan::findGoodPathForNewSyncFolder(path.path(), path.fileName(), FolderMan::NewFolderType::OC10SyncRoot, _account->account()->uuid());
 }
 
 QString FolderWizardPrivate::remotePath() const
@@ -117,15 +117,15 @@ QString FolderWizardPrivate::remotePath() const
 uint32_t FolderWizardPrivate::priority() const
 {
     if (_account->supportsSpaces()) {
-        return _spacesPage->selectedSpace(Spaces::SpacesModel::Columns::Priority).toInt();
-    };
+        return _spacesPage->currentSpace()->priority();
+    }
     return 0;
 }
 
 QUrl FolderWizardPrivate::davUrl() const
 {
     if (_account->supportsSpaces()) {
-        auto url = _spacesPage->selectedSpace(Spaces::SpacesModel::Columns::WebDavUrl).toUrl();
+        auto url = _spacesPage->currentSpace()->webdavUrl();
         if (!url.path().endsWith(QLatin1Char('/'))) {
             url.setPath(url.path() + QLatin1Char('/'));
         }
@@ -134,11 +134,19 @@ QUrl FolderWizardPrivate::davUrl() const
     return _account->account()->davUrl();
 }
 
+QString FolderWizardPrivate::spaceId() const
+{
+    if (_account->supportsSpaces()) {
+        return _spacesPage->currentSpace()->id();
+    }
+    return {};
+}
+
 QString FolderWizardPrivate::displayName() const
 {
     if (_account->supportsSpaces()) {
-        return _spacesPage->selectedSpace(Spaces::SpacesModel::Columns::Name).toString();
-    };
+        return _spacesPage->currentSpace()->displayName();
+    }
     return QString();
 }
 
@@ -150,7 +158,7 @@ const AccountStatePtr &FolderWizardPrivate::accountState()
 bool FolderWizardPrivate::useVirtualFiles() const
 {
     const auto mode = VfsPluginManager::instance().bestAvailableVfsMode();
-    const bool useVirtualFiles = (Theme::instance()->forceVirtualFilesOption() && mode == Vfs::WindowsCfApi) || (_folderWizardSelectiveSyncPage->useVirtualFiles());
+    const bool useVirtualFiles = (Theme::instance()->forceVirtualFilesOption() && mode == Vfs::WindowsCfApi) || (_folderWizardSelectiveSyncPage && _folderWizardSelectiveSyncPage->useVirtualFiles());
     if (useVirtualFiles) {
         const auto availability = Vfs::checkAvailability(initialLocalPath(), mode);
         if (!availability) {
@@ -177,30 +185,7 @@ FolderWizard::~FolderWizard()
 {
 }
 
-bool FolderWizard::eventFilter(QObject *watched, QEvent *event)
-{
-    if (event->type() == QEvent::LayoutRequest) {
-        // Workaround QTBUG-3396:  forces QWizardPrivate::updateLayout()
-        QTimer::singleShot(0, this, [this] { setTitleFormat(titleFormat()); });
-    }
-    return QWizard::eventFilter(watched, event);
-}
-
-void FolderWizard::resizeEvent(QResizeEvent *event)
-{
-    QWizard::resizeEvent(event);
-
-    // workaround for QTBUG-22819: when the error label word wrap, the minimum height is not adjusted
-    if (auto page = currentPage()) {
-        int hfw = page->heightForWidth(page->width());
-        if (page->height() < hfw) {
-            page->setMinimumSize(page->minimumSizeHint().width(), hfw);
-            setTitleFormat(titleFormat()); // And another workaround for QTBUG-3396
-        }
-    }
-}
-
-FolderWizard::Result FolderWizard::result()
+FolderMan::SyncConnectionDescription FolderWizard::result()
 {
     Q_D(FolderWizard);
 
@@ -208,18 +193,24 @@ FolderWizard::Result FolderWizard::result()
     if (!d->_account->account()->hasDefaultSyncRoot()) {
         if (FileSystem::isChildPathOf(localPath, d->defaultSyncRoot())) {
             d->_account->account()->setDefaultSyncRoot(d->defaultSyncRoot());
+            if (!QFileInfo::exists(d->defaultSyncRoot())) {
+                OC_ASSERT(QDir().mkpath(d->defaultSyncRoot()));
+            }
         }
     }
 
+    // clang-format off
     return {
         d->davUrl(),
+        d->spaceId(),
         localPath,
         d->remotePath(),
         d->displayName(),
         d->useVirtualFiles(),
         d->priority(),
-        d->_folderWizardSelectiveSyncPage ? d->_folderWizardSelectiveSyncPage->selectiveSyncBlackList() : QStringList()
+        d->_folderWizardSelectiveSyncPage ? d->_folderWizardSelectiveSyncPage->selectiveSyncBlackList() : QSet<QString>{}
     };
+    // clang-format on
 }
 
 } // end namespace
