@@ -14,13 +14,11 @@
 
 #pragma once
 
-#include <QNetworkReply>
-#include <QTimer>
+#include <QUrl>
 #include <QWizard>
 
 #include "accountfwd.h"
-
-#include "gui/folder.h"
+#include "gui/folderman.h"
 
 class QCheckBox;
 class QTreeWidgetItem;
@@ -45,48 +43,14 @@ public:
         Page_Target,
         Page_SelectiveSync
     };
-    Q_ENUM(PageType);
-
-    struct Result
-    {
-        /***
-         * The webdav url for the sync connection.
-         */
-        QUrl davUrl;
-        /***
-         * The local folder used for the sync.
-         */
-        QString localPath;
-
-        /***
-         * The relative remote path
-         */
-        QString remotePath;
-
-        /***
-         * The Space name to display in the list of folders or an empty string.
-         */
-        QString displayName;
-
-        /***
-         * Wether to use virtual files.
-         */
-        bool useVirtualFiles;
-
-        uint32_t priority;
-
-        QStringList selectiveSyncBlackList;
-    };
+    Q_ENUM(PageType)
 
     explicit FolderWizard(const AccountStatePtr &account, QWidget *parent = nullptr);
     ~FolderWizard() override;
 
-    Result result();
+    FolderMan::SyncConnectionDescription result();
 
-    bool eventFilter(QObject *watched, QEvent *event) override;
-    void resizeEvent(QResizeEvent *event) override;
-
-    Q_DECLARE_PRIVATE(FolderWizard);
+    Q_DECLARE_PRIVATE(FolderWizard)
 
 private:
     QScopedPointer<FolderWizardPrivate> d_ptr;
