@@ -15,31 +15,25 @@
 #pragma once
 
 #include "abstractsetupwizardpage.h"
+#include "gui/qmlutils.h"
+
+namespace OCC {
+class OAuth;
+namespace Wizard {
+
+    class OAuthCredentialsSetupWizardPage : public AbstractSetupWizardPage
+    {
+        Q_OBJECT
+        OC_DECLARE_WIDGET_FOCUS
+
+    public:
+        explicit OAuthCredentialsSetupWizardPage(OAuth *oauth, const QUrl &serverUrl, QWidget *parent = nullptr);
+
+        bool validateInput() override;
+
+    Q_SIGNALS:
+        void openBrowserButtonPushed(const QUrl &url);
+    };
 
-namespace Ui {
-class CredentialsSetupWizardPage;
 }
-
-namespace OCC::Wizard {
-
-class OAuthCredentialsSetupWizardPage : public AbstractSetupWizardPage
-{
-    Q_OBJECT
-
-public:
-    explicit OAuthCredentialsSetupWizardPage(const QUrl &serverUrl);
-    ~OAuthCredentialsSetupWizardPage() noexcept override;
-
-    bool validateInput() override;
-
-    void setButtonsEnabled(bool enabled);
-
-Q_SIGNALS:
-    void openBrowserButtonPushed();
-    void copyUrlToClipboardButtonPushed();
-
-private:
-    ::Ui::CredentialsSetupWizardPage *_ui;
-};
-
-} // namespace OCC::Wizard
+}
