@@ -17,14 +17,11 @@
 #include "account.h"
 #include "enums.h"
 #include "gui/settingsdialog.h"
-#include "pages/abstractsetupwizardpage.h"
-#include "setupwizardaccountbuilder.h"
 #include "setupwizardcontext.h"
-#include "setupwizardwindow.h"
+#include "setupwizardcontroller_p.h"
+#include "setupwizardwidget.h"
 #include "states/abstractsetupwizardstate.h"
 
-#include <QDialog>
-#include <optional>
 
 namespace OCC::Wizard {
 /**
@@ -44,7 +41,7 @@ public:
      * Provides access to the controller's setup wizard window.
      * @return pointer to window
      */
-    SetupWizardWindow *window();
+    SetupWizardWidget *window();
 
 Q_SIGNALS:
     /**
@@ -53,7 +50,7 @@ Q_SIGNALS:
     void finished(AccountPtr newAccount, SyncMode syncMode, const QVariantMap &dynamicRegistrationData);
 
 private:
-    void changeStateTo(SetupWizardState nextState);
+    void changeStateTo(SetupWizardState nextState, SetupWizardControllerPrivate::ChangeReason reason = SetupWizardControllerPrivate::ChangeReason::Default);
 
     SetupWizardContext *_context = nullptr;
 
