@@ -15,19 +15,16 @@
  */
 
 #include "socketapi.h"
+#include "scheduling/syncscheduler.h"
 #include "socketapi_p.h"
 
 #include "gui/commonstrings.h"
 
-#include "account.h"
 #include "accountmanager.h"
-#include "accountstate.h"
-#include "capabilities.h"
 #include "common/asserts.h"
+#include "common/depreaction.h"
 #include "common/syncjournalfilerecord.h"
 #include "common/version.h"
-#include "config.h"
-#include "configfile.h"
 #include "filesystem.h"
 #include "folder.h"
 #include "folderman.h"
@@ -37,23 +34,19 @@
 #include "syncfileitem.h"
 #include "theme.h"
 
-#include <array>
-#include <QBitArray>
-#include <QUrl>
+#include <QApplication>
+#include <QDir>
+#include <QFile>
+#include <QFileDialog>
+#include <QLocalSocket>
+#include <QMessageBox>
 #include <QMetaMethod>
 #include <QMetaObject>
-#include <QStringList>
 #include <QScopedPointer>
-#include <QFile>
-#include <QDir>
-#include <QApplication>
-#include <QLocalSocket>
-#include <QStringBuilder>
-#include <QMessageBox>
-#include <QFileDialog>
+#include <QUrl>
+#include <array>
 
 
-#include <QAction>
 #include <QJsonArray>
 #include <QJsonDocument>
 #include <QJsonObject>
@@ -62,9 +55,6 @@
 
 #include <QClipboard>
 
-#include <QProcess>
-#include <QStandardPaths>
-
 #ifdef Q_OS_MAC
 #include <CoreFoundation/CoreFoundation.h>
 #endif
@@ -143,22 +133,21 @@ SocketApi::SocketApi(QObject *parent)
     qRegisterMetaType<QSharedPointer<SocketApiJob>>("QSharedPointer<SocketApiJob>");
     qRegisterMetaType<QSharedPointer<SocketApiJobV2>>("QSharedPointer<SocketApiJobV2>");
 
-    const QString socketPath = Utility::socketApiSocketPath();
+    _socketPath = Utility::socketApiSocketPath();
 
     // Remove any old socket that might be lying around:
-    SocketApiServer::removeServer(socketPath);
+    SocketApiServer::removeServer(_socketPath);
 
     // Create the socket path:
     if (!Utility::isMac()) {
         // Not on macOS: there the directory is there, and created for us by the sandboxing
         // environment, because we belong to an App Group.
-        QFileInfo info(socketPath);
+        QFileInfo info(_socketPath);
         if (!info.dir().exists()) {
             bool result = info.dir().mkpath(QStringLiteral("."));
             qCDebug(lcSocketApi) << "creating" << info.dir().path() << result;
             if (result) {
-                QFile::setPermissions(socketPath,
-                    QFile::Permissions(QFile::ReadOwner | QFile::WriteOwner | QFile::ExeOwner));
+                QFile::setPermissions(_socketPath, QFile::Permissions(QFile::ReadOwner | QFile::WriteOwner | QFile::ExeOwner));
             }
         }
     }
@@ -166,18 +155,11 @@ SocketApi::SocketApi(QObject *parent)
     // Wire up the server instance to us, so we can accept new connections:
     connect(&_localServer, &SocketApiServer::newConnection, this, &SocketApi::slotNewConnection);
 
-    // Start listeneing:
-    if (_localServer.listen(socketPath)) {
-        qCInfo(lcSocketApi) << "server started, listening at " << socketPath;
-    } else {
-        qCWarning(lcSocketApi) << "can't start server" << socketPath;
-    }
-
-    // folder watcher
-    connect(FolderMan::instance(), &FolderMan::folderSyncStateChange, this, &SocketApi::slotUpdateFolderView);
-
-    // Now we're ready to start the native shell integration:
-    Utility::startShellIntegration();
+    connect(AccountManager::instance(), &AccountManager::accountRemoved, this, [this](const auto &accountState) {
+        if (_registeredAccounts.contains(accountState->account())) {
+            unregisterAccount(accountState->account());
+        }
+    });
 }
 
 SocketApi::~SocketApi()
@@ -189,6 +171,19 @@ SocketApi::~SocketApi()
     _listeners.clear();
 }
 
+void SocketApi::startShellIntegration()
+{
+    // Start listeneing:
+    if (_localServer.listen(_socketPath)) {
+        qCInfo(lcSocketApi) << "server started, listening at " << _socketPath;
+    } else {
+        qCWarning(lcSocketApi) << "can't start server" << _socketPath;
+    }
+
+    // Now we're ready to start the native shell integration:
+    Utility::startShellIntegration();
+}
+
 void SocketApi::slotNewConnection()
 {
     // Note that on macOS this is not actually a line-based QIODevice, it's a SocketApiSocket which is our
@@ -213,7 +208,7 @@ void SocketApi::slotNewConnection()
 
     auto listener = QSharedPointer<SocketListener>::create(socket);
     _listeners.insert(socket, listener);
-    for (const auto &a : qAsConst(_registeredAccounts)) {
+    for (const auto &a : std::as_const(_registeredAccounts)) {
         if (a->hasDefaultSyncRoot()) {
             broadcastMessage(buildRegisterPathMessage(Utility::stripTrailingSlash(a->defaultSyncRoot())));
         }
@@ -240,9 +235,7 @@ void SocketApi::slotReadSocket()
     static auto invalidListener = QSharedPointer<SocketListener>::create(nullptr);
     const auto listener = _listeners.value(socket, invalidListener);
     while (socket->canReadLine()) {
-        // Make sure to normalize the input from the socket to
-        // make sure that the path will match, especially on OS X.
-        QString line = QString::fromUtf8(socket->readLine()).normalized(QString::NormalizationForm_C);
+        QString line = QString::fromUtf8(socket->readLine());
         // Note: do NOT use QString::trimmed() here! That will also remove any trailing spaces (which _are_ part of the filename)!
         line.chop(1); // remove the '\n'
 
@@ -258,7 +251,7 @@ void SocketApi::slotReadSocket()
             } else {
                 functionWithArguments += command.toUtf8() + QByteArrayLiteral("(QString,SocketListener*)");
             }
-            Q_ASSERT(staticQtMetaObject.normalizedSignature(functionWithArguments.constData()) == functionWithArguments);
+            Q_ASSERT(staticMetaObject.normalizedSignature(functionWithArguments.constData()) == functionWithArguments);
             const auto out = staticMetaObject.indexOfMethod(functionWithArguments.constData());
             if (out == -1) {
                 listener->sendError(QStringLiteral("Function %1 not found").arg(QString::fromUtf8(functionWithArguments)));
@@ -267,7 +260,7 @@ void SocketApi::slotReadSocket()
             return out;
         }();
 
-        const auto argument = argPos != -1 ? line.midRef(argPos + 1) : QStringRef();
+        const auto argument = argPos != -1 ? line.mid(argPos + 1) : QString();
         if (command.startsWith(QLatin1String("ASYNC_"))) {
             auto arguments = argument.split(QLatin1Char('|'));
             if (arguments.size() != 2) {
@@ -279,8 +272,7 @@ void SocketApi::slotReadSocket()
 
             auto jobId = arguments[0];
 
-            auto socketApiJob = QSharedPointer<SocketApiJob>(
-                new SocketApiJob(jobId.toString(), listener, json), &QObject::deleteLater);
+            auto socketApiJob = QSharedPointer<SocketApiJob>(new SocketApiJob(jobId, listener, json), &QObject::deleteLater);
             if (indexOfMethod != -1) {
                 staticMetaObject.method(indexOfMethod)
                     .invoke(this, Qt::QueuedConnection,
@@ -294,7 +286,7 @@ void SocketApi::slotReadSocket()
             QJsonParseError error;
             const auto json = QJsonDocument::fromJson(argument.toUtf8(), &error).object();
             if (error.error != QJsonParseError::NoError) {
-                qCWarning(lcSocketApi()) << "Invalid json" << argument.toString() << error.errorString();
+                qCWarning(lcSocketApi()) << "Invalid json" << argument << error.errorString();
                 listener->sendError(error.errorString());
                 return;
             }
@@ -312,9 +304,7 @@ void SocketApi::slotReadSocket()
             if (indexOfMethod != -1) {
                 // to ensure that listener is still valid we need to call it with Qt::DirectConnection
                 OC_ASSERT(thread() == QThread::currentThread())
-                staticMetaObject.method(indexOfMethod)
-                    .invoke(this, Qt::DirectConnection, Q_ARG(QString, argument.toString()),
-                        Q_ARG(SocketListener*, listener.data()));
+                staticMetaObject.method(indexOfMethod).invoke(this, Qt::DirectConnection, Q_ARG(QString, argument), Q_ARG(SocketListener *, listener.data()));
             }
         }
     }
@@ -407,7 +397,7 @@ void SocketApi::slotUpdateFolderView(Folder *f)
 
 void SocketApi::broadcastMessage(const QString &msg, bool doWait)
 {
-    for (const auto &listener : qAsConst(_listeners)) {
+    for (const auto &listener : std::as_const(_listeners)) {
         listener->sendMessage(msg, doWait);
     }
 }
@@ -449,7 +439,7 @@ void SocketApi::processShareRequest(const QString &localFile, SocketListener *li
         const QString message = QLatin1String("SHARE:OK:") + QDir::toNativeSeparators(localFile);
         listener->sendMessage(message);
 
-        emit shareCommandReceived(remotePath, fileData.localPath, startPage);
+        Q_EMIT shareCommandReceived(remotePath, fileData.localPath, startPage);
     }
 }
 
@@ -457,8 +447,8 @@ void SocketApi::broadcastStatusPushMessage(const QString &systemPath, SyncFileSt
 {
     QString msg = buildMessage(QStringLiteral("STATUS"), systemPath, fileStatus.toSocketAPIString());
     Q_ASSERT(!systemPath.endsWith(QLatin1Char('/')));
-    uint directoryHash = qHash(systemPath.left(systemPath.lastIndexOf(QLatin1Char('/'))));
-    for (const auto &listener : qAsConst(_listeners)) {
+    auto directoryHash = qHash(systemPath.left(systemPath.lastIndexOf(QLatin1Char('/'))));
+    for (const auto &listener : std::as_const(_listeners)) {
         listener->sendMessageIfDirectoryMonitored(msg, directoryHash);
     }
 }
@@ -537,7 +527,7 @@ public:
         _shareManager.fetchShares(_serverPath);
     }
 
-private slots:
+private Q_SLOTS:
     void sharesFetched(const QList<QSharedPointer<Share>> &shares)
     {
         auto shareName = SocketApi::tr("Context menu share");
@@ -577,25 +567,25 @@ private slots:
     void linkShareCreationForbidden(const QString &message)
     {
         qCInfo(lcPublicLink) << "Could not create link share:" << message;
-        emit error(message);
+        Q_EMIT error(message);
         deleteLater();
     }
 
     void serverError(int code, const QString &message)
     {
         qCWarning(lcPublicLink) << "Share fetch/create error" << code << message;
-        emit error(message);
+        Q_EMIT error(message);
         deleteLater();
     }
 
-signals:
+Q_SIGNALS:
     void done(const QString &link);
     void error(const QString &message);
 
 private:
     void success(const QString &link)
     {
-        emit done(link);
+        Q_EMIT done(link);
         deleteLater();
     }
 
@@ -615,7 +605,7 @@ void SocketApi::command_COPY_PUBLIC_LINK(const QString &localFile, SocketListene
     connect(job, &GetOrCreatePublicLinkShare::done, this,
         [](const QString &url) { copyUrlToClipboard(QUrl(url)); });
     connect(job, &GetOrCreatePublicLinkShare::error, this,
-        [=]() { emit shareCommandReceived(fileData.serverRelativePath, fileData.localPath, ShareDialogStartPage::PublicLinks); });
+        [=]() { Q_EMIT shareCommandReceived(fileData.serverRelativePath, fileData.localPath, ShareDialogStartPage::PublicLinks); });
     job->run();
 }
 
@@ -659,14 +649,17 @@ void SocketApi::command_OPEN_PRIVATE_LINK(const QString &localFile, SocketListen
 
 void SocketApi::command_OPEN_PRIVATE_LINK_VERSIONS(const QString &localFile, SocketListener *)
 {
-    auto openVersionsLink = [](const QUrl &link) {
-        QUrl url(link);
-        QUrlQuery query(url);
-        query.addQueryItem(QStringLiteral("details"), QStringLiteral("versionsTabView"));
-        url.setQuery(query);
-        Utility::openBrowser(url, nullptr);
-    };
-    fetchPrivateLinkUrlHelper(localFile, openVersionsLink);
+    const auto fileData = FileData::get(localFile);
+    if (fileData.isValid() && fileData.folder->accountState()->account()->capabilities().filesSharing().sharing_roles) {
+        fetchPrivateLinkUrl(fileData.folder->accountState()->account(), fileData.folder->webDavUrl(), fileData.serverRelativePath, this, [](const QUrl &url) {
+            const auto queryUrl = Utility::concatUrlPath(url, QString(), {{QStringLiteral("details"), QStringLiteral("versions")}});
+            Utility::openBrowser(queryUrl, nullptr);
+        });
+    } else {
+        fetchPrivateLinkUrlHelper(localFile, [](const QUrl &link) {
+            Utility::openBrowser(Utility::concatUrlPath(link, {}, {{QStringLiteral("details"), QStringLiteral("versionsTabView")}}), nullptr);
+        });
+    }
 }
 
 void SocketApi::copyUrlToClipboard(const QUrl &link)
@@ -684,11 +677,11 @@ void SocketApi::command_MAKE_AVAILABLE_LOCALLY(const QString &filesArg, SocketLi
             continue;
 
         // Update the pin state on all items
-        data.folder->vfs().setPinState(data.folderRelativePath, PinState::AlwaysLocal);
+        std::ignore = data.folder->vfs().setPinState(data.folderRelativePath, PinState::AlwaysLocal);
 
         // Trigger sync
         data.folder->schedulePathForLocalDiscovery(data.folderRelativePath);
-        data.folder->scheduleThisFolderSoon();
+        FolderMan::instance()->scheduler()->enqueueFolder(data.folder, SyncScheduler::Priority::Medium);
     }
 }
 
@@ -703,11 +696,11 @@ void SocketApi::command_MAKE_ONLINE_ONLY(const QString &filesArg, SocketListener
             continue;
 
         // Update the pin state on all items
-        data.folder->vfs().setPinState(data.folderRelativePath, PinState::OnlineOnly);
+        std::ignore = data.folder->vfs().setPinState(data.folderRelativePath, PinState::OnlineOnly);
 
         // Trigger sync
         data.folder->schedulePathForLocalDiscovery(data.folderRelativePath);
-        data.folder->scheduleThisFolderSoon();
+        FolderMan::instance()->scheduler()->enqueueFolder(data.folder, SyncScheduler::Priority::Medium);
     }
 }
 
@@ -776,7 +769,7 @@ void SocketApi::command_MOVE_ITEM(const QString &localFile, SocketListener *)
     }
 }
 
-Q_INVOKABLE void OCC::SocketApi::command_OPEN_APP_LINK(const QString &localFile, SocketListener *listener)
+Q_INVOKABLE void OCC::SocketApi::command_OPEN_APP_LINK(const QString &localFile, [[maybe_unused]] SocketListener *listener)
 {
     const auto data = FileData::get(localFile);
     if (OC_ENSURE(data.folder)) {
@@ -792,9 +785,10 @@ void SocketApi::command_V2_LIST_ACCOUNTS(const QSharedPointer<SocketApiJobV2> &j
 {
     QJsonArray out;
     for (auto acc : AccountManager::instance()->accounts()) {
-        out << QJsonObject({ { QStringLiteral("name"), acc->account()->displayName() },
-            { QStringLiteral("id"), acc->account()->id() },
-            { QStringLiteral("uuid"), acc->account()->uuid().toString(QUuid::WithoutBraces) } });
+        OC_DISABLE_DEPRECATED_WARNING; // allow use of id
+        out << QJsonObject({{QStringLiteral("name"), acc->account()->displayNameWithHost()}, {QStringLiteral("id"), acc->account()->id()},
+            {QStringLiteral("uuid"), acc->account()->uuid().toString(QUuid::WithoutBraces)}});
+        OC_ENABLE_DEPRECATED_WARNING
     }
     job->success({ { QStringLiteral("accounts"), out } });
 }
@@ -899,9 +893,14 @@ void SocketApi::sendSharingContextMenuOptions(const FileData &fileData, SocketLi
             && !capabilities.sharePublicLinkEnforcePasswordForReadOnly();
 
         if (canCreateDefaultPublicLink) {
-            listener->sendMessage(QStringLiteral("MENU_ITEM:COPY_PUBLIC_LINK") + flagString + tr("Create and copy public link to clipboard"));
+            if (fileData.folder->accountState()->supportsSpaces()) {
+                // TODO: See https://github.com/owncloud/client/issues/10845 : oCIS is getting a new sharing API, waiting for that before implementing a
+                // temporary solution.
+            } else {
+                listener->sendMessage(QStringLiteral("MENU_ITEM:COPY_PUBLIC_LINK") + flagString + tr("Create and copy public link to clipboard"));
+            }
         } else if (publicLinksEnabled) {
-            listener->sendMessage(QStringLiteral("MENU_ITEM:MANAGE_PUBLIC_LINKS") + flagString + tr("Copy public link to clipboard"));
+            listener->sendMessage(QStringLiteral("MENU_ITEM:MANAGE_PUBLIC_LINKS") + flagString + tr("Manage public links"));
         }
     }
 
@@ -944,8 +943,9 @@ bool OCC::SocketApi::FileData::isSyncFolder() const
 
 SyncFileStatus SocketApi::FileData::syncFileStatus() const
 {
-    if (!folder)
+    if (!folder || !folder->canSync()) {
         return SyncFileStatus::StatusNone;
+    }
     return folder->syncEngine().syncFileStatusTracker().fileStatus(folderRelativePath);
 }
 
@@ -994,8 +994,10 @@ void SocketApi::command_GET_MENU_ITEMS(const QString &argument, OCC::SocketListe
                     listener->sendMessage(QLatin1String("MENU_ITEM:OPEN_PRIVATE_LINK") + flagString + CommonStrings::showInWebBrowser());
                 }
                 // Add link to versions pane if possible
-                if (capabilities.versioningEnabled() && capabilities.privateLinkDetailsParamAvailable() && isOnTheServer && !record.isDirectory()) {
-                    listener->sendMessage(QLatin1String("MENU_ITEM:OPEN_PRIVATE_LINK_VERSIONS") + flagString + tr("Show file versions in browser"));
+                if (capabilities.versioningEnabled() && isOnTheServer && !record.isDirectory()) {
+                    if (capabilities.privateLinkDetailsParamAvailable() || capabilities.filesSharing().sharing_roles) {
+                        listener->sendMessage(QLatin1String("MENU_ITEM:OPEN_PRIVATE_LINK_VERSIONS") + flagString + tr("Show file versions in web browser"));
+                    }
                 }
 
                 // Conflict files get conflict resolution actions
@@ -1108,8 +1110,7 @@ void SocketApi::command_GET_MENU_ITEMS(const QString &argument, OCC::SocketListe
         // TODO: Should be a submenu, should use icons
         auto makePinContextMenu = [&](bool makeAvailableLocally, bool freeSpace) {
             listener->sendMessage(QStringLiteral("MENU_SEPARATOR:d::"));
-            listener->sendMessage(QStringLiteral("MENU_ITEM:CURRENT_PIN:d:")
-                + Utility::vfsCurrentAvailabilityText(*combined));
+            listener->sendMessage(QStringLiteral("MENU_ITEM:CURRENT_PIN:d:") + Utility::enumToDisplayName(combined.get()));
             listener->sendMessage(QStringLiteral("MENU_ITEM:MAKE_AVAILABLE_LOCALLY:")
                 + (makeAvailableLocally ? QStringLiteral(":") : QStringLiteral("d:"))
                 + Utility::vfsPinActionText());
