@@ -14,18 +14,22 @@
 
 #include "accessmanager.h"
 #include "application.h"
+#include "common/restartmanager.h"
 #include "common/utility.h"
 #include "common/version.h"
 #include "configfile.h"
 #include "theme.h"
 
+#include "settingsdialog.h"
+#include "updatedownloadedwidget.h"
+#include "updater/newversionavailablewidget.h"
 #include "updater/ocupdater.h"
 #include "updater/updater_private.h"
 
 #include <QObject>
 #include <QtCore>
-#include <QtNetwork>
 #include <QtGui>
+#include <QtNetwork>
 #include <QtWidgets>
 
 using namespace std::chrono_literals;
@@ -34,18 +38,27 @@ using namespace std::chrono_literals;
 
 namespace OCC {
 
-UpdaterScheduler::UpdaterScheduler(QObject *parent)
+UpdaterScheduler::UpdaterScheduler(Application *app, QObject *parent)
     : QObject(parent)
 {
     connect(&_updateCheckTimer, &QTimer::timeout,
         this, &UpdaterScheduler::slotTimerFired);
 
     // Note: the sparkle-updater is not an OCUpdater
-    if (OCUpdater *updater = qobject_cast<OCUpdater *>(Updater::instance())) {
-        connect(updater, &OCUpdater::newUpdateAvailable,
-            this, &UpdaterScheduler::updaterAnnouncement);
+    if (auto *updater = qobject_cast<OCUpdater *>(Updater::instance())) {
+        connect(updater, &OCUpdater::updateAvailableThroughSystem, app,
+            [app, updater]() { app->gui()->slotShowTrayMessage(tr("Update available"), updater->statusString()); });
 
-        connect(updater, &OCUpdater::requestRestart, this, &UpdaterScheduler::requestRestart);
+        connect(updater, &OCUpdater::updateDownloaded, this, [app, updater, this]() {
+            // prevent dialog from being displayed twice (rather unlikely, but it won't hurt)
+            if (_updateDownloadedWidget == nullptr) {
+                _updateDownloadedWidget = new UpdateDownloadedWidget(app->gui()->settingsDialog(), updater->statusString());
+                ocApp()->gui()->settingsDialog()->addModalWidget(_updateDownloadedWidget);
+
+                connect(_updateDownloadedWidget, &UpdateDownloadedWidget::accepted, this, []() { Updater::instance()->applyUpdateAndRestart(); });
+                connect(_updateDownloadedWidget, &UpdateDownloadedWidget::finished, this, [this]() { delete _updateDownloadedWidget.data(); });
+            }
+        });
 
         connect(updater, &OCUpdater::retryUpdateCheckLater, this, [this]() {
             qCInfo(lcUpdater) << "Retrying update check in 10 minutes";
@@ -101,24 +114,6 @@ void OCUpdater::setUpdateUrl(const QUrl &url)
     _updateUrl = url;
 }
 
-bool OCUpdater::performUpdate()
-{
-    auto settings = ConfigFile::makeQSettings();
-    QString updateFile = settings.value(updateAvailableC).toString();
-    if (!updateFile.isEmpty() && QFile(updateFile).exists()
-        && !updateSucceeded() /* Someone might have run the updater manually between restarts */) {
-        const QString name = Theme::instance()->appNameGUI();
-        if (QMessageBox::information(nullptr, tr("New %1 Update Ready").arg(name),
-                tr("A new update for %1 is about to be installed. The updater may ask\n"
-                   "for additional privileges during the process.")
-                    .arg(name),
-                QMessageBox::Ok)) {
-            slotStartInstaller();
-            return true;
-        }
-    }
-    return false;
-}
 
 void OCUpdater::backgroundCheckForUpdate()
 {
@@ -150,7 +145,11 @@ QString OCUpdater::statusString() const
     case Downloading:
         return tr("Downloading %1. Please wait...").arg(updateVersion);
     case DownloadComplete:
-        return tr("%1 available. Restart application to start the update.").arg(updateVersion);
+        if (Utility::runningInAppImage()) {
+            return tr("%1 installed successfully. Restart the application to finish installing the update.").arg(updateVersion);
+        } else {
+            return tr("%1 available. Restart application to start the update.").arg(updateVersion);
+        }
     case DownloadFailed:
         return tr("Could not download update. Please click <a href='%1'>here</a> to download the update manually.").arg(_updateInfo.web());
     case DownloadTimedOut:
@@ -184,46 +183,22 @@ void OCUpdater::setDownloadState(DownloadState state)
 {
     auto oldState = _state;
     _state = state;
-    emit downloadStateChanged();
+    Q_EMIT downloadStateChanged();
 
     // show the notification if the download is complete (on every check)
     // or once for system based updates.
-    if (_state == OCUpdater::DownloadComplete || (oldState != OCUpdater::UpdateOnlyAvailableThroughSystem
-                                                     && _state == OCUpdater::UpdateOnlyAvailableThroughSystem)) {
-        emit newUpdateAvailable(tr("Update Check"), statusString());
+    if (_state == OCUpdater::DownloadComplete) {
+        Q_EMIT updateDownloaded();
+    }
+
+    if (oldState != OCUpdater::UpdateOnlyAvailableThroughSystem && _state == OCUpdater::UpdateOnlyAvailableThroughSystem) {
+        Q_EMIT updateAvailableThroughSystem();
     }
 }
 
-void OCUpdater::slotStartInstaller()
+QString OCUpdater::availableVersionString() const
 {
-    auto settings = ConfigFile::makeQSettings();
-    QString updateFile = settings.value(updateAvailableC).toString();
-    settings.setValue(autoUpdateAttemptedC, true);
-    settings.sync();
-    qCInfo(lcUpdater) << "Running updater" << updateFile;
-
-    if (updateFile.endsWith(QLatin1String(".exe"))) {
-        QProcess::startDetached(updateFile, QStringList() << QStringLiteral("/S") << QStringLiteral("/launch"));
-    } else if (updateFile.endsWith(QLatin1String(".msi"))) {
-        // When MSIs are installed without gui they cannot launch applications
-        // as they lack the user context. That is why we need to run the client
-        // manually here. We wrap the msiexec and client invocation in a powershell
-        // script because owncloud.exe will be shut down for installation.
-        // | Out-Null forces powershell to wait for msiexec to finish.
-        auto preparePathForPowershell = [](QString path) {
-            path.replace(QLatin1String("'"), QLatin1String("''"));
-
-            return QDir::toNativeSeparators(path);
-        };
-
-        QString msiLogFile = ConfigFile::configPath() + QStringLiteral("msi.log");
-        const QString command = QStringLiteral("&{msiexec /norestart /passive /i '%1' /L*V '%2'| Out-Null ; &'%3'}")
-                                    .arg(preparePathForPowershell(updateFile),
-                                        preparePathForPowershell(msiLogFile),
-                                        preparePathForPowershell(QCoreApplication::applicationFilePath()));
-
-        QProcess::startDetached(QStringLiteral("powershell.exe"), QStringList { QStringLiteral("-Command"), command });
-    }
+    return _updateInfo.versionString();
 }
 
 void OCUpdater::checkForUpdate()
@@ -298,12 +273,12 @@ void OCUpdater::setPreviouslySkippedVersion(const QString &previouslySkippedVers
 
 ////////////////////////////////////////////////////////////////////////
 
-NSISUpdater::NSISUpdater(const QUrl &url)
+WindowsUpdater::WindowsUpdater(const QUrl &url)
     : OCUpdater(url)
 {
 }
 
-void NSISUpdater::slotWriteFile()
+void WindowsUpdater::slotWriteFile()
 {
     QNetworkReply *reply = qobject_cast<QNetworkReply *>(sender());
     if (_file->isOpen()) {
@@ -311,7 +286,7 @@ void NSISUpdater::slotWriteFile()
     }
 }
 
-void NSISUpdater::wipeUpdateData()
+void WindowsUpdater::wipeUpdateData()
 {
     auto settings = ConfigFile::makeQSettings();
     QString updateFileName = settings.value(updateAvailableC).toString();
@@ -323,7 +298,7 @@ void NSISUpdater::wipeUpdateData()
     settings.remove(autoUpdateAttemptedC);
 }
 
-void NSISUpdater::slotDownloadFinished()
+void WindowsUpdater::slotDownloadFinished()
 {
     QNetworkReply *reply = qobject_cast<QNetworkReply *>(sender());
     const QUrl url(reply->url());
@@ -356,7 +331,7 @@ void NSISUpdater::slotDownloadFinished()
     settings.setValue(updateAvailableC, _targetFile);
 }
 
-void NSISUpdater::versionInfoArrived(const UpdateInfo &info)
+void WindowsUpdater::versionInfoArrived(const UpdateInfo &info)
 {
     auto settings = ConfigFile::makeQSettings();
     const auto infoVersion = QVersionNumber::fromString(info.version());
@@ -379,7 +354,7 @@ void NSISUpdater::versionInfoArrived(const UpdateInfo &info)
     } else {
         const QString url = info.downloadUrl();
         if (url.isEmpty()) {
-            showNoUrlDialog(info);
+            showNewVersionAvailableWidget(info);
         } else {
             _targetFile = ConfigFile::configPath() + url.mid(url.lastIndexOf(QLatin1Char('/')) + 1);
             if (QFile::exists(_targetFile)) {
@@ -388,8 +363,8 @@ void NSISUpdater::versionInfoArrived(const UpdateInfo &info)
                 auto request = QNetworkRequest(QUrl(url));
                 request.setAttribute(QNetworkRequest::RedirectPolicyAttribute, QNetworkRequest::NoLessSafeRedirectPolicy);
                 QNetworkReply *reply = qnam()->get(request);
-                connect(reply, &QIODevice::readyRead, this, &NSISUpdater::slotWriteFile);
-                connect(reply, &QNetworkReply::finished, this, &NSISUpdater::slotDownloadFinished);
+                connect(reply, &QIODevice::readyRead, this, &WindowsUpdater::slotWriteFile);
+                connect(reply, &QNetworkReply::finished, this, &WindowsUpdater::slotDownloadFinished);
                 setDownloadState(Downloading);
                 _file.reset(new QTemporaryFile);
                 _file->setAutoRemove(true);
@@ -399,58 +374,23 @@ void NSISUpdater::versionInfoArrived(const UpdateInfo &info)
     }
 }
 
-void NSISUpdater::showNoUrlDialog(const UpdateInfo &info)
+void WindowsUpdater::showNewVersionAvailableWidget(const UpdateInfo &info)
 {
     // if the version tag is set, there is a newer version.
-    QDialog *msgBox = new QDialog;
-    msgBox->setAttribute(Qt::WA_DeleteOnClose);
-
-    QIcon infoIcon = msgBox->style()->standardIcon(QStyle::SP_MessageBoxInformation);
-    int iconSize = msgBox->style()->pixelMetric(QStyle::PM_MessageBoxIconSize);
-
-    msgBox->setWindowIcon(infoIcon);
-
-    QVBoxLayout *layout = new QVBoxLayout(msgBox);
-    QHBoxLayout *hlayout = new QHBoxLayout;
-    layout->addLayout(hlayout);
-
-    msgBox->setWindowTitle(tr("New Version Available"));
-
-    QLabel *ico = new QLabel;
-    ico->setFixedSize(iconSize, iconSize);
-    ico->setPixmap(infoIcon.pixmap(iconSize));
-    QLabel *lbl = new QLabel;
     QString txt = tr("<p>A new version of the %1 Client is available.</p>"
                      "<p><b>%2</b> is available for download. The installed version is %3.</p>")
                       .arg(Utility::escape(Theme::instance()->appNameGUI()),
                           Utility::escape(info.versionString()), Utility::escape(Version::versionWithBuildNumber().toString()));
+    auto *widget = new NewVersionAvailableWidget(ocApp()->gui()->settingsDialog(), txt);
 
-    lbl->setText(txt);
-    lbl->setTextFormat(Qt::RichText);
-    lbl->setWordWrap(true);
+    connect(widget, &NewVersionAvailableWidget::versionSkipped, this, &WindowsUpdater::slotSetPreviouslySkippedVersion);
+    connect(widget, &NewVersionAvailableWidget::updateNow, this, &WindowsUpdater::slotOpenUpdateUrl);
+    connect(widget, &NewVersionAvailableWidget::finished, this, [widget]() { delete widget; });
 
-    hlayout->addWidget(ico);
-    hlayout->addWidget(lbl);
-
-    QDialogButtonBox *bb = new QDialogButtonBox;
-    QPushButton *skip = bb->addButton(tr("Skip this version"), QDialogButtonBox::ResetRole);
-    QPushButton *reject = bb->addButton(tr("Skip this time"), QDialogButtonBox::AcceptRole);
-    QPushButton *getupdate = bb->addButton(tr("Get update"), QDialogButtonBox::AcceptRole);
-
-    connect(skip, &QAbstractButton::clicked, msgBox, &QDialog::reject);
-    connect(reject, &QAbstractButton::clicked, msgBox, &QDialog::reject);
-    connect(getupdate, &QAbstractButton::clicked, msgBox, &QDialog::accept);
-
-    connect(skip, &QAbstractButton::clicked, this, &NSISUpdater::slotSetPreviouslySkippedVersion);
-    connect(getupdate, &QAbstractButton::clicked, this, &NSISUpdater::slotOpenUpdateUrl);
-
-    layout->addWidget(bb);
-
-    msgBox->open();
-    ownCloudGui::raiseDialog(msgBox);
+    ocApp()->gui()->settingsDialog()->addModalWidget(widget);
 }
 
-void NSISUpdater::showUpdateErrorDialog(const QString &targetVersion)
+void WindowsUpdater::showUpdateErrorDialog(const QString &targetVersion)
 {
     QDialog *msgBox = new QDialog;
     msgBox->setAttribute(Qt::WA_DeleteOnClose);
@@ -499,7 +439,7 @@ void NSISUpdater::showUpdateErrorDialog(const QString &targetVersion)
     });
     // askagain: do nothing
     connect(retry, &QAbstractButton::clicked, this, [this]() {
-        slotStartInstaller();
+        applyUpdateAndRestart();
         qApp->quit();
     });
     connect(getupdate, &QAbstractButton::clicked, this, [this]() {
@@ -511,11 +451,10 @@ void NSISUpdater::showUpdateErrorDialog(const QString &targetVersion)
     msgBox->open();
 }
 
-bool NSISUpdater::handleStartup()
+void WindowsUpdater::validateUpdate()
 {
-    ConfigFile cfg;
-    auto settings = ConfigFile::makeQSettings();
-    QString updateFileName = settings.value(updateAvailableC).toString();
+    const auto settings = ConfigFile::makeQSettings();
+    const QString updateFileName = settings.value(updateAvailableC).toString();
     // has the previous run downloaded an update?
     if (!updateFileName.isEmpty() && QFile(updateFileName).exists()) {
         qCInfo(lcUpdater) << "An updater file is available";
@@ -526,47 +465,79 @@ bool NSISUpdater::handleStartup()
                 qCInfo(lcUpdater) << "The requested update attempt has succeeded"
                                   << Version::versionWithBuildNumber();
                 wipeUpdateData();
-                return false;
             } else {
                 // auto update failed. Ask user what to do
                 qCInfo(lcUpdater) << "The requested update attempt has failed"
                         << settings.value(updateTargetVersionC).toString();
                 showUpdateErrorDialog(settings.value(updateTargetVersionStringC).toString());
-                return false;
             }
-        } else {
-            qCInfo(lcUpdater) << "Triggering an update";
-            return performUpdate();
         }
     }
-    return false;
 }
 
-void NSISUpdater::slotSetPreviouslySkippedVersion()
+void WindowsUpdater::slotSetPreviouslySkippedVersion()
 {
     setPreviouslySkippedVersion(updateInfo().version());
 }
 
+void WindowsUpdater::applyUpdateAndRestart()
+{
+    Q_ASSERT(downloadState() == DownloadState::DownloadComplete);
+
+    auto settings = ConfigFile::makeQSettings();
+    QString updateFile = settings.value(updateAvailableC).toString();
+    settings.setValue(autoUpdateAttemptedC, true);
+    settings.sync();
+    qCInfo(lcUpdater) << "Running updater" << updateFile;
+
+    Q_ASSERT(updateFile.endsWith(QLatin1String(".msi")));
+    // When MSIs are installed without gui they cannot launch applications
+    // as they lack the user context. That is why we need to run the client
+    // manually here. We wrap the msiexec and client invocation in a powershell
+    // script because owncloud.exe will be shut down for installation.
+    // | Out-Null forces powershell to wait for msiexec to finish.
+    auto preparePathForPowershell = [](QString path) {
+        path.replace(QLatin1String("'"), QLatin1String("''"));
+
+        return QDir::toNativeSeparators(path);
+    };
+
+    QString msiLogFile = ConfigFile::configPath() + QStringLiteral("msi.log");
+    const QString command =
+        QStringLiteral("&{msiexec /norestart /passive /i '%1' /L*V '%2'| Out-Null ; &'%3'}")
+            .arg(preparePathForPowershell(updateFile), preparePathForPowershell(msiLogFile), preparePathForPowershell(QCoreApplication::applicationFilePath()));
+
+    QProcess::startDetached(QStringLiteral("powershell.exe"), QStringList{QStringLiteral("-Command"), command});
+    QTimer::singleShot(0, QApplication::instance(), &QApplication::quit);
+}
+
 ////////////////////////////////////////////////////////////////////////
 
+namespace {
+    QDateTime applicationMTime()
+    {
+        return QFileInfo(QApplication::applicationFilePath()).lastModified().toUTC();
+    }
+}
+
 PassiveUpdateNotifier::PassiveUpdateNotifier(const QUrl &url)
     : OCUpdater(url)
+    , _initialAppMTime(applicationMTime())
 {
-    // remember the version of the currently running binary. On Linux it might happen that the
-    // package management updates the package while the app is running. This is detected in the
-    // updater slot: If the installed binary on the hd has a different version than the one
-    // running, the running app is restarted. That happens in folderman.
-    _runningAppVersion = Utility::versionOfInstalledBinary();
 }
 
 void PassiveUpdateNotifier::backgroundCheckForUpdate()
 {
-    if (Utility::isLinux()) {
-        // on linux, check if the installed binary is still the same version
-        // as the one that is running. If not, restart if possible.
-        const QByteArray fsVersion = Utility::versionOfInstalledBinary();
-        if (!(fsVersion.isEmpty() || _runningAppVersion.isEmpty()) && fsVersion != _runningAppVersion) {
-            emit requestRestart();
+    if (Utility::isLinux() && !Utility::runningInAppImage()) {
+        // a bit of a naive approach, since it doesn't exclude downgrades or forced reinstalls
+        // whenever the application binary's mtime changes, we assume it has changed, since users are expected not to call touch on it or otherwise change it
+        // manually
+        const auto currentMTime = applicationMTime();
+        qCDebug(lcUpdater) << "initial mtime:" << _initialAppMTime << "current mtime:" << currentMTime;
+        if (currentMTime != _initialAppMTime) {
+            qCInfo(lcUpdater) << "Binary mtime changed since application started, requesting restart";
+            setDownloadState(DownloadState::DownloadComplete);
+            Q_EMIT updateDownloaded();
         }
     }
 
