@@ -12,11 +12,12 @@
  * for more details.
  */
 
-#include "config.h"
-
 #include "common/asserts.h"
 #include "common/utility.h"
 #include "common/version.h"
+#ifdef Q_OS_WIN
+#include "common/utility_win.h"
+#endif
 #include "configfile.h"
 #include "logger.h"
 #include "theme.h"
@@ -25,7 +26,6 @@
 
 #include "csync_exclude.h"
 
-#include <QWidget>
 #include <QHeaderView>
 #include <QCoreApplication>
 #include <QDir>
@@ -55,8 +55,10 @@ const QString notificationRefreshIntervalC() { return QStringLiteral("notificati
 const QString monoIconsC() { return QStringLiteral("monoIcons"); }
 const QString promptDeleteC() { return QStringLiteral("promptDeleteAllFiles"); }
 const QString crashReporterC() { return QStringLiteral("crashReporter"); }
-const QString optionalDesktopNoficationsC() { return QStringLiteral("optionalDesktopNotifications"); }
-const QString showInExplorerNavigationPaneC() { return QStringLiteral("showInExplorerNavigationPane"); }
+const QString optionalDesktopNoficationsC()
+{
+    return QStringLiteral("optionalDesktopNotifications");
+}
 const QString skipUpdateCheckC() { return QStringLiteral("skipUpdateCheck"); }
 const QString updateCheckIntervalC() { return QStringLiteral("updateCheckInterval"); }
 const QString updateChannelC() { return QStringLiteral("updateChannel"); }
@@ -68,15 +70,22 @@ const QString minChunkSizeC() { return QStringLiteral("minChunkSize"); }
 const QString maxChunkSizeC() { return QStringLiteral("maxChunkSize"); }
 const QString targetChunkUploadDurationC() { return QStringLiteral("targetChunkUploadDuration"); }
 const QString automaticLogDirC() { return QStringLiteral("logToTemporaryLogDir"); }
-const QString numberOfLogsToKeepC() { return QStringLiteral("numberOfLogsToKeep"); }
-const QString showExperimentalOptionsC() { return QStringLiteral("showExperimentalOptions"); }
+const QString numberOfLogsToKeepC()
+{
+    return QStringLiteral("numberOfLogsToKeep");
+}
+
+// The key `clientVersion` stores the version *with* build number of the config file. It is named
+// this way, because before 5.0, only the version *without* build number was stored.
 const QString clientVersionC() { return QStringLiteral("clientVersion"); }
 
 const QString proxyHostC() { return QStringLiteral("Proxy/host"); }
 const QString proxyTypeC() { return QStringLiteral("Proxy/type"); }
 const QString proxyPortC() { return QStringLiteral("Proxy/port"); }
-const QString proxyUserC() { return QStringLiteral("Proxy/user"); }
-const QString proxyPassC() { return QStringLiteral("Proxy/pass"); }
+const QString proxyUserC()
+{
+    return QStringLiteral("Proxy/user");
+}
 const QString proxyNeedsAuthC() { return QStringLiteral("Proxy/needsAuth"); }
 
 const QString useUploadLimitC() { return QStringLiteral("BWLimit/useUploadLimit"); }
@@ -84,11 +93,24 @@ const QString useDownloadLimitC() { return QStringLiteral("BWLimit/useDownloadLi
 const QString uploadLimitC() { return QStringLiteral("BWLimit/uploadLimit"); }
 const QString downloadLimitC() { return QStringLiteral("BWLimit/downloadLimit"); }
 
-const QString newBigFolderSizeLimitC() { return QStringLiteral("newBigFolderSizeLimit"); }
-const QString useNewBigFolderSizeLimitC() { return QStringLiteral("useNewBigFolderSizeLimit"); }
-const QString confirmExternalStorageC() { return QStringLiteral("confirmExternalStorage"); }
-const QString moveToTrashC() { return QStringLiteral("moveToTrash"); }
+const QString pauseSyncWhenMeteredC()
+{
+    return QStringLiteral("pauseWhenMetered");
 }
+const QString moveToTrashC() { return QStringLiteral("moveToTrash"); }
+
+const QString issuesWidgetFilterC()
+{
+    return QStringLiteral("issuesWidgetFilter");
+}
+
+QString excludeFileNameC()
+{
+    static_assert(!std::string_view(EXCLUDE_FILE_NAME).empty());
+    return QStringLiteral(EXCLUDE_FILE_NAME);
+}
+
+} // anonymous namespace
 
 QString ConfigFile::_confDir = QString();
 const std::chrono::seconds DefaultRemotePollInterval { 30 };
@@ -130,16 +152,20 @@ bool ConfigFile::optionalDesktopNotifications() const
     return settings.value(optionalDesktopNoficationsC(), true).toBool();
 }
 
-bool ConfigFile::showInExplorerNavigationPane() const
+std::optional<QStringList> ConfigFile::issuesWidgetFilter() const
 {
     auto settings = makeQSettings();
-    return settings.value(showInExplorerNavigationPaneC(), QOperatingSystemVersion::current() >= QOperatingSystemVersion::Windows10).toBool();
+    if (settings.contains(issuesWidgetFilterC())) {
+        return settings.value(issuesWidgetFilterC()).toStringList();
+    }
+
+    return {};
 }
 
-void ConfigFile::setShowInExplorerNavigationPane(bool show)
+void ConfigFile::setIssuesWidgetFilter(const QStringList &checked)
 {
     auto settings = makeQSettings();
-    settings.setValue(showInExplorerNavigationPaneC(), show);
+    settings.setValue(issuesWidgetFilterC(), checked);
     settings.sync();
 }
 
@@ -224,15 +250,14 @@ QVariant ConfigFile::getPolicySetting(const QString &setting, const QVariant &de
 {
     if (Utility::isWindows()) {
         // check for policies first and return immediately if a value is found.
-        QSettings userPolicy(QStringLiteral("HKEY_CURRENT_USER\\Software\\Policies\\%1\\%2")
-                                 .arg(QStringLiteral(APPLICATION_VENDOR), Theme::instance()->appNameGUI()),
+        QSettings userPolicy(QStringLiteral("HKEY_CURRENT_USER\\Software\\Policies\\%1\\%2").arg(Theme::instance()->vendor(), Theme::instance()->appNameGUI()),
             QSettings::NativeFormat);
         if (userPolicy.contains(setting)) {
             return userPolicy.value(setting);
         }
 
-        QSettings machinePolicy(QStringLiteral("HKEY_LOCAL_MACHINE\\Software\\Policies\\%1\\%2")
-                                    .arg(QStringLiteral(APPLICATION_VENDOR), Theme::instance()->appNameGUI()),
+        QSettings machinePolicy(
+            QStringLiteral("HKEY_LOCAL_MACHINE\\Software\\Policies\\%1\\%2").arg(Theme::instance()->vendor(), Theme::instance()->appNameGUI()),
             QSettings::NativeFormat);
         if (machinePolicy.contains(setting)) {
             return machinePolicy.value(setting);
@@ -255,8 +280,6 @@ QString ConfigFile::configPath()
     return dir;
 }
 
-static const QLatin1String exclFile("sync-exclude.lst");
-
 QString ConfigFile::excludeFile(Scope scope) const
 {
 #ifdef Q_OS_WIN
@@ -269,13 +292,13 @@ QString ConfigFile::excludeFile(Scope scope) const
 
     switch (scope) {
     case UserScope:
-        fi.setFile(configPath(), exclFile);
+        fi.setFile(configPath(), excludeFileNameC());
 
         if (!fi.isReadable()) {
             fi.setFile(configPath(), QStringLiteral("exclude.lst"));
         }
         if (!fi.isReadable()) {
-            fi.setFile(configPath(), exclFile);
+            fi.setFile(configPath(), excludeFileNameC());
         }
         return fi.absoluteFilePath();
     case SystemScope:
@@ -283,43 +306,37 @@ QString ConfigFile::excludeFile(Scope scope) const
     }
 
     OC_ASSERT(false);
-    return QString();
+    return {};
 }
 
 QString ConfigFile::excludeFileFromSystem()
 {
     QFileInfo fi;
 #ifdef Q_OS_WIN
-    fi.setFile(QCoreApplication::applicationDirPath(), exclFile);
+    fi.setFile(QCoreApplication::applicationDirPath(), excludeFileNameC());
 #endif
 #ifdef Q_OS_UNIX
-    fi.setFile(QStringLiteral(SYSCONFDIR "/%1").arg(Theme::instance()->appName()), exclFile);
+    fi.setFile(QStringLiteral(SYSCONFDIR "/%1").arg(Theme::instance()->appName()), excludeFileNameC());
     if (!fi.exists()) {
         // Prefer to return the preferred path! Only use the fallback location
         // if the other path does not exist and the fallback is valid.
-        QFileInfo nextToBinary(QCoreApplication::applicationDirPath(), exclFile);
+        QFileInfo nextToBinary(QCoreApplication::applicationDirPath(), excludeFileNameC());
         if (nextToBinary.exists()) {
             fi = nextToBinary;
         } else {
-            // For AppImage, the file might reside under a temporary mount path
-            QDir d(QCoreApplication::applicationDirPath()); // supposed to be /tmp/mount.xyz/usr/bin
-            d.cdUp(); // go out of bin
-            d.cdUp(); // go out of usr
-            if (!d.isRoot()) { // it is really a mountpoint
-                if (d.cd(QStringLiteral("etc")) && d.cd(Theme::instance()->appName())) {
-                    QFileInfo inMountDir(d, exclFile);
-                    if (inMountDir.exists()) {
-                        fi = inMountDir;
-                    }
-                };
+            // use from install tree (e.g., AppImage, local dev installation)
+            // for example, if the binary is in .../AppDir/usr/bin/<binary>, the exclude file will be in .../AppDir/usr/etc/<appname>/
+            QFileInfo relativeToBinary(
+                QStringLiteral("%1/../etc/%2/%3").arg(QCoreApplication::applicationDirPath(), Theme::instance()->appName(), excludeFileNameC()));
+            if (relativeToBinary.exists()) {
+                fi = relativeToBinary;
             }
         }
     }
 #endif
 #ifdef Q_OS_MAC
     // exec path is inside the bundle
-    fi.setFile(QCoreApplication::applicationDirPath(),
-        QLatin1String("../Resources/") + exclFile);
+    fi.setFile(QCoreApplication::applicationDirPath(), QLatin1String("../Resources/") + excludeFileNameC());
 #endif
 
     return fi.absoluteFilePath();
@@ -328,7 +345,7 @@ QString ConfigFile::excludeFileFromSystem()
 QString ConfigFile::backup() const
 {
     QString baseFile = configFile();
-    auto versionString = clientVersionString();
+    auto versionString = clientVersionWithBuildNumberString();
     if (!versionString.isEmpty())
         versionString.prepend(QLatin1Char('_'));
     const QString backupFile =
@@ -459,7 +476,7 @@ chrono::milliseconds OCC::ConfigFile::fullLocalDiscoveryInterval() const
 {
     auto settings = makeQSettings();
     settings.beginGroup(defaultConnection());
-    return millisecondsValue(settings, fullLocalDiscoveryIntervalC(), chrono::hours(1));
+    return millisecondsValue(settings, fullLocalDiscoveryIntervalC(), 1h);
 }
 
 chrono::milliseconds ConfigFile::notificationRefreshInterval(const QString &connection) const
@@ -558,11 +575,7 @@ void ConfigFile::setUiLanguage(const QString &uiLanguage)
     settings.setValue(uiLanguageC(), uiLanguage);
 }
 
-void ConfigFile::setProxyType(int proxyType,
-    const QString &host,
-    int port, bool needsAuth,
-    const QString &user,
-    const QString &pass)
+void ConfigFile::setProxyType(QNetworkProxy::ProxyType proxyType, const QString &host, int port, bool needsAuth, const QString &user)
 {
     auto settings = makeQSettings();
 
@@ -573,7 +586,6 @@ void ConfigFile::setProxyType(int proxyType,
         settings.setValue(proxyPortC(), port);
         settings.setValue(proxyNeedsAuthC(), needsAuth);
         settings.setValue(proxyUserC(), user);
-        settings.setValue(proxyPassC(), pass.toUtf8().toBase64());
     }
     settings.sync();
 }
@@ -583,7 +595,7 @@ QVariant ConfigFile::getValue(const QString &param, const QString &group,
 {
     QVariant systemSetting;
     if (Utility::isMac()) {
-        QSettings systemSettings(QStringLiteral("/Library/Preferences/" APPLICATION_REV_DOMAIN ".plist"), QSettings::NativeFormat);
+        QSettings systemSettings(QStringLiteral("/Library/Preferences/%1.plist").arg(Theme::instance()->orgDomainName()), QSettings::NativeFormat);
         if (!group.isEmpty()) {
             systemSettings.beginGroup(group);
         }
@@ -595,9 +607,8 @@ QVariant ConfigFile::getValue(const QString &param, const QString &group,
         }
         systemSetting = systemSettings.value(param, defaultValue);
     } else { // Windows
-        QSettings systemSettings(QStringLiteral("HKEY_LOCAL_MACHINE\\Software\\" APPLICATION_VENDOR "\\%1")
-                                     .arg(Theme::instance()->appNameGUI()),
-            QSettings::NativeFormat);
+        QSettings systemSettings(
+            QStringLiteral("HKEY_LOCAL_MACHINE\\Software\\%1\\%2").arg(Theme::instance()->vendor(), Theme::instance()->appNameGUI()), QSettings::NativeFormat);
         if (!group.isEmpty()) {
             systemSettings.beginGroup(group);
         }
@@ -646,12 +657,6 @@ QString ConfigFile::proxyUser() const
     return getValue(proxyUserC()).toString();
 }
 
-QString ConfigFile::proxyPassword() const
-{
-    QByteArray pass = getValue(proxyPassC()).toByteArray();
-    return QString::fromUtf8(QByteArray::fromBase64(pass));
-}
-
 int ConfigFile::useUploadLimit() const
 {
     return getValue(useUploadLimitC(), QString(), 0).toInt();
@@ -692,33 +697,23 @@ void ConfigFile::setDownloadLimit(int kbytes)
     setValue(downloadLimitC(), kbytes);
 }
 
-QPair<bool, qint64> ConfigFile::newBigFolderSizeLimit() const
+bool ConfigFile::pauseSyncWhenMetered() const
 {
-    auto defaultValue = Theme::instance()->newBigFolderSizeLimit();
-    qint64 value = getValue(newBigFolderSizeLimitC(), QString(), defaultValue).toLongLong();
-    bool use = value >= 0 && getValue(useNewBigFolderSizeLimitC(), QString(), true).toBool();
-    return qMakePair(use, qMax<qint64>(0, value));
+    return getValue(pauseSyncWhenMeteredC(), {}, false).toBool();
 }
 
-void ConfigFile::setNewBigFolderSizeLimit(bool isChecked, qint64 mbytes)
+void ConfigFile::setPauseSyncWhenMetered(bool isChecked)
 {
-    setValue(newBigFolderSizeLimitC(), mbytes);
-    setValue(useNewBigFolderSizeLimitC(), isChecked);
-}
-
-bool ConfigFile::confirmExternalStorage() const
-{
-    return getValue(confirmExternalStorageC(), QString(), true).toBool();
-}
-
-void ConfigFile::setConfirmExternalStorage(bool isChecked)
-{
-    setValue(confirmExternalStorageC(), isChecked);
+    setValue(pauseSyncWhenMeteredC(), isChecked);
 }
 
 bool ConfigFile::moveToTrash() const
 {
-    return getValue(moveToTrashC(), QString(), false).toBool();
+    if (Theme::instance()->enableMoveToTrash()) {
+        return getValue(moveToTrashC(), QString(), false).toBool();
+    }
+
+    return false;
 }
 
 void ConfigFile::setMoveToTrash(bool isChecked)
@@ -744,7 +739,7 @@ bool ConfigFile::monoIcons() const
     bool monoDefault = false; // On Mac we want bw by default
 #ifdef Q_OS_MAC
     // OEM themes are not obliged to ship mono icons
-    monoDefault = (0 == (strcmp("ownCloud", APPLICATION_NAME)));
+    monoDefault = Theme::instance()->appNameGUI() == QStringLiteral("ownCloud");
 #endif
     return settings.value(monoIconsC(), monoDefault).toBool();
 }
@@ -815,19 +810,13 @@ bool ConfigFile::logHttp() const
     return settings.value(logHttpC(), false).toBool();
 }
 
-bool ConfigFile::showExperimentalOptions() const
-{
-    auto settings = makeQSettings();
-    return settings.value(showExperimentalOptionsC(), false).toBool();
-}
-
-QString ConfigFile::clientVersionString() const
+QString ConfigFile::clientVersionWithBuildNumberString() const
 {
     auto settings = makeQSettings();
     return settings.value(clientVersionC(), QString()).toString();
 }
 
-void ConfigFile::setClientVersionString(const QString &version)
+void ConfigFile::setClientVersionWithBuildNumberString(const QString &version)
 {
     auto settings = makeQSettings();
     settings.setValue(clientVersionC(), version);
