@@ -13,36 +13,29 @@
  * for more details.
  */
 #include "creds/httpcredentials.h"
-#include "creds/httpcredentials_p.h"
 
-#include "account.h"
 #include "accessmanager.h"
+#include "account.h"
 #include "configfile.h"
-#include "theme.h"
-#include "syncengine.h"
-#include "oauth.h"
-#include "creds/credentialscommon.h"
 #include "creds/credentialmanager.h"
+#include "oauth.h"
+#include "syncengine.h"
 
 #include <QAuthenticator>
-#include <QBuffer>
-#include <QJsonDocument>
 #include <QJsonObject>
 #include <QLoggingCategory>
-#include <QMutex>
+#include <QNetworkInformation>
 #include <QNetworkReply>
-#include <QSettings>
-#include <QSslKey>
 
 #include <chrono>
 
 using namespace std::chrono_literals;
 
 Q_LOGGING_CATEGORY(lcHttpCredentials, "sync.credentials.http", QtInfoMsg)
-Q_LOGGING_CATEGORY(lcHttpLegacyCredentials, "sync.credentials.http.legacy", QtInfoMsg)
 
 namespace {
 constexpr int TokenRefreshMaxRetries = 3;
+constexpr std::chrono::seconds TokenRefreshDefaultTimeout = 30s;
 constexpr int CredentialVersion = 1;
 const char authenticationFailedC[] = "owncloud-authentication-failed";
 
@@ -111,7 +104,6 @@ HttpCredentials::HttpCredentials(DetermineAuthTypeJob::AuthType authType, const
     : _user(user)
     , _password(password)
     , _ready(true)
-    , _retryOnKeyChainError(false)
     , _authType(authType)
 {
 }
@@ -188,15 +180,12 @@ void HttpCredentials::fetchFromKeychain()
 
 void HttpCredentials::fetchFromKeychainHelper()
 {
-    Q_ASSERT(!_user.isEmpty());
-    const int version = _account->credentialSetting(CredentialVersionKey()).toInt();
-    if (version < CredentialVersion && !_credentialMigration) {
-        auto migration = new HttpLegacyCredentials(this);
-        _credentialMigration = migration;
-        migration->fetchFromKeychainHelper();
+    if (_user.isEmpty()) {
+        _password.clear();
+        _ready = false;
+        Q_EMIT fetched();
         return;
     }
-
     auto job = _account->credentialManager()->get(isUsingOAuth() ? refreshTokenKeyC() : passwordKeyC());
     connect(job, &CredentialJob::finished, this, [job, this] {
         auto handleError = [job, this] {
@@ -209,7 +198,7 @@ void HttpCredentials::fetchFromKeychainHelper()
 
             _password.clear();
             _ready = false;
-            emit fetched();
+            Q_EMIT fetched();
         };
         if (job->error() != QKeychain::NoError) {
             handleError();
@@ -223,7 +212,7 @@ void HttpCredentials::fetchFromKeychainHelper()
             } else {
                 _password = data;
                 _ready = true;
-                emit fetched();
+                Q_EMIT fetched();
             }
         } else {
             handleError();
@@ -236,7 +225,7 @@ bool HttpCredentials::stillValid(QNetworkReply *reply)
     if (isUsingOAuth()) {
         // The function is called in order to determine whether we need to ask the user for a password
         // if we are using OAuth, we already started a refresh in slotAuthentication, at least in theory, ensure the auth is started.
-        // If the refresh fails, we are going to emit authenticationFailed ourselves
+        // If the refresh fails, we are going to Q_EMIT authenticationFailed ourselves
         if (reply->error() == QNetworkReply::AuthenticationRequiredError) {
             slotAuthentication(reply, nullptr);
         }
@@ -285,29 +274,47 @@ bool HttpCredentials::refreshAccessTokenInternal(int tokenRefreshRetriesCount)
     _oAuthJob = new AccountBasedOAuth(_account->sharedFromThis(), _account->accessManager());
     connect(_oAuthJob, &AccountBasedOAuth::refreshError, this, [tokenRefreshRetriesCount, this](QNetworkReply::NetworkError error, const QString &) {
         _oAuthJob->deleteLater();
+
+        auto networkUnavailable = []() {
+            if (auto qni = QNetworkInformation::instance()) {
+                if (qni->reachability() == QNetworkInformation::Reachability::Disconnected) {
+                    return true;
+                }
+            }
+
+            return false;
+        };
+
         int nextTry = tokenRefreshRetriesCount + 1;
         std::chrono::seconds timeout = {};
-        switch (error) {
-        case QNetworkReply::ContentNotFoundError:
-            // 404: bigip f5?
-            timeout = 0s;
-            break;
-        case QNetworkReply::HostNotFoundError:
-            [[fallthrough]];
-        case QNetworkReply::TimeoutError:
-            [[fallthrough]];
-        // Qt reports OperationCanceledError if the request timed out
-        case QNetworkReply::OperationCanceledError:
-            [[fallthrough]];
-        case QNetworkReply::TemporaryNetworkFailureError:
-            [[fallthrough]];
-        // VPN not ready?
-        case QNetworkReply::ConnectionRefusedError:
+
+        if (networkUnavailable()) {
             nextTry = 0;
-            [[fallthrough]];
-        default:
-            timeout = 30s;
+            timeout = TokenRefreshDefaultTimeout;
+        } else {
+            switch (error) {
+            case QNetworkReply::ContentNotFoundError:
+                // 404: bigip f5?
+                timeout = 0s;
+                break;
+            case QNetworkReply::HostNotFoundError:
+                [[fallthrough]];
+            case QNetworkReply::TimeoutError:
+                [[fallthrough]];
+            // Qt reports OperationCanceledError if the request timed out
+            case QNetworkReply::OperationCanceledError:
+                [[fallthrough]];
+            case QNetworkReply::TemporaryNetworkFailureError:
+                [[fallthrough]];
+            // VPN not ready?
+            case QNetworkReply::ConnectionRefusedError:
+                nextTry = 0;
+                [[fallthrough]];
+            default:
+                timeout = TokenRefreshDefaultTimeout;
+            }
         }
+
         if (nextTry >= TokenRefreshMaxRetries) {
             qCWarning(lcHttpCredentials) << "Too many failed refreshes" << nextTry << "-> log out";
             forgetSensitiveData();
@@ -336,7 +343,7 @@ bool HttpCredentials::refreshAccessTokenInternal(int tokenRefreshRetriesCount)
             _password = accessToken;
             persist();
         }
-        emit fetched();
+        Q_EMIT fetched();
     });
     Q_EMIT authenticationStarted();
     _oAuthJob->refreshAuthentication(_refreshToken);
@@ -346,6 +353,8 @@ bool HttpCredentials::refreshAccessTokenInternal(int tokenRefreshRetriesCount)
 
 void HttpCredentials::invalidateToken()
 {
+    qCWarning(lcHttpCredentials) << "Invalidating the credentials";
+
     if (!_password.isEmpty()) {
         _previousPassword = _password;
     }
