@@ -25,14 +25,12 @@ namespace OCC {
 
 class OWNCLOUDSYNC_EXPORT DetermineAuthTypeJobFactory : public AbstractCoreJobFactory
 {
-    Q_OBJECT
 public:
     using AuthType = DetermineAuthTypeJob::AuthType;
 
-    DetermineAuthTypeJobFactory(QNetworkAccessManager *nam, QObject *parent = nullptr);
+    DetermineAuthTypeJobFactory(QNetworkAccessManager *nam);
     ~DetermineAuthTypeJobFactory() override;
 
-    CoreJob *startJob(const QUrl &url) override;
+    CoreJob *startJob(const QUrl &url, QObject *parent) override;
 };
-
 }
