@@ -30,8 +30,7 @@
 #include "propagatorjobs.h"
 
 #ifdef Q_OS_WIN
-#include <winbase.h>
-#include <windef.h>
+#include "common/utility_win.h"
 #endif
 
 #include <QApplication>
@@ -41,7 +40,6 @@
 #include <QObject>
 #include <QStack>
 #include <QTimer>
-#include <QTimerEvent>
 #include <qmath.h>
 
 using namespace std::chrono_literals;
@@ -111,12 +109,12 @@ PropagateItemJob::~PropagateItemJob()
 
 bool PropagateItemJob::scheduleSelfOrChild()
 {
-    if (_state != NotYetStarted) {
+    if (state() != NotYetStarted) {
         return false;
     }
-    qCInfo(lcPropagator) << "Starting" << _item->_instruction << "propagation of" << _item->destination() << "by" << this;
+    qCInfo(lcPropagator) << "Starting propagation of" << _item << "by" << this;
 
-    _state = Running;
+    setState(Running);
     if (thread() != QApplication::instance()->thread()) {
         QMetaObject::invokeMethod(this, &PropagateItemJob::start); // We could be in a different thread (neon jobs)
     } else {
@@ -228,8 +226,8 @@ static void blacklistUpdate(SyncJournalDb *journal, SyncFileItem &item)
 void PropagateItemJob::done(SyncFileItem::Status statusArg, const QString &errorString)
 {
     // Duplicate calls to done() are a logic error
-    OC_ENFORCE(_state != Finished);
-    _state = Finished;
+    OC_ENFORCE(state() != Finished);
+    setState(Finished);
 
     _item->_status = statusArg;
 
@@ -255,14 +253,19 @@ void PropagateItemJob::done(SyncFileItem::Status statusArg, const QString &error
     // Blacklist handling
     switch (_item->_status) {
     case SyncFileItem::SoftError:
+        [[fallthrough]];
     case SyncFileItem::FatalError:
+        [[fallthrough]];
     case SyncFileItem::NormalError:
+        [[fallthrough]];
     case SyncFileItem::DetailError:
+        [[fallthrough]];
     case SyncFileItem::Message:
         // Check the blacklist, possibly adjusting the item (including its status)
         blacklistUpdate(propagator()->_journal, *_item);
         break;
     case SyncFileItem::Success:
+        [[fallthrough]];
     case SyncFileItem::Restoration:
         if (_item->_hasBlacklistEntry) {
             // wipe blacklist entry.
@@ -274,10 +277,16 @@ void PropagateItemJob::done(SyncFileItem::Status statusArg, const QString &error
         }
         break;
     case SyncFileItem::Conflict:
+        [[fallthrough]];
     case SyncFileItem::FileIgnored:
+        [[fallthrough]];
     case SyncFileItem::NoStatus:
+        [[fallthrough]];
     case SyncFileItem::BlacklistedError:
+        [[fallthrough]];
     case SyncFileItem::Excluded:
+        [[fallthrough]];
+    case SyncFileItem::FilenameReserved:
         // nothing
         break;
     case SyncFileItem::StatusCount:
@@ -294,13 +303,13 @@ void PropagateItemJob::done(SyncFileItem::Status statusArg, const QString &error
         // we are either not a directory or we are a PropagateDirectory job
         // and an actual instruction was performed for this directory.
         Q_ASSERT(!_item->_relevantDirectoyInstruction || qobject_cast<PropagateDirectory *>(this));
-        emit propagator()->itemCompleted(_item);
+        Q_EMIT propagator()->itemCompleted(_item);
     } else {
         // the directoy needs to call done() in PropagateDirectory::slotSubJobsFinished
         // we don't notify itemCompleted yet as the directory is only complete once its child items are complete.
         _item->_relevantDirectoyInstruction = true;
     }
-    emit finished(_item->_status);
+    Q_EMIT finished(_item->_status);
     if (_item->_status == SyncFileItem::FatalError) {
         // Abort all remaining jobs.
         propagator()->abort();
@@ -311,13 +320,15 @@ void PropagateItemJob::done(SyncFileItem::Status statusArg, const QString &error
 
 PropagateItemJob *OwncloudPropagator::createJob(const SyncFileItemPtr &item)
 {
-    bool deleteExisting = item->_instruction == CSYNC_INSTRUCTION_TYPE_CHANGE;
-    switch (item->_instruction) {
+    qCDebug(lcPropagator) << "Propagating:" << item;
+    const bool deleteExisting = item->instruction() == CSYNC_INSTRUCTION_TYPE_CHANGE;
+    switch (item->instruction()) {
     case CSYNC_INSTRUCTION_REMOVE:
-        if (item->_direction == SyncFileItem::Down)
+        if (item->_direction == SyncFileItem::Down) {
             return new PropagateLocalRemove(this, item);
-        else
+        } else {
             return new PropagateRemoteDelete(this, item);
+        }
     case CSYNC_INSTRUCTION_NEW:
     case CSYNC_INSTRUCTION_TYPE_CHANGE:
     case CSYNC_INSTRUCTION_CONFLICT:
@@ -370,9 +381,6 @@ PropagateItemJob *OwncloudPropagator::createJob(const SyncFileItemPtr &item)
     case CSYNC_INSTRUCTION_ERROR:
         return new PropagateIgnoreJob(this, item);
     case CSYNC_INSTRUCTION_NONE:
-    case CSYNC_INSTRUCTION_STAT_ERROR:
-    case CSYNC_INSTRUCTION_EVAL_RENAME:
-    case CSYNC_INSTRUCTION_EVAL:
         return nullptr;
     }
     Q_UNREACHABLE();
@@ -433,11 +441,11 @@ void OwncloudPropagator::start(SyncFileItemSet &&items)
     // See the `else` statment in the second step.
     QString maybeConflictDirectory;
 
-    for (const auto &item : qAsConst(items)) {
+    for (const auto &item : std::as_const(items)) {
         // First check if this is an item in a directory which is going to be removed.
         if (currentRemoveDirectoryJob && FileSystem::isChildPathOf(item->_file, currentRemoveDirectoryJob->path())) {
             // Check the sync instruction for the item:
-            if (item->_instruction == CSYNC_INSTRUCTION_REMOVE) {
+            if (item->instruction() == CSYNC_INSTRUCTION_REMOVE) {
                 // already taken care of. (by the removal of the parent directory)
 
                 if (auto delDirJob = qobject_cast<PropagateDirectory *>(currentRemoveDirectoryJob)) {
@@ -445,9 +453,7 @@ void OwncloudPropagator::start(SyncFileItemSet &&items)
                     delDirJob->increaseAffectedCount();
                 }
                 continue;
-            } else if (item->isDirectory()
-                && (item->_instruction == CSYNC_INSTRUCTION_NEW
-                       || item->_instruction == CSYNC_INSTRUCTION_TYPE_CHANGE)) {
+            } else if (item->isDirectory() && (item->instruction() & (CSYNC_INSTRUCTION_NEW | CSYNC_INSTRUCTION_TYPE_CHANGE))) {
                 // Create a new directory within a deleted directory? That can happen if the directory
                 // etag was not fetched properly on the previous sync because the sync was aborted
                 // while uploading this directory (which is now removed).  We can ignore it.
@@ -456,13 +462,12 @@ void OwncloudPropagator::start(SyncFileItemSet &&items)
                     delDirJob->increaseAffectedCount();
                 }
                 continue;
-            } else if (item->_instruction == CSYNC_INSTRUCTION_IGNORE) {
+            } else if (item->instruction() & (CSYNC_INSTRUCTION_IGNORE | CSYNC_INSTRUCTION_NONE)) {
                 continue;
-            } else if (item->_instruction != CSYNC_INSTRUCTION_RENAME) {
+            } else if (item->instruction() != CSYNC_INSTRUCTION_RENAME) {
                 // all is good, the rename will be executed before the directory deletion
+                qCWarning(lcPropagator) << "WARNING:  Job within a removed directory?  This should not happen!" << item->_file << item->instruction();
                 Q_ASSERT(false); // we shouldn't land here, but assert for debug purposes
-                qCWarning(lcPropagator) << "WARNING:  Job within a removed directory?  This should not happen!"
-                                        << item->_file << item->_instruction;
             }
         }
 
@@ -472,9 +477,8 @@ void OwncloudPropagator::start(SyncFileItemSet &&items)
         if (!maybeConflictDirectory.isEmpty()) {
             if (FileSystem::isChildPathOf(item->destination(), maybeConflictDirectory)) {
                 // We're processing an item in a CONFLICT directory.
-                qCInfo(lcPropagator) << "Skipping job inside CONFLICT directory"
-                                     << item->_file << item->_instruction;
-                item->_instruction = CSYNC_INSTRUCTION_NONE;
+                qCInfo(lcPropagator) << "Skipping job inside CONFLICT directory" << item->_file << item->instruction();
+                item->setInstruction(CSYNC_INSTRUCTION_NONE);
                 continue;
             } else {
                 // This is not an item inside the conflict directory, which means we're done
@@ -497,22 +501,21 @@ void OwncloudPropagator::start(SyncFileItemSet &&items)
         if (item->isDirectory()) {
             PropagateDirectory *dir = new PropagateDirectory(this, item);
 
-            if (item->_instruction == CSYNC_INSTRUCTION_TYPE_CHANGE
-                && item->_direction == SyncFileItem::Up) {
+            if (item->instruction() == CSYNC_INSTRUCTION_TYPE_CHANGE && item->_direction == SyncFileItem::Up) {
                 // Skip all potential uploads to the new folder.
                 // Processing them now leads to problems with permissions:
                 // checkForPermissions() has already run and used the permissions
                 // of the file we're about to delete to decide whether uploading
                 // to the new dir is ok...
-                for (const auto &item2 : qAsConst(items)) {
+                for (const auto &item2 : std::as_const(items)) {
                     if (item2 != item && FileSystem::isChildPathOf(item2->destination(), item->destination())) {
-                        item2->_instruction = CSYNC_INSTRUCTION_NONE;
+                        item2->setInstruction(CSYNC_INSTRUCTION_NONE);
                         _anotherSyncNeeded = true;
                     }
                 }
             }
 
-            if (item->_instruction == CSYNC_INSTRUCTION_REMOVE) {
+            if (item->instruction() == CSYNC_INSTRUCTION_REMOVE) {
                 // We do the removal of directories at the end, because there might be moves from
                 // these directories that will happen later.
                 currentRemoveDirectoryJob = dir;
@@ -523,8 +526,8 @@ void OwncloudPropagator::start(SyncFileItemSet &&items)
                 // NOTE: Currently this means that we don't update those etag at all in this sync,
                 //       but it should not be a problem, they will be updated in the next sync.
                 for (auto &dir : directories) {
-                    if (dir.second->item()->_instruction == CSYNC_INSTRUCTION_UPDATE_METADATA) {
-                        dir.second->item()->_instruction = CSYNC_INSTRUCTION_NONE;
+                    if (dir.second->item()->instruction() == CSYNC_INSTRUCTION_UPDATE_METADATA) {
+                        dir.second->item()->setInstruction(CSYNC_INSTRUCTION_NONE);
                         _anotherSyncNeeded = true;
                     }
                 }
@@ -535,7 +538,7 @@ void OwncloudPropagator::start(SyncFileItemSet &&items)
             directories.push(qMakePair(item->destination(), dir));
         } else {
             // The item is not a directory, but a file.
-            if (item->_instruction == CSYNC_INSTRUCTION_TYPE_CHANGE) {
+            if (item->instruction() == CSYNC_INSTRUCTION_TYPE_CHANGE) {
                 // will delete directories, so defer execution
                 currentRemoveDirectoryJob = createJob(item);
                 _rootJob->addDeleteJob(currentRemoveDirectoryJob);
@@ -543,7 +546,7 @@ void OwncloudPropagator::start(SyncFileItemSet &&items)
                 directories.top().second->appendTask(item);
             }
 
-            if (item->_instruction == CSYNC_INSTRUCTION_CONFLICT) {
+            if (item->instruction() == CSYNC_INSTRUCTION_CONFLICT) {
                 // This might be a file or a directory on the local side. If it's a
                 // directory we want to skip processing items inside it.
                 maybeConflictDirectory = item->_file;
@@ -567,7 +570,6 @@ Result<QString, bool> OwncloudPropagator::localFileNameClash(const QString &relF
     OC_ASSERT(!relFile.isEmpty());
     if (!relFile.isEmpty() && Utility::fsCasePreserving()) {
         const QFileInfo fileInfo(_localDir + relFile);
-        qCDebug(lcPropagator) << "CaseClashCheck for " << fileInfo.filePath();
 #ifdef Q_OS_MAC
         if (!fileInfo.exists()) {
             return false;
@@ -581,12 +583,10 @@ Result<QString, bool> OwncloudPropagator::localFileNameClash(const QString &relF
         }
 #elif defined(Q_OS_WIN)
         WIN32_FIND_DATA FindFileData;
-        HANDLE hFind;
-
-        hFind = FindFirstFileW(reinterpret_cast<const wchar_t *>(FileSystem::longWinPath(fileInfo.filePath()).utf16()), &FindFileData);
+        const Utility::Handle hFind(FindFirstFileW(reinterpret_cast<const wchar_t *>(FileSystem::longWinPath(fileInfo.filePath()).utf16()), &FindFileData),
+            [](HANDLE h) { FindClose(h); });
         if (hFind != INVALID_HANDLE_VALUE) {
             const QString realFileName = QString::fromWCharArray(FindFileData.cFileName);
-            FindClose(hFind);
 
             if (!fileInfo.filePath().endsWith(realFileName, Qt::CaseSensitive)) {
                 const QString clashName = fileInfo.path() + QLatin1Char('/') + realFileName;
@@ -689,7 +689,7 @@ void OwncloudPropagator::scheduleNextJobImpl()
 
 void OwncloudPropagator::reportFileTotal(const SyncFileItem &item, qint64 newSize)
 {
-    emit updateFileTotal(item, newSize);
+    Q_EMIT updateFileTotal(item, newSize);
 }
 
 void OwncloudPropagator::abort()
@@ -717,7 +717,7 @@ void OwncloudPropagator::abort()
 
 void OwncloudPropagator::reportProgress(const SyncFileItem &item, qint64 bytes)
 {
-    emit progress(item, bytes);
+    Q_EMIT progress(item, bytes);
 }
 
 AccountPtr OwncloudPropagator::account() const
@@ -760,14 +760,12 @@ bool OwncloudPropagator::createConflict(const SyncFileItemPtr &item,
     // If the file is locked, we want to retry this sync when it
     // becomes available again.
     if (FileSystem::isFileLocked(fn, FileSystem::LockMode::Exclusive)) {
-        emit seenLockedFile(fn, FileSystem::LockMode::Exclusive);
+        Q_EMIT seenLockedFile(fn, FileSystem::LockMode::Exclusive);
         if (error)
             *error = tr("File %1 is currently in use").arg(fn);
         return false;
     }
 
-    emit touchedFile(fn);
-    emit touchedFile(conflictFilePath);
     if (!FileSystem::rename(fn, conflictFilePath, &renameError)) {
         // If the rename fails, don't replace it.
         if (error)
@@ -799,10 +797,10 @@ bool OwncloudPropagator::createConflict(const SyncFileItemPtr &item,
             conflictItem->_file = conflictFileName;
             conflictItem->_type = ItemTypeFile;
             conflictItem->_direction = SyncFileItem::Up;
-            conflictItem->_instruction = CSYNC_INSTRUCTION_NEW;
+            conflictItem->setInstruction(CSYNC_INSTRUCTION_NEW);
             conflictItem->_modtime = conflictModTime;
             conflictItem->_size = item->_previousSize;
-            emit newItem(conflictItem);
+            Q_EMIT newItem(conflictItem);
             composite->appendTask(conflictItem);
         } else {
             // Directories we can't process in one go. The next sync run
@@ -858,11 +856,16 @@ Result<Vfs::ConvertToPlaceholderResult, QString> OwncloudPropagator::updateMetad
 
 PropagatorJob::PropagatorJob(OwncloudPropagator *propagator, const QString &path)
     : QObject(propagator)
-    , _state(NotYetStarted)
     , _path(path)
+    , _jobState(NotYetStarted)
 {
 }
 
+void PropagatorJob::setState(JobState state)
+{
+    _jobState = state;
+}
+
 OwncloudPropagator *PropagatorJob::propagator() const
 {
     return qobject_cast<OwncloudPropagator *>(parent());
@@ -888,7 +891,7 @@ void PropagatorCompositeJob::slotSubJobAbortFinished()
 
     // Emit abort if last job has been aborted
     if (_abortsCount == 0) {
-        emit abortFinished();
+        Q_EMIT abortFinished();
     }
 }
 
@@ -900,18 +903,18 @@ void PropagatorCompositeJob::appendJob(PropagatorJob *job)
 
 bool PropagatorCompositeJob::scheduleSelfOrChild()
 {
-    if (_state == Finished) {
+    if (state() == Finished) {
         return false;
     }
 
     // Start the composite job
-    if (_state == NotYetStarted) {
-        _state = Running;
+    if (state() == NotYetStarted) {
+        setState(Running);
     }
 
     // Ask all the running composite jobs if they have something new to schedule.
     for (int i = 0; i < _runningJobs.size(); ++i) {
-        OC_ASSERT(_runningJobs.at(i)->_state == Running);
+        OC_ASSERT(_runningJobs.at(i)->state() == Running);
 
         if (possiblyRunNextJob(_runningJobs.at(i))) {
             return true;
@@ -932,7 +935,7 @@ bool PropagatorCompositeJob::scheduleSelfOrChild()
         _tasksToDo.erase(_tasksToDo.begin());
         PropagatorJob *job = propagator()->createJob(nextTask);
         if (!job) {
-            qCWarning(lcDirectory) << "Useless task found for file" << nextTask->destination() << "instruction" << nextTask->_instruction;
+            qCWarning(lcDirectory) << "Useless task found for file" << nextTask->destination() << "instruction" << nextTask->instruction();
             continue;
         }
         appendJob(job);
@@ -994,11 +997,11 @@ void PropagatorCompositeJob::finalize()
 {
     // The propagator will do parallel scheduling and this could be posted
     // multiple times on the event loop, ignore the duplicate calls.
-    if (_state == Finished)
+    if (state() == Finished)
         return;
 
-    _state = Finished;
-    emit finished(_errorPaths.empty() ? SyncFileItem::Success : _errorPaths.last());
+    setState(Finished);
+    Q_EMIT finished(_errorPaths.empty() ? SyncFileItem::Success : _errorPaths.last());
 }
 
 qint64 PropagatorCompositeJob::committedDiskSpace() const
@@ -1023,7 +1026,7 @@ PropagateDirectory::PropagateDirectory(OwncloudPropagator *propagator, const Syn
     , _subJobs(propagator, path())
 {
     if (_firstJob) {
-        connect(_firstJob.data(), &PropagatorJob::finished, this, &PropagateDirectory::slotFirstJobFinished);
+        connect(_firstJob.get(), &PropagatorJob::finished, this, &PropagateDirectory::slotFirstJobFinished);
         _firstJob->setAssociatedComposite(&_subJobs);
     }
     connect(&_subJobs, &PropagatorJob::finished, this, &PropagateDirectory::slotSubJobsFinished);
@@ -1044,19 +1047,19 @@ PropagatorJob::JobParallelism PropagateDirectory::parallelism()
 
 bool PropagateDirectory::scheduleSelfOrChild()
 {
-    if (_state == Finished) {
+    if (state() == Finished) {
         return false;
     }
 
-    if (_state == NotYetStarted) {
-        _state = Running;
+    if (state() == NotYetStarted) {
+        setState(Running);
     }
 
-    if (_firstJob && _firstJob->_state == NotYetStarted) {
+    if (_firstJob && _firstJob->state() == NotYetStarted) {
         return _firstJob->scheduleSelfOrChild();
     }
 
-    if (_firstJob && _firstJob->_state == Running) {
+    if (_firstJob && _firstJob->state() == Running) {
         // Don't schedule any more job until this is done.
         return false;
     }
@@ -1066,7 +1069,7 @@ bool PropagateDirectory::scheduleSelfOrChild()
 
 void PropagateDirectory::slotFirstJobFinished(SyncFileItem::Status status)
 {
-    _firstJob.take()->deleteLater();
+    _firstJob.release()->deleteLater();
 
     switch (status) {
     // non critical states
@@ -1078,7 +1081,7 @@ void PropagateDirectory::slotFirstJobFinished(SyncFileItem::Status status)
         break;
     // handle all other cases as errors
     default:
-        if (_state != Finished) {
+        if (state() != Finished) {
             // Synchronously abort
             abort(AbortType::Synchronous);
             done(status);
@@ -1103,13 +1106,12 @@ void PropagateDirectory::slotSubJobsFinished(const SyncFileItem::Status status)
         if (status == SyncFileItem::Success) {
             // If a directory is renamed, recursively delete any stale items
             // that may still exist below the old path.
-            if (_item->_instruction == CSYNC_INSTRUCTION_RENAME
-                && _item->_originalFile != _item->_renameTarget) {
+            if (_item->instruction() == CSYNC_INSTRUCTION_RENAME && _item->_originalFile != _item->_renameTarget) {
                 // TODO: check result but it breaks TestDatabaseError
                 propagator()->_journal->deleteFileRecord(_item->_originalFile, true);
             }
 
-            if (_item->_instruction == CSYNC_INSTRUCTION_NEW && _item->_direction == SyncFileItem::Down) {
+            if (_item->instruction() == CSYNC_INSTRUCTION_NEW && _item->_direction == SyncFileItem::Down) {
                 // special case for local MKDIR, set local directory mtime
                 // (it's not synced later at all, but can be nice to have it set initially)
                 OC_ASSERT(FileSystem::setModTime(propagator()->fullLocalPath(_item->destination()), _item->_modtime));
@@ -1118,7 +1120,7 @@ void PropagateDirectory::slotSubJobsFinished(const SyncFileItem::Status status)
             // the directory has been propagated. Otherwise the directory
             // could appear locally without being added to the database.
             // Additionally we need to convert those folders to placeholders with cfapi vfs.
-            if (_item->_instruction & (CSYNC_INSTRUCTION_RENAME | CSYNC_INSTRUCTION_NEW | CSYNC_INSTRUCTION_UPDATE_METADATA)) {
+            if (_item->instruction() & (CSYNC_INSTRUCTION_RENAME | CSYNC_INSTRUCTION_NEW | CSYNC_INSTRUCTION_UPDATE_METADATA)) {
                 // metatdata changes are relevant
                 _item->_relevantDirectoyInstruction = true;
                 _item->_status = SyncFileItem::Success;
@@ -1141,11 +1143,11 @@ void PropagateDirectory::slotSubJobsFinished(const SyncFileItem::Status status)
 
     // don't call done, we only propagate the state of the child items
     // and we don't want error handling for this folder for an error that happend on a child
-    Q_ASSERT(_state != Finished);
-    _state = Finished;
-    emit finished(status);
+    Q_ASSERT(state() != Finished);
+    setState(Finished);
+    Q_EMIT finished(status);
     if (_item->_relevantDirectoyInstruction) {
-        emit propagator()->itemCompleted(_item);
+        Q_EMIT propagator()->itemCompleted(_item);
     }
 }
 
@@ -1184,12 +1186,12 @@ void PropagateRootDirectory::abort(PropagatorJob::AbortType abortType)
         connect(&_subJobs, &PropagatorCompositeJob::abortFinished, this, [this, abortStatus]() {
             abortStatus->subJobsFinished = true;
             if (abortStatus->subJobsFinished && abortStatus->dirDeletionFinished)
-                emit abortFinished();
+                Q_EMIT abortFinished();
         });
         connect(&_dirDeletionJobs, &PropagatorCompositeJob::abortFinished, this, [this, abortStatus]() {
             abortStatus->dirDeletionFinished = true;
             if (abortStatus->subJobsFinished && abortStatus->dirDeletionFinished)
-                emit abortFinished();
+                Q_EMIT abortFinished();
         });
     }
     _subJobs.abort(abortType);
@@ -1203,7 +1205,7 @@ qint64 PropagateRootDirectory::committedDiskSpace() const
 
 bool PropagateRootDirectory::scheduleSelfOrChild()
 {
-    if (_state == Finished) {
+    if (state() == Finished) {
         return false;
     }
 
@@ -1212,7 +1214,7 @@ bool PropagateRootDirectory::scheduleSelfOrChild()
     }
 
     // Important: Finish _subJobs before scheduling any deletes.
-    if (_subJobs._state != Finished) {
+    if (_subJobs.state() != Finished) {
         return false;
     }
 
@@ -1232,7 +1234,7 @@ void PropagateRootDirectory::slotSubJobsFinished(SyncFileItem::Status status)
     // handle all other cases as errors
     default:
         _status = status;
-        if (_state != Finished) {
+        if (state() != Finished) {
             auto subJob = qobject_cast<PropagatorCompositeJob *>(sender());
             if (OC_ENSURE(subJob)) {
                 const QMap<QString, SyncFileItem::Status> errorPaths = subJob->errorPaths();
@@ -1257,8 +1259,8 @@ void PropagateRootDirectory::slotSubJobsFinished(SyncFileItem::Status status)
 
 void PropagateRootDirectory::slotDirDeletionJobsFinished(SyncFileItem::Status status)
 {
-    _state = Finished;
-    emit finished(_status != SyncFileItem::NoStatus ? _status : status);
+    setState(Finished);
+    Q_EMIT finished(_status != SyncFileItem::NoStatus ? _status : status);
 }
 
 void PropagateRootDirectory::addDeleteJob(PropagatorJob *job)
