@@ -14,15 +14,10 @@
 
 #include "owncloudtheme.h"
 
+#include <QCoreApplication>
+#include <QIcon>
 #include <QString>
 #include <QVariant>
-#include <QPixmap>
-#include <QIcon>
-#include <QCoreApplication>
-
-#include "common/utility.h"
-#include "common/version.h"
-#include "config.h"
 
 namespace OCC {
 
@@ -43,11 +38,22 @@ QColor ownCloudTheme::wizardHeaderTitleColor() const
 
 QIcon ownCloudTheme::wizardHeaderLogo() const
 {
-    return themeUniversalIcon(QStringLiteral("wizard_logo"));
+    return Resources::themeUniversalIcon(QStringLiteral("wizard_logo"));
 }
 
 QIcon ownCloudTheme::aboutIcon() const
 {
-    return themeUniversalIcon(QStringLiteral("oc-image-about"));
+    return Resources::themeUniversalIcon(QStringLiteral("oc-image-about"));
+}
+
+QmlButtonColor ownCloudTheme::primaryButtonColor() const
+{
+    const QColor button("#709cd2");
+    return {button, Qt::white, button.darker()};
+}
+
+QmlButtonColor ownCloudTheme::secondaryButtonColor() const
+{
+    return {"#d4d3d0", Qt::black, QColor(Qt::black).lighter()};
 }
 }
