@@ -28,21 +28,12 @@ class ownCloudTheme : public Theme
     Q_OBJECT
 public:
     ownCloudTheme();
-#ifndef TOKEN_AUTH_ONLY
-    QVariant customMedia(CustomMediaType type) override;
-
     QColor wizardHeaderBackgroundColor() const override;
     QColor wizardHeaderTitleColor() const override;
-    QColor wizardHeaderSubTitleColor() const override;
     QIcon wizardHeaderLogo() const override;
     QIcon aboutIcon() const override;
-#endif
-
-    // For owncloud-brandings *do* show the virtual files option.
-    bool showVirtualFilesOption() const override { return true; }
-    bool enableExperimentalFeatures() const override { return true; };
-
-private:
+    QmlButtonColor primaryButtonColor() const override;
+    QmlButtonColor secondaryButtonColor() const override;
 };
 }
-#endif // OWNCLOUD_MIRALL_THEME_H
+#endif // OWNCLOUD_THEME_H
