@@ -15,44 +15,30 @@
 
 #include "syncengine.h"
 #include "account.h"
-#include "owncloudpropagator.h"
+#include "common/asserts.h"
+#include "common/syncfilestatus.h"
 #include "common/syncjournaldb.h"
 #include "common/syncjournalfilerecord.h"
-#include "discoveryphase.h"
-#include "creds/abstractcredentials.h"
-#include "common/syncfilestatus.h"
-#include "csync_exclude.h"
-#include "filesystem.h"
-#include "propagateremotedelete.h"
-#include "propagatedownload.h"
-#include "common/asserts.h"
-#include "discovery.h"
 #include "common/vfs.h"
+#include "configfile.h"
+#include "creds/abstractcredentials.h"
+#include "csync_exclude.h"
+#include "discovery.h"
+#include "discoveryphase.h"
+#include "filesystem.h"
+#include "owncloudpropagator.h"
+#include "propagatedownload.h"
+#include "propagateremotedelete.h"
 
-#ifdef Q_OS_WIN
-#include <windows.h>
-#else
-#include <unistd.h>
-#endif
-
-#include <climits>
-#include <assert.h>
 #include <chrono>
 
-#include <QCoreApplication>
-#include <QSslSocket>
 #include <QDir>
 #include <QLoggingCategory>
-#include <QMutexLocker>
-#include <QThread>
+#include <QSslSocket>
 #include <QStringList>
 #include <QTextStream>
 #include <QTime>
 #include <QUrl>
-#include <QSslCertificate>
-#include <QProcess>
-#include <QElapsedTimer>
-#include <qtextcodec.h>
 
 using namespace std::chrono_literals;
 
@@ -60,26 +46,10 @@ namespace OCC {
 
 Q_LOGGING_CATEGORY(lcEngine, "sync.engine", QtInfoMsg)
 
-/** When the client touches a file, block change notifications for this duration (ms)
- *
- * On Linux and Windows the file watcher can't distinguish a change that originates
- * from the client (like a download during a sync operation) and an external change.
- * To work around that, all files the client touches are recorded and file change
- * notifications for these are blocked for some time. This value controls for how
- * long.
- *
- * Reasons this delay can't be very small:
- * - it takes time for the change notification to arrive and to be processed by the client
- * - some time could pass between the client recording that a file will be touched
- *   and its filesystem operation finishing, triggering the notification
- */
-static const std::chrono::milliseconds s_touchedFilesMaxAgeMs(3 * 1000);
-
 // doc in header
-std::chrono::milliseconds SyncEngine::minimumFileAgeForUpload(2000);
+std::chrono::seconds SyncEngine::minimumFileAgeForUpload(2s);
 
-SyncEngine::SyncEngine(AccountPtr account, const QUrl &baseUrl, const QString &localPath,
-    const QString &remotePath, OCC::SyncJournalDb *journal)
+SyncEngine::SyncEngine(AccountPtr account, const QUrl &baseUrl, const QString &localPath, const QString &remotePath, OCC::SyncJournalDb *journal)
     : _account(account)
     , _baseUrl(baseUrl)
     , _needsUpdate(false)
@@ -88,11 +58,8 @@ SyncEngine::SyncEngine(AccountPtr account, const QUrl &baseUrl, const QString &l
     , _remotePath(remotePath)
     , _journal(journal)
     , _progressInfo(new ProgressInfo)
-    , _hasNoneFiles(false)
-    , _hasRemoveFile(false)
     , _uploadLimit(0)
     , _downloadLimit(0)
-    , _anotherSyncNeeded(NoFollowUpSync)
 {
     qRegisterMetaType<SyncFileItem>("SyncFileItem");
     qRegisterMetaType<SyncFileItemPtr>("SyncFileItemPtr");
@@ -107,16 +74,12 @@ SyncEngine::SyncEngine(AccountPtr account, const QUrl &baseUrl, const QString &l
     _excludedFiles.reset(new ExcludedFiles);
 
     _syncFileStatusTracker.reset(new SyncFileStatusTracker(this));
-
-    _clearTouchedFilesTimer.setSingleShot(true);
-    _clearTouchedFilesTimer.setInterval(30s);
-    connect(&_clearTouchedFilesTimer, &QTimer::timeout, this, &SyncEngine::slotClearTouchedFiles);
 }
 
 SyncEngine::~SyncEngine()
 {
     _goingDown = true;
-    abort();
+    abort(tr("application exit", "abort reason"));
     _excludedFiles.reset();
 }
 
@@ -180,14 +143,14 @@ bool SyncEngine::checkErrorBlacklisting(SyncFileItem &item)
     // for reporting and for making sure we don't update the blacklist
     // entry yet.
     // Classification is this _instruction and _status
-    item._instruction = CSYNC_INSTRUCTION_IGNORE;
+    item.setInstruction(CSYNC_INSTRUCTION_IGNORE);
     if (entry._errorCategory == SyncJournalErrorBlacklistRecord::Category::LocalSoftError) {
         item._status = SyncFileItem::SoftError;
         item._errorString = entry._errorString;
     } else {
         item._status = SyncFileItem::BlacklistedError;
 
-        auto waitSecondsStr = Utility::durationToDescriptiveString1(1000 * waitSeconds);
+        auto waitSecondsStr = Utility::durationToDescriptiveString1(std::chrono::seconds(waitSeconds));
         item._errorString = tr("%1 (skipped due to earlier error, trying again in %2)").arg(entry._errorString, waitSecondsStr);
 
         if (entry._errorCategory == SyncJournalErrorBlacklistRecord::Category::InsufficientRemoteStorage) {
@@ -210,9 +173,7 @@ void SyncEngine::deleteStaleDownloadInfos(const SyncFileItemSet &syncItems)
     // Find all downloadinfo paths that we want to preserve.
     QSet<QString> download_file_paths;
     for (const auto &it : syncItems) {
-        if (it->_direction == SyncFileItem::Down
-            && it->_type == ItemTypeFile
-            && isFileTransferInstruction(it->_instruction)) {
+        if (it->_direction == SyncFileItem::Down && it->_type == ItemTypeFile && isFileTransferInstruction(it->instruction())) {
             download_file_paths.insert(it->_file);
         }
     }
@@ -232,9 +193,7 @@ void SyncEngine::deleteStaleUploadInfos(const SyncFileItemSet &syncItems)
     // Find all blacklisted paths that we want to preserve.
     QSet<QString> upload_file_paths;
     for (const auto &it : syncItems) {
-        if (it->_direction == SyncFileItem::Up
-            && it->_type == ItemTypeFile
-            && isFileTransferInstruction(it->_instruction)) {
+        if (it->_direction == SyncFileItem::Up && it->_type == ItemTypeFile && isFileTransferInstruction(it->instruction())) {
             upload_file_paths.insert(it->_file);
         }
     }
@@ -283,7 +242,7 @@ void SyncEngine::conflictRecordMaintenance()
     //
     // This happens when the conflicts table is new or when conflict files
     // are downlaoded but the server doesn't send conflict headers.
-    for (const auto &path : qAsConst(_seenConflictFiles)) {
+    for (const auto &path : std::as_const(_seenConflictFiles)) {
         OC_ASSERT(Utility::isConflictFile(path));
 
         auto bapath = path.toUtf8();
@@ -309,30 +268,16 @@ void OCC::SyncEngine::slotItemDiscovered(const OCC::SyncFileItemPtr &item)
 {
     if (Utility::isConflictFile(item->_file))
         _seenConflictFiles.insert(item->_file);
-    if (item->_instruction == CSYNC_INSTRUCTION_UPDATE_METADATA && !item->isDirectory()) {
-        _hasNoneFiles = true;
-    } else if (item->_instruction == CSYNC_INSTRUCTION_NONE) {
-        _hasNoneFiles = true;
+    if (item->instruction() == CSYNC_INSTRUCTION_NONE) {
         if (_account->capabilities().uploadConflictFiles() && Utility::isConflictFile(item->_file)) {
             // For uploaded conflict files, files with no action performed on them should
             // be displayed: but we mustn't overwrite the instruction if something happens
             // to the file!
             item->_errorString = tr("Unresolved conflict.");
-            item->_instruction = CSYNC_INSTRUCTION_IGNORE;
+            item->setInstruction(CSYNC_INSTRUCTION_IGNORE);
             item->_status = SyncFileItem::Conflict;
         }
         return;
-    } else if (item->_instruction == CSYNC_INSTRUCTION_REMOVE && !item->_isSelectiveSync) {
-        _hasRemoveFile = true;
-    } else if (item->_instruction == CSYNC_INSTRUCTION_RENAME) {
-        _hasNoneFiles = true; // If a file (or every file) has been renamed, it means not al files where deleted
-    } else if (item->_instruction == CSYNC_INSTRUCTION_TYPE_CHANGE
-        || item->_instruction == CSYNC_INSTRUCTION_SYNC) {
-        if (item->_direction == SyncFileItem::Up) {
-            // An upload of an existing file means that the file was left unchanged on the server
-            // This counts as a NONE for detecting if all the files on the server were changed
-            _hasNoneFiles = true;
-        }
     }
 
     // check for blacklisting of this item.
@@ -344,7 +289,8 @@ void OCC::SyncEngine::slotItemDiscovered(const OCC::SyncFileItemPtr &item)
         const auto it = _syncItems.find(item);
         if (it != _syncItems.cend()) {
             const auto &item2 = it->get();
-            qCWarning(lcEngine) << "We already have an item for " << item2->_file << ":" << item2->_instruction << item2->_direction << "|" << item->_instruction << item->_direction;
+            qCWarning(lcEngine) << "We already have an item for " << item2->_file << ":" << item2->instruction() << item2->_direction << "|"
+                                << item->instruction() << item->_direction;
             return false;
         }
         return true;
@@ -364,19 +310,16 @@ void SyncEngine::startSync()
         OC_ASSERT(false);
         return;
     }
+    _duration.reset();
 
     _syncRunning = true;
-    _anotherSyncNeeded = NoFollowUpSync;
-    _clearTouchedFilesTimer.stop();
+    _anotherSyncNeeded = false;
 
-    _hasNoneFiles = false;
-    _hasRemoveFile = false;
     _seenConflictFiles.clear();
 
     _progressInfo->reset();
 
     if (!QFileInfo::exists(_localPath)) {
-        _anotherSyncNeeded = DelayedFollowUp;
         // No _tr, it should only occur in non-mirall
         Q_EMIT syncError(QStringLiteral("Unable to find local sync folder."));
         finalize(false);
@@ -388,9 +331,8 @@ void SyncEngine::startSync()
     const qint64 freeBytes = Utility::freeDiskSpace(_localPath);
     if (freeBytes >= 0) {
         if (freeBytes < minFree) {
-            qCWarning(lcEngine()) << "Too little space available at" << _localPath << ". Have"
-                                  << freeBytes << "bytes and require at least" << minFree << "bytes";
-            _anotherSyncNeeded = DelayedFollowUp;
+            qCWarning(lcEngine()) << "Too little space available at" << _localPath << ". Have" << freeBytes << "bytes and require at least" << minFree
+                                  << "bytes";
             Q_EMIT syncError(tr("Only %1 are available, need at least %2 to start",
                 "Placeholders are postfixed with file sizes using Utility::octetsToString()")
                                  .arg(
@@ -452,18 +394,16 @@ void SyncEngine::startSync()
         return;
     }
 
-    _stopWatch.start();
-
-    qCInfo(lcEngine) << "#### Discovery start ####################################################";
+    qCInfo(lcEngine) << "#### Discovery start ####################################################" << _duration.duration();
     qCInfo(lcEngine) << "Server" << account()->capabilities().status().versionString()
                      << (account()->isHttp2Supported() ? "Using HTTP/2" : "");
     _progressInfo->_status = ProgressInfo::Discovery;
-    emit transmissionProgress(*_progressInfo);
+    Q_EMIT transmissionProgress(*_progressInfo);
 
     // TODO: add a constructor to DiscoveryPhase
     // pass a syncEngine object rather than copying everyhting to another object
     _discoveryPhase.reset(new DiscoveryPhase(_account, syncOptions(), _baseUrl));
-    _discoveryPhase->_excludes = _excludedFiles.data();
+    _discoveryPhase->_excludes = _excludedFiles.get();
     _discoveryPhase->_statedb = _journal;
     _discoveryPhase->_localDir = _localPath;
     if (!_discoveryPhase->_localDir.endsWith(QLatin1Char('/')))
@@ -483,27 +423,22 @@ void SyncEngine::startSync()
 
     const QString invalidFilenamePattern = _account->capabilities().invalidFilenameRegex();
     if (!invalidFilenamePattern.isEmpty()) {
-        _discoveryPhase->_invalidFilenameRx = QRegExp(invalidFilenamePattern);
+        _discoveryPhase->_invalidFilenameRx = QRegularExpression(invalidFilenamePattern);
     }
     _discoveryPhase->_serverBlacklistedFiles = _account->capabilities().blacklistedFiles();
     _discoveryPhase->_ignoreHiddenFiles = ignoreHiddenFiles();
 
-    connect(_discoveryPhase.data(), &DiscoveryPhase::itemDiscovered, this, &SyncEngine::slotItemDiscovered);
-    connect(_discoveryPhase.data(), &DiscoveryPhase::newBigFolder, this, &SyncEngine::newBigFolder);
-    connect(_discoveryPhase.data(), &DiscoveryPhase::fatalError, this, [this](const QString &errorString) {
+    connect(_discoveryPhase.get(), &DiscoveryPhase::itemDiscovered, this, &SyncEngine::slotItemDiscovered);
+    connect(_discoveryPhase.get(), &DiscoveryPhase::fatalError, this, [this](const QString &errorString) {
         Q_EMIT syncError(errorString);
         finalize(false);
     });
-    connect(_discoveryPhase.data(), &DiscoveryPhase::finished, this, &SyncEngine::slotDiscoveryFinished);
-    connect(_discoveryPhase.data(), &DiscoveryPhase::silentlyExcluded,
-        _syncFileStatusTracker.data(), &SyncFileStatusTracker::slotAddSilentlyExcluded);
-    connect(_discoveryPhase.data(), &DiscoveryPhase::excluded,
-        _syncFileStatusTracker.data(), &SyncFileStatusTracker::slotAddSilentlyExcluded);
-    connect(_discoveryPhase.data(), &DiscoveryPhase::excluded,
-        this, &SyncEngine::excluded);
+    connect(_discoveryPhase.get(), &DiscoveryPhase::finished, this, &SyncEngine::slotDiscoveryFinished);
+    connect(_discoveryPhase.get(), &DiscoveryPhase::silentlyExcluded, _syncFileStatusTracker.data(), &SyncFileStatusTracker::slotAddSilentlyExcluded);
+    connect(_discoveryPhase.get(), &DiscoveryPhase::excluded, _syncFileStatusTracker.data(), &SyncFileStatusTracker::slotAddSilentlyExcluded);
+    connect(_discoveryPhase.get(), &DiscoveryPhase::excluded, this, &SyncEngine::excluded);
 
-    auto discoveryJob = new ProcessDirectoryJob(
-        _discoveryPhase.data(), PinState::AlwaysLocal, _discoveryPhase.data());
+    auto discoveryJob = new ProcessDirectoryJob(_discoveryPhase.get(), PinState::AlwaysLocal, _discoveryPhase.get());
     _discoveryPhase->startJob(discoveryJob);
     connect(discoveryJob, &ProcessDirectoryJob::etag, this, &SyncEngine::slotRootEtagReceived);
 }
@@ -526,7 +461,7 @@ void SyncEngine::slotFolderDiscovered(bool local, const QString &folder)
         _progressInfo->_currentDiscoveredRemoteFolder = folder;
         _progressInfo->_currentDiscoveredLocalFolder.clear();
     }
-    emit transmissionProgress(*_progressInfo);
+    Q_EMIT transmissionProgress(*_progressInfo);
 }
 
 void SyncEngine::slotRootEtagReceived(const QString &e, const QDateTime &time)
@@ -534,7 +469,7 @@ void SyncEngine::slotRootEtagReceived(const QString &e, const QDateTime &time)
     if (_remoteRootEtag.isEmpty()) {
         qCDebug(lcEngine) << "Root etag:" << e;
         _remoteRootEtag = e;
-        emit rootEtag(_remoteRootEtag, time);
+        Q_EMIT rootEtag(_remoteRootEtag, time);
     }
 }
 
@@ -550,7 +485,7 @@ void SyncEngine::slotDiscoveryFinished()
         return;
     }
 
-    qCInfo(lcEngine) << "#### Discovery end #################################################### " << _stopWatch.addLapTime(QStringLiteral("Discovery Finished")) << "ms";
+    qCInfo(lcEngine) << "#### Discovery end ####################################################" << _duration.duration();
 
     // Sanity check
     if (!_journal->open()) {
@@ -566,7 +501,7 @@ void SyncEngine::slotDiscoveryFinished()
     _progressInfo->_currentDiscoveredRemoteFolder.clear();
     _progressInfo->_currentDiscoveredLocalFolder.clear();
     _progressInfo->_status = ProgressInfo::Reconcile;
-    emit transmissionProgress(*_progressInfo);
+    Q_EMIT transmissionProgress(*_progressInfo);
 
     //    qCInfo(lcEngine) << "Permissions of the root folder: " << _csync_ctx->remote.root_perms.toString();
     auto finish = [this]{
@@ -581,21 +516,21 @@ void SyncEngine::slotDiscoveryFinished()
             restoreOldFiles(_syncItems);
         }
 
-        if (_discoveryPhase->_anotherSyncNeeded && _anotherSyncNeeded == NoFollowUpSync) {
-            _anotherSyncNeeded = ImmediateFollowUp;
+        if (_discoveryPhase->_anotherSyncNeeded) {
+            _anotherSyncNeeded = true;
         }
 
         Q_ASSERT(std::is_sorted(_syncItems.begin(), _syncItems.end()));
 
         const auto regex = syncOptions().fileRegex();
         if (regex.isValid()) {
-            QSet<QStringRef> names;
+            QSet<QStringView> names;
             for (auto &i : _syncItems) {
                 if (regex.match(i->_file).hasMatch()) {
                     int index = -1;
-                    QStringRef ref;
+                    QStringView ref;
                     do {
-                        ref = i->_file.midRef(0, index);
+                        ref = QStringView(i->_file).mid(0, index);
                         names.insert(ref);
                         index = ref.lastIndexOf(QLatin1Char('/'));
                     } while (index > 0);
@@ -614,37 +549,23 @@ void SyncEngine::slotDiscoveryFinished()
                 }
                 return old_size - c.size();
             };
-            erase_if(_syncItems, [&names](const SyncFileItemPtr &i) {
-                return !names.contains(QStringRef { &i->_file });
-            });
+            erase_if(_syncItems, [&names](const SyncFileItemPtr &i) { return !names.contains(QStringView{i->_file}); });
         }
 
-        qCInfo(lcEngine) << "#### Reconcile (aboutToPropagate) #################################################### " << _stopWatch.addLapTime(QStringLiteral("Reconcile (aboutToPropagate)")) << "ms";
+        qCInfo(lcEngine) << "#### Reconcile (aboutToPropagate) ####################################################" << _duration.duration();
 
         _localDiscoveryPaths.clear();
 
         // To announce the beginning of the sync
-        emit aboutToPropagate(_syncItems);
+        Q_EMIT aboutToPropagate(_syncItems);
 
-        qCInfo(lcEngine) << "#### Reconcile (aboutToPropagate OK) #################################################### "<< _stopWatch.addLapTime(QStringLiteral("Reconcile (aboutToPropagate OK)")) << "ms";
+        qCInfo(lcEngine) << "#### Reconcile (aboutToPropagate OK) ####################################################" << _duration.duration();
 
         // it's important to do this before ProgressInfo::start(), to announce start of new sync
         _progressInfo->_status = ProgressInfo::Propagation;
-        emit transmissionProgress(*_progressInfo);
+        Q_EMIT transmissionProgress(*_progressInfo);
         _progressInfo->startEstimateUpdates();
 
-        // post update phase script: allow to tweak stuff by a custom script in debug mode.
-        if (!qEnvironmentVariableIsEmpty("OWNCLOUD_POST_UPDATE_SCRIPT")) {
-#ifndef NDEBUG
-            const QString script = qEnvironmentVariable("OWNCLOUD_POST_UPDATE_SCRIPT");
-
-            qCDebug(lcEngine) << "Post Update Script: " << script;
-            QProcess::execute(script, {});
-#else
-            qCWarning(lcEngine) << "**** Attention: POST_UPDATE_SCRIPT installed, but not executed because compiled with NDEBUG";
-#endif
-        }
-
         // do a database commit
         _journal->commit(QStringLiteral("post treewalk"));
 
@@ -657,7 +578,6 @@ void SyncEngine::slotDiscoveryFinished()
             this, &SyncEngine::updateFileTotal);
         connect(_propagator.data(), &OwncloudPropagator::finished, this, &SyncEngine::slotPropagationFinished, Qt::QueuedConnection);
         connect(_propagator.data(), &OwncloudPropagator::seenLockedFile, this, &SyncEngine::seenLockedFile);
-        connect(_propagator.data(), &OwncloudPropagator::touchedFile, this, &SyncEngine::slotAddTouchedFile);
         connect(_propagator.data(), &OwncloudPropagator::insufficientLocalStorage, this, &SyncEngine::slotInsufficientLocalStorage);
         connect(_propagator.data(), &OwncloudPropagator::insufficientRemoteStorage, this, &SyncEngine::slotInsufficientRemoteStorage);
         connect(_propagator.data(), &OwncloudPropagator::newItem, this, &SyncEngine::slotNewItem);
@@ -676,38 +596,10 @@ void SyncEngine::slotDiscoveryFinished()
 
         _propagator->start(std::move(_syncItems));
 
-        qCInfo(lcEngine) << "#### Post-Reconcile end #################################################### " << _stopWatch.addLapTime(QStringLiteral("Post-Reconcile Finished")) << "ms";
+
+        qCInfo(lcEngine) << "#### Post-Reconcile end ####################################################" << _duration.duration();
     };
 
-    if (!_hasNoneFiles && _hasRemoveFile) {
-        qCInfo(lcEngine) << "All the files are going to be changed, asking the user";
-        int side = 0; // > 0 means more deleted on the server.  < 0 means more deleted on the client
-        for (const auto &it : qAsConst(_syncItems)) {
-            if (it->_instruction == CSYNC_INSTRUCTION_REMOVE) {
-                side += it->_direction == SyncFileItem::Down ? 1 : -1;
-            }
-        }
-
-        QPointer<QObject> guard = new QObject();
-        QPointer<QObject> self = this;
-        auto callback = [this, self, finish, guard](bool cancel) -> void {
-            // use a guard to ensure its only called once...
-            // qpointer to self to ensure we still exist
-            if (!guard || !self) {
-                return;
-            }
-            guard->deleteLater();
-            if (cancel) {
-                qCInfo(lcEngine) << "User aborted sync";
-                finalize(false);
-                return;
-            } else {
-                finish();
-            }
-        };
-        emit aboutToRemoveAllFiles(side >= 0 ? SyncFileItem::Down : SyncFileItem::Up, callback);
-        return;
-    }
     finish();
 }
 
@@ -723,7 +615,7 @@ void SyncEngine::setNetworkLimits(int upload, int download)
                 _propagator->_bandwidthManager = new BandwidthManager(_propagator.data());
             }
         }
-        // this might set the limit to 0 but only the next sync will have no bandwithd manager set
+        // this might set the limit to 0 but only the next sync will have no bandwidth manager set
         if (_propagator->_bandwidthManager) {
             _propagator->_bandwidthManager->setCurrentDownloadLimit(download);
             _propagator->_bandwidthManager->setCurrentUploadLimit(upload);
@@ -742,14 +634,14 @@ void SyncEngine::slotItemCompleted(const SyncFileItemPtr &item)
 
     _progressInfo->setProgressComplete(*item);
 
-    emit transmissionProgress(*_progressInfo);
-    emit itemCompleted(item);
+    Q_EMIT transmissionProgress(*_progressInfo);
+    Q_EMIT itemCompleted(item);
 }
 
 void SyncEngine::slotPropagationFinished(bool success)
 {
-    if (_propagator->_anotherSyncNeeded && _anotherSyncNeeded == NoFollowUpSync) {
-        _anotherSyncNeeded = ImmediateFollowUp;
+    if (_propagator->_anotherSyncNeeded) {
+        _anotherSyncNeeded = true;
     }
 
     if (success && _discoveryPhase) {
@@ -772,21 +664,21 @@ void SyncEngine::slotPropagationFinished(bool success)
     // so we don't count this twice (like Recent Files)
     _progressInfo->_lastCompletedItem = SyncFileItem();
     _progressInfo->_status = ProgressInfo::Done;
-    emit transmissionProgress(*_progressInfo);
+    Q_EMIT transmissionProgress(*_progressInfo);
 
     finalize(success);
 }
 
 void SyncEngine::finalize(bool success)
 {
-    qCInfo(lcEngine) << "Sync run took " << _stopWatch.addLapTime(QStringLiteral("Sync Finished")) << "ms";
-    _stopWatch.stop();
+    qCInfo(lcEngine) << "Sync run took" << _duration.duration();
+    _duration.stop();
 
     if (_discoveryPhase) {
-        _discoveryPhase.take()->deleteLater();
+        _discoveryPhase.release()->deleteLater();
     }
     _syncRunning = false;
-    emit finished(success);
+    Q_EMIT finished(success);
 
     // Delete the propagator only after emitting the signal.
     _propagator.clear();
@@ -794,20 +686,18 @@ void SyncEngine::finalize(bool success)
     _uniqueErrors.clear();
     _localDiscoveryPaths.clear();
     _localDiscoveryStyle = LocalDiscoveryStyle::FilesystemOnly;
-
-    _clearTouchedFilesTimer.start();
 }
 
 void SyncEngine::slotProgress(const SyncFileItem &item, qint64 current)
 {
     _progressInfo->setProgressItem(item, current);
-    emit transmissionProgress(*_progressInfo);
+    Q_EMIT transmissionProgress(*_progressInfo);
 }
 
 void SyncEngine::updateFileTotal(const SyncFileItem &item, qint64 newSize)
 {
     _progressInfo->updateTotalsForFile(item, newSize);
-    emit transmissionProgress(*_progressInfo);
+    Q_EMIT transmissionProgress(*_progressInfo);
 }
 void SyncEngine::restoreOldFiles(SyncFileItemSet &syncItems)
 {
@@ -821,14 +711,14 @@ void SyncEngine::restoreOldFiles(SyncFileItemSet &syncItems)
         if ((*it)->_direction != SyncFileItem::Down)
             continue;
 
-        switch ((*it)->_instruction) {
+        switch ((*it)->instruction()) {
         case CSYNC_INSTRUCTION_SYNC:
             qCWarning(lcEngine) << "restoreOldFiles: RESTORING" << (*it)->_file;
-            (*it)->_instruction = CSYNC_INSTRUCTION_CONFLICT;
+            (*it)->setInstruction(CSYNC_INSTRUCTION_CONFLICT);
             break;
         case CSYNC_INSTRUCTION_REMOVE:
             qCWarning(lcEngine) << "restoreOldFiles: RESTORING" << (*it)->_file;
-            (*it)->_instruction = CSYNC_INSTRUCTION_NEW;
+            (*it)->setInstruction(CSYNC_INSTRUCTION_NEW);
             (*it)->_direction = SyncFileItem::Up;
             break;
         case CSYNC_INSTRUCTION_RENAME:
@@ -842,49 +732,6 @@ void SyncEngine::restoreOldFiles(SyncFileItemSet &syncItems)
     }
 }
 
-void SyncEngine::slotAddTouchedFile(const QString &fn)
-{
-    QElapsedTimer now;
-    now.start();
-    const QString file = QDir::cleanPath(fn);
-
-    // Iterate from the oldest and remove anything older than 15 seconds.
-    while (true) {
-        // don't use a loop as we do erase in here
-        // don't use remove_if as we can stop as soon as we find the first new item
-        auto start = _touchedFiles.cbegin();
-        if (start == _touchedFiles.cend()) {
-            break;
-        }
-        // Compare to our new QElapsedTimer instead of using elapsed().
-        // This avoids querying the current time from the OS for every loop.
-        const auto elapsed = std::chrono::milliseconds(now.msecsSinceReference() - start->first.msecsSinceReference());
-        if (elapsed <= s_touchedFilesMaxAgeMs) {
-            // We found the first path younger than the maximum age, keep the rest.
-            break;
-        }
-        _touchedFiles.erase(start);
-    }
-
-    // This should be the largest QElapsedTimer yet, use constEnd() as hint.
-    _touchedFiles.insert(_touchedFiles.cend(), { now, file });
-}
-
-void SyncEngine::slotClearTouchedFiles()
-{
-    _touchedFiles.clear();
-}
-
-bool SyncEngine::wasFileTouched(const QString &fn) const
-{
-    // Start from the end (most recent) and look for our path. Check the time just in case.
-    for (auto it = _touchedFiles.crbegin(); it != _touchedFiles.crend(); ++it) {
-        if (it->second == fn)
-            return std::chrono::milliseconds(it->first.elapsed()) <= s_touchedFilesMaxAgeMs;
-    }
-    return false;
-}
-
 AccountPtr SyncEngine::account() const
 {
     return _account;
@@ -948,7 +795,7 @@ bool SyncEngine::shouldDiscoverLocally(const QString &path) const
 
     // Maybe a parent folder of something in the list?
     // check for a prefix + / match
-    forever {
+    while (true) {
         if (it->size() > path.size() && it->at(path.size()) == QLatin1Char('/'))
             return true;
         ++it;
@@ -958,47 +805,23 @@ bool SyncEngine::shouldDiscoverLocally(const QString &path) const
     return false;
 }
 
-void SyncEngine::wipeVirtualFiles(const QString &localPath, SyncJournalDb &journal, Vfs &vfs)
+void SyncEngine::abort(const QString &reason)
 {
-    qCInfo(lcEngine) << "Wiping virtual files inside" << localPath;
-    journal.getFilesBelowPath(QByteArray(), [&](const SyncJournalFileRecord &rec) {
-        if (rec._type != ItemTypeVirtualFile && rec._type != ItemTypeVirtualFileDownload)
-            return;
-
-        qCDebug(lcEngine) << "Removing db record for" << rec._path;
-        journal.deleteFileRecord(QString::fromUtf8(rec._path));
-
-        // If the local file is a dehydrated placeholder, wipe it too.
-        // Otherwise leave it to allow the next sync to have a new-new conflict.
-        QString localFile = localPath + QString::fromUtf8(rec._path);
-        if (QFile::exists(localFile) && vfs.isDehydratedPlaceholder(localFile)) {
-            qCDebug(lcEngine) << "Removing local dehydrated placeholder" << rec._path;
-            FileSystem::remove(localFile);
-        }
-    });
-
-    journal.forceRemoteDiscoveryNextSync();
-
-    // Postcondition: No ItemTypeVirtualFile / ItemTypeVirtualFileDownload left in the db.
-    // But hydrated placeholders may still be around.
-}
-
-void SyncEngine::abort()
-{
-    if (_propagator)
-        qCInfo(lcEngine) << "Aborting sync";
-
+    bool aborting = false;
     if (_propagator) {
+        aborting = true;
         // If we're already in the propagation phase, aborting that is sufficient
         _propagator->abort();
     } else if (_discoveryPhase) {
+        aborting = true;
         // Delete the discovery and all child jobs after ensuring
         // it can't finish and start the propagator
-        disconnect(_discoveryPhase.data(), nullptr, this, nullptr);
-        _discoveryPhase.take()->deleteLater();
-
+        disconnect(_discoveryPhase.get(), nullptr, this, nullptr);
+    }
+    if (aborting) {
+        qCInfo(lcEngine) << "Aborting sync, stated reason:" << reason;
         if (!_goingDown) {
-            Q_EMIT syncError(tr("Aborted"));
+            Q_EMIT syncError(tr("Aborted due to %1").arg(reason));
         }
         finalize(false);
     }
@@ -1010,7 +833,7 @@ void SyncEngine::slotSummaryError(const QString &message)
         return;
 
     _uniqueErrors.insert(message);
-    emit syncError(message, ErrorCategory::Normal);
+    Q_EMIT syncError(message, ErrorCategory::Normal);
 }
 
 void SyncEngine::slotInsufficientLocalStorage()
@@ -1028,7 +851,39 @@ void SyncEngine::slotInsufficientRemoteStorage()
         return;
 
     _uniqueErrors.insert(msg);
-    emit syncError(msg, ErrorCategory::InsufficientRemoteStorage);
+    Q_EMIT syncError(msg, ErrorCategory::InsufficientRemoteStorage);
+}
+
+bool SyncEngine::isExcluded(QStringView filePath) const
+{
+    Q_ASSERT(QDir::isAbsolutePath(filePath.toString()));
+    return _excludedFiles->isExcluded(filePath, localPath(), ignoreHiddenFiles());
+}
+
+bool SyncEngine::loadDefaultExcludes()
+{
+    ConfigFile::setupDefaultExcludeFilePaths(*_excludedFiles);
+    return _excludedFiles->reloadExcludeFiles();
+}
+
+void SyncEngine::clearManualExcludes()
+{
+    _excludedFiles->clearManualExcludes();
+}
+
+bool SyncEngine::reloadExcludes()
+{
+    return _excludedFiles->reloadExcludeFiles();
+}
+
+void SyncEngine::addExcludeList(const QString &filePath)
+{
+    _excludedFiles->addExcludeFilePath(filePath);
+}
+
+void SyncEngine::addManualExclude(const QString &filePath)
+{
+    _excludedFiles->addManualExclude(filePath);
 }
 
 } // namespace OCC
