@@ -14,11 +14,10 @@
  */
 
 #include "syncfilestatustracker.h"
-#include "syncengine.h"
+#include "common/asserts.h"
 #include "common/syncjournaldb.h"
 #include "common/syncjournalfilerecord.h"
-#include "common/asserts.h"
-#include "csync_exclude.h"
+#include "syncengine.h"
 
 #include <QFileInfo>
 #include <QLoggingCategory>
@@ -72,20 +71,14 @@ SyncFileStatus::SyncFileStatusTag SyncFileStatusTracker::lookupProblem(const QSt
 static inline bool hasErrorStatus(const SyncFileItem &item)
 {
     const auto status = item._status;
-    return item._instruction == CSYNC_INSTRUCTION_ERROR
-        || status == SyncFileItem::NormalError
-        || status == SyncFileItem::FatalError
-        || status == SyncFileItem::DetailError
-        || status == SyncFileItem::BlacklistedError
-        || item._hasBlacklistEntry;
+    return item.instruction() == CSYNC_INSTRUCTION_ERROR || status == SyncFileItem::NormalError || status == SyncFileItem::FatalError
+        || status == SyncFileItem::DetailError || status == SyncFileItem::BlacklistedError || item._hasBlacklistEntry;
 }
 
 static inline bool hasExcludedStatus(const SyncFileItem &item)
 {
     const auto status = item._status;
-    return item._instruction == CSYNC_INSTRUCTION_IGNORE
-        || status == SyncFileItem::FileIgnored
-        || status == SyncFileItem::Conflict
+    return item.instruction() == CSYNC_INSTRUCTION_IGNORE || status == SyncFileItem::FileIgnored || status == SyncFileItem::Conflict
         || status == SyncFileItem::Restoration;
 }
 
@@ -125,9 +118,7 @@ SyncFileStatus SyncFileStatusTracker::fileStatus(const QString &relativePath)
     // it's an acceptable compromize to treat all exclude types the same.
     // Update: This extra check shouldn't hurt even though silently excluded files
     // are now available via slotAddSilentlyExcluded().
-    if (_syncEngine->excludedFiles().isExcluded(_syncEngine->syncOptions()._vfs->underlyingFileName(absolutePath),
-            _syncEngine->localPath(),
-            _syncEngine->ignoreHiddenFiles())) {
+    if (_syncEngine->isExcluded(_syncEngine->syncOptions()._vfs->underlyingFileName(absolutePath))) {
         return SyncFileStatus(SyncFileStatus::StatusExcluded);
     }
 
@@ -152,13 +143,13 @@ void SyncFileStatusTracker::slotPathTouched(const QString &fileName)
     QString localPath = fileName.mid(folderPath.size());
     _dirtyPaths.insert(localPath);
 
-    emit fileStatusChanged(fileName, SyncFileStatus::StatusSync);
+    Q_EMIT fileStatusChanged(fileName, SyncFileStatus::StatusSync);
 }
 
 void SyncFileStatusTracker::slotAddSilentlyExcluded(const QString &folderPath)
 {
     _syncProblems[folderPath] = SyncFileStatus::StatusExcluded;
-    emit fileStatusChanged(getSystemDestination(folderPath), resolveSyncAndErrorStatus(folderPath, NotShared));
+    Q_EMIT fileStatusChanged(getSystemDestination(folderPath), resolveSyncAndErrorStatus(folderPath, NotShared));
 }
 
 void SyncFileStatusTracker::incSyncCountAndEmitStatusChanged(const QString &relativePath, SharedFlag sharedFlag)
@@ -169,7 +160,7 @@ void SyncFileStatusTracker::incSyncCountAndEmitStatusChanged(const QString &rela
         SyncFileStatus status = sharedFlag == UnknownShared
             ? fileStatus(relativePath)
             : resolveSyncAndErrorStatus(relativePath, sharedFlag);
-        emit fileStatusChanged(getSystemDestination(relativePath), status);
+        Q_EMIT fileStatusChanged(getSystemDestination(relativePath), status);
 
         // We passed from OK to SYNC, increment the parent to keep it marked as
         // SYNC while we propagate ourselves and our own children.
@@ -192,7 +183,7 @@ void SyncFileStatusTracker::decSyncCountAndEmitStatusChanged(const QString &rela
         SyncFileStatus status = sharedFlag == UnknownShared
             ? fileStatus(relativePath)
             : resolveSyncAndErrorStatus(relativePath, sharedFlag);
-        emit fileStatusChanged(getSystemDestination(relativePath), status);
+        Q_EMIT fileStatusChanged(getSystemDestination(relativePath), status);
 
         // We passed from SYNC to OK, decrement our parent.
         OC_ASSERT(!relativePath.endsWith(QLatin1Char('/')));
@@ -211,8 +202,8 @@ void SyncFileStatusTracker::slotAboutToPropagate(const SyncFileItemSet &items)
     ProblemsMap oldProblems;
     std::swap(_syncProblems, oldProblems);
 
-    for (const auto &item : qAsConst(items)) {
-        qCDebug(lcStatusTracker) << "Investigating" << item->destination() << item->_status << item->_instruction;
+    for (const auto &item : std::as_const(items)) {
+        qCDebug(lcStatusTracker) << "Investigating" << item->destination() << item->_status << item->instruction();
         _dirtyPaths.remove(item->destination());
         _dirtyPaths.remove(item->_originalFile);
 
@@ -224,14 +215,11 @@ void SyncFileStatusTracker::slotAboutToPropagate(const SyncFileItemSet &items)
         }
 
         SharedFlag sharedFlag = item->_remotePerm.hasPermission(RemotePermissions::IsShared) ? Shared : NotShared;
-        if (item->_instruction != CSYNC_INSTRUCTION_NONE
-            && item->_instruction != CSYNC_INSTRUCTION_UPDATE_METADATA
-            && item->_instruction != CSYNC_INSTRUCTION_IGNORE
-            && item->_instruction != CSYNC_INSTRUCTION_ERROR) {
+        if (item->instruction() & ~(CSYNC_INSTRUCTION_NONE | CSYNC_INSTRUCTION_UPDATE_METADATA | CSYNC_INSTRUCTION_IGNORE | CSYNC_INSTRUCTION_ERROR)) {
             // Mark this path as syncing for instructions that will result in propagation.
             incSyncCountAndEmitStatusChanged(item->destination(), sharedFlag);
         } else {
-            emit fileStatusChanged(getSystemDestination(item->destination()), resolveSyncAndErrorStatus(item->destination(), sharedFlag));
+            Q_EMIT fileStatusChanged(getSystemDestination(item->destination()), resolveSyncAndErrorStatus(item->destination(), sharedFlag));
         }
     }
 
@@ -241,7 +229,7 @@ void SyncFileStatusTracker::slotAboutToPropagate(const SyncFileItemSet &items)
     QSet<QString> oldDirtyPaths;
     std::swap(_dirtyPaths, oldDirtyPaths);
     for (auto it = oldDirtyPaths.constBegin(); it != oldDirtyPaths.constEnd(); ++it)
-        emit fileStatusChanged(getSystemDestination(*it), fileStatus(*it));
+        Q_EMIT fileStatusChanged(getSystemDestination(*it), fileStatus(*it));
 
     // Make sure to push any status that might have been resolved indirectly since the last sync
     // (like an error file being deleted from disk)
@@ -252,13 +240,13 @@ void SyncFileStatusTracker::slotAboutToPropagate(const SyncFileItemSet &items)
         SyncFileStatus::SyncFileStatusTag severity = it->second;
         if (severity == SyncFileStatus::StatusError)
             invalidateParentPaths(path);
-        emit fileStatusChanged(getSystemDestination(path), fileStatus(path));
+        Q_EMIT fileStatusChanged(getSystemDestination(path), fileStatus(path));
     }
 }
 
 void SyncFileStatusTracker::slotItemCompleted(const SyncFileItemPtr &item)
 {
-    qCDebug(lcStatusTracker) << "Item completed" << item->destination() << item->_status << item->_instruction;
+    qCDebug(lcStatusTracker) << "Item completed" << item->destination() << item->_status << item->instruction();
 
     if (hasErrorStatus(*item)) {
         _syncProblems[item->destination()] = SyncFileStatus::StatusError;
@@ -270,14 +258,11 @@ void SyncFileStatusTracker::slotItemCompleted(const SyncFileItemPtr &item)
     }
 
     SharedFlag sharedFlag = item->_remotePerm.hasPermission(RemotePermissions::IsShared) ? Shared : NotShared;
-    if (item->_instruction != CSYNC_INSTRUCTION_NONE
-        && item->_instruction != CSYNC_INSTRUCTION_UPDATE_METADATA
-        && item->_instruction != CSYNC_INSTRUCTION_IGNORE
-        && item->_instruction != CSYNC_INSTRUCTION_ERROR) {
+    if (item->instruction() & ~(CSYNC_INSTRUCTION_NONE | CSYNC_INSTRUCTION_UPDATE_METADATA | CSYNC_INSTRUCTION_IGNORE | CSYNC_INSTRUCTION_ERROR)) {
         // decSyncCount calls *must* be symetric with incSyncCount calls in slotAboutToPropagate
         decSyncCountAndEmitStatusChanged(item->destination(), sharedFlag);
     } else {
-        emit fileStatusChanged(getSystemDestination(item->destination()), resolveSyncAndErrorStatus(item->destination(), sharedFlag));
+        Q_EMIT fileStatusChanged(getSystemDestination(item->destination()), resolveSyncAndErrorStatus(item->destination(), sharedFlag));
     }
 }
 
@@ -287,12 +272,12 @@ void SyncFileStatusTracker::slotSyncFinished()
     QHash<QString, int> oldSyncCount;
     std::swap(_syncCount, oldSyncCount);
     for (auto it = oldSyncCount.begin(); it != oldSyncCount.end(); ++it)
-        emit fileStatusChanged(getSystemDestination(it.key()), fileStatus(it.key()));
+        Q_EMIT fileStatusChanged(getSystemDestination(it.key()), fileStatus(it.key()));
 }
 
 void SyncFileStatusTracker::slotSyncEngineRunningChanged()
 {
-    emit fileStatusChanged(getSystemDestination(QString()), resolveSyncAndErrorStatus(QString(), NotShared));
+    Q_EMIT fileStatusChanged(getSystemDestination(QString()), resolveSyncAndErrorStatus(QString(), NotShared));
 }
 
 SyncFileStatus SyncFileStatusTracker::resolveSyncAndErrorStatus(const QString &relativePath, SharedFlag sharedFlag, PathKnownFlag isPathKnown)
@@ -323,7 +308,7 @@ void SyncFileStatusTracker::invalidateParentPaths(const QString &path)
     QStringList splitPath = path.split(QLatin1Char('/'), Qt::SkipEmptyParts);
     for (int i = 0; i < splitPath.size(); ++i) {
         QString parentPath = QStringList(splitPath.mid(0, i)).join(QLatin1Char('/'));
-        emit fileStatusChanged(getSystemDestination(parentPath), fileStatus(parentPath));
+        Q_EMIT fileStatusChanged(getSystemDestination(parentPath), fileStatus(parentPath));
     }
 }
 
