@@ -20,29 +20,29 @@
 #include "config.h"
 #include "configfile.h"
 
+#include "resources/qmlresources.h"
 #include "resources/resources.h"
 
+#include <QSslSocket>
+#include <QStyle>
 #include <QtCore>
 #include <QtGui>
-#include <QStyle>
-#include <QApplication>
-#include <QSslSocket>
 
-#include "owncloudtheme.h"
+#include "themewatcher.h"
 
 #ifdef THEME_INCLUDE
 #include THEME_INCLUDE
 #endif
 
 namespace {
-QString vanillaThemePath()
+QString whiteTheme()
 {
-    return QStringLiteral(":/client/ownCloud/theme");
+    return QStringLiteral("white");
 }
 
-QString brandThemePath()
+QString blackTheme()
 {
-    return QStringLiteral(":/client/" APPLICATION_SHORTNAME "/theme");
+    return QStringLiteral("black");
 }
 
 QString darkTheme()
@@ -54,78 +54,44 @@ QString coloredTheme()
 {
     return QStringLiteral("colored");
 }
-
-QString whiteTheme()
-{
-    return QStringLiteral("white");
-}
-
-QString blackTheme()
-{
-    return QStringLiteral("black");
-}
-
-constexpr bool isVanilla()
-{
-    return std::string_view(APPLICATION_SHORTNAME) == "ownCloud";
-}
 }
 namespace OCC {
 
 Theme *Theme::_instance = nullptr;
 
+QmlUrlButton::QmlUrlButton() { }
+
+QmlUrlButton::QmlUrlButton(const std::tuple<QString, QString, QUrl> &tuple)
+    : icon(QStringLiteral("urlIcons/%1").arg(std::get<0>(tuple)))
+    , name(std::get<1>(tuple))
+    , url(std::get<2>(tuple))
+{
+}
+
+bool QmlButtonColor::valid() const
+{
+    return color.isValid() && textColor.isValid() && textColorDisabled.isValid();
+}
+
 Theme *Theme::instance()
 {
     if (!_instance) {
         _instance = new THEME_CLASS;
+        auto *watcher = new Resources::ThemeWatcher(_instance);
+        connect(watcher, &Resources::ThemeWatcher::themeChanged, _instance, &Theme::themeChanged);
     }
     return _instance;
 }
 
-Theme::~Theme()
+Theme *Theme::create(QQmlEngine *qmlEngine, QJSEngine *)
 {
+    Q_ASSERT(qmlEngine->thread() == Theme::instance()->thread());
+    QJSEngine::setObjectOwnership(Theme::instance(), QJSEngine::CppOwnership);
+    return instance();
 }
 
-QString Theme::statusHeaderText(SyncResult::Status status) const
+Theme::~Theme()
 {
-    QString resultStr;
-
-    switch (status) {
-    case SyncResult::Undefined:
-        resultStr = QCoreApplication::translate("theme", "Status undefined");
-        break;
-    case SyncResult::NotYetStarted:
-        resultStr = QCoreApplication::translate("theme", "Waiting to start sync");
-        break;
-    case SyncResult::SyncRunning:
-        resultStr = QCoreApplication::translate("theme", "Sync is running");
-        break;
-    case SyncResult::Success:
-        resultStr = QCoreApplication::translate("theme", "Sync Success");
-        break;
-    case SyncResult::Problem:
-        resultStr = QCoreApplication::translate("theme", "Sync Success, some files were ignored.");
-        break;
-    case SyncResult::Error:
-        resultStr = QCoreApplication::translate("theme", "Sync Error");
-        break;
-    case SyncResult::SetupError:
-        resultStr = QCoreApplication::translate("theme", "Setup Error");
-        break;
-    case SyncResult::SyncPrepare:
-        resultStr = QCoreApplication::translate("theme", "Preparing to sync");
-        break;
-    case SyncResult::SyncAbortRequested:
-        resultStr = QCoreApplication::translate("theme", "Aborting...");
-        break;
-    case SyncResult::Paused:
-        resultStr = QCoreApplication::translate("theme", "Sync is paused");
-        break;
-    case SyncResult::Offline:
-        resultStr = QCoreApplication::translate("theme", "Offline");
-        break;
-    }
-    return resultStr;
 }
 
 QString Theme::appNameGUI() const
@@ -138,6 +104,21 @@ QString Theme::appName() const
     return QStringLiteral(APPLICATION_SHORTNAME);
 }
 
+QString Theme::appDotVirtualFileSuffix() const
+{
+    return QStringLiteral(APPLICATION_DOTVIRTUALFILE_SUFFIX);
+}
+
+QString Theme::orgDomainName() const
+{
+    return QStringLiteral(APPLICATION_REV_DOMAIN);
+}
+
+QString Theme::vendor() const
+{
+    return QStringLiteral(APPLICATION_VENDOR);
+}
+
 QString Theme::configFileName() const
 {
     return QStringLiteral(APPLICATION_EXECUTABLE ".cfg");
@@ -145,7 +126,12 @@ QString Theme::configFileName() const
 
 QIcon Theme::applicationIcon() const
 {
-    return themeUniversalIcon(QStringLiteral(APPLICATION_ICON_NAME "-icon"));
+    return Resources::themeUniversalIcon(applicationIconName() + QStringLiteral("-icon"));
+}
+
+QString Theme::applicationIconName() const
+{
+    return QStringLiteral(APPLICATION_ICON_NAME);
 }
 
 QIcon Theme::aboutIcon() const
@@ -153,20 +139,25 @@ QIcon Theme::aboutIcon() const
     return applicationIcon();
 }
 
-bool Theme::allowDarkTheme() const
+QIcon Theme::themeTrayIcon(const SyncResult &result, [[maybe_unused]] bool sysTrayMenuVisible, Resources::IconType iconType) const
 {
-    return _hasBrandedColored == _hasBrandedDark;
-}
+    auto systrayIconFlavor = [&]() {
+        QString flavor;
+        if (_mono) {
+            flavor = Utility::hasDarkSystray() ? whiteTheme() : blackTheme();
 
-
-QIcon Theme::themeUniversalIcon(const QString &name, Theme::IconType iconType) const
-{
-    return loadIcon(QStringLiteral("universal"), name, iconType);
-}
-
-QIcon Theme::themeTrayIcon(const QString &name, bool sysTrayMenuVisible, IconType iconType) const
-{
-    auto icon = loadIcon(systrayIconFlavor(_mono, sysTrayMenuVisible), name, iconType);
+#ifdef Q_OS_MAC
+            if (sysTrayMenuVisible) {
+                flavor = whiteTheme();
+            }
+#endif
+        } else {
+            // we have a dark sys tray and the theme has support for that
+            flavor = (Utility::hasDarkSystray() && Resources::hasDarkTheme()) ? darkTheme() : coloredTheme();
+        }
+        return flavor;
+    };
+    auto icon = Resources::loadIcon(systrayIconFlavor(), QStringLiteral("state-%1").arg(syncStateIconName(result)), iconType);
 #ifdef Q_OS_MAC
     // This defines the icon as a template and enables automatic macOS color handling
     // See https://bugreports.qt.io/browse/QTBUG-42109
@@ -175,94 +166,22 @@ QIcon Theme::themeTrayIcon(const QString &name, bool sysTrayMenuVisible, IconTyp
     return icon;
 }
 
-QIcon Theme::themeIcon(const QString &name, Theme::IconType iconType) const
-{
-    return loadIcon((Resources::isUsingDarkTheme() && allowDarkTheme()) ? darkTheme() : coloredTheme(), name, iconType);
-}
-/*
- * helper to load a icon from either the icon theme the desktop provides or from
- * the apps Qt resources.
- */
-QIcon Theme::loadIcon(const QString &flavor, const QString &name, IconType iconType) const
-{
-    // prevent recusion
-    const bool useCoreIcon = (iconType == IconType::VanillaIcon) || isVanilla();
-    const QString path = useCoreIcon ? vanillaThemePath() : brandThemePath();
-    const QString key = name + QLatin1Char(',') + flavor;
-    QIcon &cached = _iconCache[key]; // Take reference, this will also "set" the cache entry
-    if (cached.isNull()) {
-        if (isVanilla() && QIcon::hasThemeIcon(name)) {
-            // use from theme
-            return cached = QIcon::fromTheme(name);
-        }
-        const QString svg = QStringLiteral("%1/%2/%3.svg").arg(path, flavor, name);
-        if (QFile::exists(svg)) {
-            return cached = QIcon(svg);
-        }
-
-        const QString png = QStringLiteral("%1/%2/%3.png").arg(path, flavor, name);
-        if (QFile::exists(png)) {
-            return cached = QIcon(png);
-        }
-
-        const QList<int> sizes {16, 22, 32, 48, 64, 128, 256, 512, 1024};
-        QString previousIcon;
-        for (int size : sizes) {
-            QString pixmapName = QStringLiteral("%1/%2/%3-%4.png").arg(path, flavor, name, QString::number(size));
-            if (QFile::exists(pixmapName)) {
-                previousIcon = pixmapName;
-                cached.addFile(pixmapName, { size, size });
-            } else if (size >= 128) {
-                if (!previousIcon.isEmpty()) {
-                    qWarning() << "Upscaling:" << previousIcon << "to" << size;
-                    cached.addPixmap(QPixmap(previousIcon).scaled({ size, size }, Qt::KeepAspectRatio, Qt::SmoothTransformation));
-                }
-            }
-        }
-    }
-    if (cached.isNull()) {
-        if (!useCoreIcon && iconType == IconType::BrandedIconWithFallbackToVanillaIcon) {
-            return loadIcon(flavor, name, IconType::VanillaIcon);
-        }
-        qWarning() << "Failed to locate the icon" << name;
-    }
-    return cached;
-}
-
-bool Theme::hasTheme(IconType type, const QString &theme) const
-{
-    const auto key = qMakePair(type != IconType::VanillaIcon, theme);
-    auto it = _themeCache.constFind(key);
-    if (it == _themeCache.cend()) {
-        return _themeCache[key] = QFileInfo(QStringLiteral("%1/%2/").arg(type == IconType::VanillaIcon ? vanillaThemePath() : brandThemePath(), theme)).isDir();
-    }
-    return it.value();
-}
-
-QString Theme::systrayIconFlavor(bool mono, bool sysTrayMenuVisible) const
-{
-    Q_UNUSED(sysTrayMenuVisible)
-    QString flavor;
-    if (mono) {
-        flavor = Utility::hasDarkSystray() ? whiteTheme() : blackTheme();
-
-#ifdef Q_OS_MAC
-        if (sysTrayMenuVisible) {
-            flavor = QLatin1String("white");
-        }
-#endif
-    } else {
-        // we have a dark sys tray and the theme has support for that
-        flavor = (Utility::hasDarkSystray() && allowDarkTheme()) ? darkTheme() : coloredTheme();
-    }
-    return flavor;
-}
-
 Theme::Theme()
     : QObject(nullptr)
 {
 }
 
+QList<QmlUrlButton> Theme::qmlUrlButtons() const
+{
+    const auto urls = urlButtons();
+    QList<QmlUrlButton> out;
+    out.reserve(urls.size());
+    for (const auto &u : urls) {
+        out.append(QmlUrlButton(u));
+    }
+    return out;
+}
+
 // If this option returns true, the client only supports one folder to sync.
 // The Add-Button is removed accordingly.
 bool Theme::singleSyncFolder() const
@@ -311,49 +230,24 @@ QString Theme::overrideServerUrlV2() const
     return serverOverride;
 }
 
-QString Theme::forceConfigAuthType() const
+QString Theme::overrideServerPath() const
 {
-    return QString();
-}
-
-
-QString Theme::defaultClientFolder() const
-{
-    return appName();
+    return {};
 }
 
 void Theme::setSystrayUseMonoIcons(bool mono)
 {
     _mono = mono;
-    emit systrayUseMonoIconsChanged(mono);
+    Q_EMIT systrayUseMonoIconsChanged(mono);
 }
 
-bool Theme::systrayUseMonoIcons() const
+QUrl Theme::updateCheckUrl() const
 {
-    return _mono;
-}
-
-bool Theme::monoIconsAvailable() const
-{
-    // mono icons are only supported in vanilla and if a customer provides them
-    // no fallback to vanilla
-    return hasTheme(IconType::BrandedIcon, systrayIconFlavor(true));
-}
-
-QString Theme::updateCheckUrl() const
-{
-    return QStringLiteral("https://updates.owncloud.com/client/");
-}
-
-qint64 Theme::newBigFolderSizeLimit() const
-{
-    // Default to 500MB
-    return 500;
-}
-
-bool Theme::wizardHideExternalStorageConfirmationCheckbox() const
-{
-    return false;
+#ifndef APPLICATION_UPDATE_URL
+    return QUrl(QString());
+#else
+    return QUrl(QStringLiteral(APPLICATION_UPDATE_URL));
+#endif
 }
 
 bool Theme::wizardSkipAdvancedPage() const
@@ -361,11 +255,6 @@ bool Theme::wizardSkipAdvancedPage() const
     return false;
 }
 
-bool Theme::wizardHideFolderSizeLimitCheckbox() const
-{
-    return false;
-}
-
 QString Theme::gitSHA1(VersionFormat format) const
 {
     const QString gitShahSort = Version::gitSha().left(6);
@@ -412,30 +301,21 @@ QString Theme::aboutVersions(Theme::VersionFormat format) const
             gitUrl = gitSHA1(format) + br;
         }
     }
-
-
-#if defined(OC_PLUGIN_DIR)
-    const QString pluginDirComment = QCoreApplication::translate("ownCloudTheme::pluginDir", "Plugin dir: %1%2").arg(QDir(QCoreApplication::applicationDirPath()).filePath(QStringLiteral(OC_PLUGIN_DIR)), br);
-#else
-    const QString pluginDirComment = QString();
-#endif
+    QStringList sysInfo = {QStringLiteral("OS: %1-%2 (build arch: %3, CPU arch: %4)")
+                               .arg(QSysInfo::productType(), QSysInfo::kernelVersion(), QSysInfo::buildCpuArchitecture(), Utility::currentCpuArch())};
+    // may be called by both GUI and CLI, but we can display QPA only for the former
+    if (auto guiApp = qobject_cast<QGuiApplication *>(qApp)) {
+        sysInfo << QStringLiteral("QPA: %1").arg(guiApp->platformName());
+    }
 
     return QCoreApplication::translate("ownCloudTheme::aboutVersions()",
         "%1 %2%7"
         "%8"
         "Libraries Qt %3, %4%7"
-        "%9"
         "Using virtual files plugin: %5%7"
         "%6")
-        .arg(appName(),
-            _version,
-            qtVersionString,
-            QSslSocket::sslLibraryVersionString(),
-            Vfs::modeToString(VfsPluginManager::instance().bestAvailableVfsMode()),
-            QSysInfo::productType() % QLatin1Char('-') % QSysInfo::kernelVersion(),
-            br,
-            gitUrl,
-            pluginDirComment);
+        .arg(appName(), _version, qtVersionString, QSslSocket::sslLibraryVersionString(),
+            Utility::enumToString(VfsPluginManager::instance().bestAvailableVfsMode()), sysInfo.join(br), br, gitUrl);
 }
 
 
@@ -443,13 +323,13 @@ QString Theme::about() const
 {
     // Ideally, the vendor should be "ownCloud GmbH", but it cannot be changed without
     // changing the location of the settings and other registery keys.
-    const QString vendor = isVanilla() ? QStringLiteral("ownCloud GmbH") : QStringLiteral(APPLICATION_VENDOR);
+    const QString vendor = Resources::isVanillaTheme() ? QStringLiteral("ownCloud GmbH") : QStringLiteral(APPLICATION_VENDOR);
     return tr("<p>Version %1. For more information visit <a href=\"%2\">https://%3</a></p>"
               "<p>For known issues and help, please visit: <a href=\"https://central.owncloud.com/c/desktop-client\">https://central.owncloud.com</a></p>"
               "<p><small>By Klaas Freitag, Daniel Molkentin, Olivier Goffart, Markus Götz, "
               " Jan-Christoph Borchardt, Thomas Müller,<br>"
               "Dominik Schmidt, Michael Stingl, Hannah von Reth, Fabian Müller and others.</small></p>"
-              "<p>Copyright ownCloud GmbH</p>"
+              "<p>Copyright ownCloud GmbH (A Kiteworks Company)</p>"
               "<p>Distributed by %4 and licensed under the GNU General Public License (GPL) Version 2.0.<br/>"
               "%5 and the %5 logo are registered trademarks of %4 in the "
               "United States, other countries, or both.</p>"
@@ -498,53 +378,39 @@ QString Theme::syncStateIconName(const SyncResult &result) const
     Q_UNREACHABLE();
 }
 
-QIcon Theme::syncStateIcon(SyncResult::Status status, bool sysTray, bool sysTrayMenuVisible) const
-{
-    return syncStateIcon(SyncResult{status}, sysTray, sysTrayMenuVisible);
-}
-
-QIcon Theme::syncStateIcon(const SyncResult &result, bool sysTray, bool sysTrayMenuVisible) const
-{
-    return syncStateIcon(QStringLiteral("state-%1").arg(syncStateIconName(result)), sysTray, sysTrayMenuVisible);
-}
-
-QIcon Theme::syncStateIcon(const QString &iconName, bool sysTray, bool sysTrayMenuVisible) const
-{
-    if (sysTray) {
-        return themeTrayIcon(iconName, sysTrayMenuVisible);
-    } else {
-        return themeIcon(iconName);
-    }
-}
-
 QColor Theme::wizardHeaderTitleColor() const
 {
     return qApp->palette().text().color();
 }
 
-QColor Theme::wizardHeaderSubTitleColor() const
-{
-    return wizardHeaderTitleColor();
-}
-
 QColor Theme::wizardHeaderBackgroundColor() const
 {
     return QColor();
 }
 
+QmlButtonColor Theme::primaryButtonColor() const
+{
+    return {};
+}
+
+QmlButtonColor Theme::secondaryButtonColor() const
+{
+    return {};
+}
+
 QIcon Theme::wizardHeaderLogo() const
 {
     return applicationIcon();
 }
 
-QPixmap Theme::wizardHeaderBanner(const QSize &size) const
+QColor Theme::avatarColor() const
 {
-    const QColor c = wizardHeaderBackgroundColor();
-    if (!c.isValid())
-        return QPixmap();
-    QPixmap pix(size);
-    pix.fill(c);
-    return pix;
+    return {};
+}
+
+QColor Theme::avatarColorChecked() const
+{
+    return {};
 }
 
 QString Theme::webDavPath() const
@@ -588,11 +454,6 @@ QString Theme::wizardUrlPostfix() const
     return QString();
 }
 
-QString Theme::wizardUrlHint() const
-{
-    return QString();
-}
-
 QString Theme::quotaBaseFolder() const
 {
     return QStringLiteral("/");
@@ -618,6 +479,12 @@ QPair<QString, QString> Theme::oauthOverrideAuthUrl() const
     return {};
 }
 
+QVector<quint16> Theme::oauthPorts() const
+{
+    // zero means a random port
+    return {0};
+}
+
 QString Theme::openIdConnectScopes() const
 {
     return QStringLiteral("openid offline_access email profile");
@@ -640,7 +507,7 @@ QString Theme::versionSwitchOutput() const
 
 bool Theme::showVirtualFilesOption() const
 {
-    return enableExperimentalFeatures();
+    return true;
 }
 
 bool Theme::forceVirtualFilesOption() const
@@ -648,11 +515,6 @@ bool Theme::forceVirtualFilesOption() const
     return false;
 }
 
-bool Theme::enableExperimentalFeatures() const
-{
-    return ConfigFile().showExperimentalOptions();
-}
-
 bool Theme::connectionValidatorClearCookies() const
 {
     return false;
@@ -665,7 +527,7 @@ bool Theme::enableSocketApiIconSupport() const
 
 bool Theme::warnOnMultipleDb() const
 {
-    return isVanilla();
+    return Resources::isVanillaTheme();
 }
 
 bool Theme::allowDuplicatedFolderSyncPair() const
@@ -683,11 +545,35 @@ QVector<std::tuple<QString, QString, QUrl>> Theme::urlButtons() const
     return {};
 }
 
+bool Theme::enableMoveToTrash() const
+{
+    return true;
+}
+
+bool Theme::syncNewlyDiscoveredSpaces() const
+{
+    return false;
+}
+
 bool Theme::enableCernBranding() const
 {
     return false;
 }
 
+bool Theme::spacesAreCalledFolders() const
+{
+    return false;
+}
+
+bool Theme::withCrashReporter() const
+{
+#ifdef WITH_CRASHREPORTER
+    return true;
+#else
+    return false;
+#endif
+}
+
 template <>
 OWNCLOUDSYNC_EXPORT QString Utility::enumToDisplayName(Theme::UserIDType userIdType)
 {
