@@ -1,8 +1,3 @@
-from pageObjects.EnterPassword import EnterPassword
-
-from helpers.UserHelper import getDisplaynameForUser, getPasswordForUser
-from helpers.SetupClientHelper import setUpClient, getResourcePath
-from helpers.SyncHelper import waitForInitialSyncToComplete
 from helpers.SpaceHelper import (
     create_space,
     create_space_folder,
@@ -35,22 +30,12 @@ def step(context, user, space_name, role):
     add_user_to_space(user, space_name, role)
 
 
-@Given('user "|any|" has set up a client with space "|any|"')
-def step(context, user, space_name):
-    password = getPasswordForUser(user)
-    displayName = getDisplaynameForUser(user)
-    setUpClient(user, displayName, space_name)
-    EnterPassword.loginAfterSetup(user, password)
-    # wait for files to sync
-    waitForInitialSyncToComplete(getResourcePath('/', user, space_name))
-
-
 @Then(
     'as "|any|" the file "|any|" in the space "|any|" should have content "|any|" in the server'
 )
 def step(context, user, file_name, space_name, content):
     downloaded_content = get_file_content(space_name, file_name, user)
-    test.compare(downloaded_content, content, "Comparing file content")
+    test.compare(downloaded_content, content, 'Comparing file content')
 
 
 @Then(
@@ -58,6 +43,5 @@ def step(context, user, file_name, space_name, content):
     regexp=True,
 )
 def step(context, user, space_name, resource_name):
-    print(user, space_name, resource_name)
     exists = resource_exists(space_name, resource_name, user)
-    test.compare(exists, True, "Resource exists")
+    test.compare(exists, True, 'Resource exists')
