@@ -19,7 +19,7 @@ class TestLocalDiscovery : public QObject
 {
     Q_OBJECT
 
-private slots:
+private Q_SLOTS:
     void initTestCase_data()
     {
         QTest::addColumn<Vfs::Mode>("vfsMode");
@@ -33,7 +33,7 @@ private slots:
             // TODO: the hydrated version will fail due to an issue in the winvfs plugin, so leave it disabled for now.
             // QTest::newRow("Vfs::WindowsCfApi hydrated") << Vfs::WindowsCfApi << false;
         } else if (Utility::isWindows()) {
-            QWARN("Skipping Vfs::WindowsCfApi");
+            qWarning("Skipping Vfs::WindowsCfApi");
         }
     }
 
@@ -75,11 +75,11 @@ private slots:
         tracker.startSyncPartialDiscovery();
         QVERIFY(fakeFolder.applyLocalModificationsAndSync());
 
-        QVERIFY(fakeFolder.currentRemoteState().find("A/a3"));
-        QVERIFY(fakeFolder.currentRemoteState().find("A/X/x2"));
-        QVERIFY(!fakeFolder.currentRemoteState().find("A/Y/y2"));
-        QVERIFY(!fakeFolder.currentRemoteState().find("B/b3"));
-        QVERIFY(fakeFolder.currentLocalState().find("C/c3"));
+        QVERIFY(fakeFolder.currentRemoteState().find(QStringLiteral("A/a3")));
+        QVERIFY(fakeFolder.currentRemoteState().find(QStringLiteral("A/X/x2")));
+        QVERIFY(!fakeFolder.currentRemoteState().find(QStringLiteral("A/Y/y2")));
+        QVERIFY(!fakeFolder.currentRemoteState().find(QStringLiteral("B/b3")));
+        QVERIFY(fakeFolder.currentLocalState().find(QStringLiteral("C/c3")));
         QCOMPARE(fakeFolder.syncEngine().lastLocalDiscoveryStyle(), LocalDiscoveryStyle::DatabaseAndFilesystem);
         QVERIFY(tracker.localDiscoveryPaths().empty());
 
@@ -97,43 +97,43 @@ private slots:
         FakeFolder fakeFolder(FileInfo::A12_B12_C12_S12(), vfsMode, filesAreDehydrated);
         auto &engine = fakeFolder.syncEngine();
 
-        QVERIFY(engine.shouldDiscoverLocally(""));
-        QVERIFY(engine.shouldDiscoverLocally("A"));
-        QVERIFY(engine.shouldDiscoverLocally("A/X"));
+        QVERIFY(engine.shouldDiscoverLocally(QString()));
+        QVERIFY(engine.shouldDiscoverLocally(QStringLiteral("A")));
+        QVERIFY(engine.shouldDiscoverLocally(QStringLiteral("A/X")));
 
-        fakeFolder.syncEngine().setLocalDiscoveryOptions(
-            LocalDiscoveryStyle::DatabaseAndFilesystem,
-            { "A/X", "A/X space", "A/X/beta", "foo bar space/touch", "foo/", "zzz", "zzzz" });
+        fakeFolder.syncEngine().setLocalDiscoveryOptions(LocalDiscoveryStyle::DatabaseAndFilesystem,
+            {QStringLiteral("A/X"), QStringLiteral("A/X space"), QStringLiteral("A/X/beta"), QStringLiteral("foo bar space/touch"), QStringLiteral("foo/"),
+                QStringLiteral("zzz"), QStringLiteral("zzzz")});
 
-        QVERIFY(engine.shouldDiscoverLocally(""));
-        QVERIFY(engine.shouldDiscoverLocally("A"));
-        QVERIFY(engine.shouldDiscoverLocally("A/X"));
-        QVERIFY(!engine.shouldDiscoverLocally("B"));
-        QVERIFY(!engine.shouldDiscoverLocally("A B"));
-        QVERIFY(!engine.shouldDiscoverLocally("B/X"));
-        QVERIFY(engine.shouldDiscoverLocally("foo bar space"));
-        QVERIFY(engine.shouldDiscoverLocally("foo"));
-        QVERIFY(!engine.shouldDiscoverLocally("foo bar"));
-        QVERIFY(!engine.shouldDiscoverLocally("foo bar/touch"));
-        // These are within "A/X" so they should be discovered
-        QVERIFY(engine.shouldDiscoverLocally("A/X/alpha"));
-        QVERIFY(engine.shouldDiscoverLocally("A/X beta"));
-        QVERIFY(engine.shouldDiscoverLocally("A/X/Y"));
-        QVERIFY(engine.shouldDiscoverLocally("A/X space"));
-        QVERIFY(engine.shouldDiscoverLocally("A/X space/alpha"));
-        QVERIFY(!engine.shouldDiscoverLocally("A/Xylo/foo"));
-        QVERIFY(engine.shouldDiscoverLocally("zzzz/hello"));
-        QVERIFY(!engine.shouldDiscoverLocally("zzza/hello"));
+        QVERIFY(engine.shouldDiscoverLocally(QString()));
+        QVERIFY(engine.shouldDiscoverLocally(QStringLiteral("A")));
+        QVERIFY(engine.shouldDiscoverLocally(QStringLiteral("A/X")));
+        QVERIFY(!engine.shouldDiscoverLocally(QStringLiteral("B")));
+        QVERIFY(!engine.shouldDiscoverLocally(QStringLiteral("A B")));
+        QVERIFY(!engine.shouldDiscoverLocally(QStringLiteral("B/X")));
+        QVERIFY(engine.shouldDiscoverLocally(QStringLiteral("foo bar space")));
+        QVERIFY(engine.shouldDiscoverLocally(QStringLiteral("foo")));
+        QVERIFY(!engine.shouldDiscoverLocally(QStringLiteral("foo bar")));
+        QVERIFY(!engine.shouldDiscoverLocally(QStringLiteral("foo bar/touch")));
+        // These are within QStringLiteral("A/X") so they should be discovered
+        QVERIFY(engine.shouldDiscoverLocally(QStringLiteral("A/X/alpha")));
+        QVERIFY(engine.shouldDiscoverLocally(QStringLiteral("A/X beta")));
+        QVERIFY(engine.shouldDiscoverLocally(QStringLiteral("A/X/Y")));
+        QVERIFY(engine.shouldDiscoverLocally(QStringLiteral("A/X space")));
+        QVERIFY(engine.shouldDiscoverLocally(QStringLiteral("A/X space/alpha")));
+        QVERIFY(!engine.shouldDiscoverLocally(QStringLiteral("A/Xylo/foo")));
+        QVERIFY(engine.shouldDiscoverLocally(QStringLiteral("zzzz/hello")));
+        QVERIFY(!engine.shouldDiscoverLocally(QStringLiteral("zzza/hello")));
 
         QEXPECT_FAIL("", "There is a possibility of false positives if the set contains a path "
             "which is a prefix, and that prefix is followed by a character less than '/'", Continue);
-        QVERIFY(!engine.shouldDiscoverLocally("A/X o"));
+        QVERIFY(!engine.shouldDiscoverLocally(QStringLiteral("A/X o")));
 
         fakeFolder.syncEngine().setLocalDiscoveryOptions(
             LocalDiscoveryStyle::DatabaseAndFilesystem,
             {});
 
-        QVERIFY(!engine.shouldDiscoverLocally(""));
+        QVERIFY(!engine.shouldDiscoverLocally(QString()));
     }
 
     // Check whether item success and item failure adjusts the
@@ -149,7 +149,7 @@ private slots:
         connect(&fakeFolder.syncEngine(), &SyncEngine::itemCompleted, &tracker, &LocalDiscoveryTracker::slotItemCompleted);
         connect(&fakeFolder.syncEngine(), &SyncEngine::finished, &tracker, &LocalDiscoveryTracker::slotSyncFinished);
         auto trackerContains = [&](const char *path) {
-            return tracker.localDiscoveryPaths().find(path) != tracker.localDiscoveryPaths().end();
+            return tracker.localDiscoveryPaths().find(QString::fromLatin1(path)) != tracker.localDiscoveryPaths().end();
         };
 
         tracker.addTouchedPath(QStringLiteral("A/spurious"));
@@ -166,8 +166,8 @@ private slots:
         tracker.startSyncPartialDiscovery();
         QVERIFY(!fakeFolder.applyLocalModificationsAndSync());
 
-        QVERIFY(fakeFolder.currentRemoteState().find("A/a3"));
-        QVERIFY(!fakeFolder.currentRemoteState().find("A/a4"));
+        QVERIFY(fakeFolder.currentRemoteState().find(QStringLiteral("A/a3")));
+        QVERIFY(!fakeFolder.currentRemoteState().find(QStringLiteral("A/a4")));
         QVERIFY(!trackerContains("A/a3"));
         QVERIFY(trackerContains("A/a4"));
         QVERIFY(trackerContains("A/spurious")); // not removed since overall sync not successful
@@ -176,7 +176,7 @@ private slots:
         tracker.startSyncFullDiscovery();
         QVERIFY(!fakeFolder.applyLocalModificationsAndSync());
 
-        QVERIFY(!fakeFolder.currentRemoteState().find("A/a4"));
+        QVERIFY(!fakeFolder.currentRemoteState().find(QStringLiteral("A/a4")));
         QVERIFY(trackerContains("A/a4")); // had an error, still here
         QVERIFY(!trackerContains("A/spurious")); // removed due to full discovery
 
@@ -188,7 +188,7 @@ private slots:
         tracker.startSyncPartialDiscovery();
         QVERIFY(fakeFolder.applyLocalModificationsAndSync());
 
-        QVERIFY(fakeFolder.currentRemoteState().find("A/a4"));
+        QVERIFY(fakeFolder.currentRemoteState().find(QStringLiteral("A/a4")));
         QVERIFY(tracker.localDiscoveryPaths().empty());
     }
 
@@ -201,7 +201,7 @@ private slots:
 
         fakeFolder.localModifier().mkdir(QStringLiteral("A/newDir"));
         fakeFolder.localModifier().mkdir(QStringLiteral("A/newDir/subDir"));
-        fakeFolder.localModifier().insert(QStringLiteral("A/newDir/subDir/file"), 10_b);
+        fakeFolder.localModifier().insert(QStringLiteral("A/newDir/subDir/file"), 10_B);
 
         // Only "A" was modified according to the file system tracker
         fakeFolder.syncEngine().setLocalDiscoveryOptions(
@@ -211,7 +211,7 @@ private slots:
         QVERIFY(fakeFolder.applyLocalModificationsAndSync());
 
         QCOMPARE(fakeFolder.currentLocalState(), fakeFolder.currentRemoteState());
-        QVERIFY(fakeFolder.currentRemoteState().find("A/newDir/subDir/file"));
+        QVERIFY(fakeFolder.currentRemoteState().find(QStringLiteral("A/newDir/subDir/file")));
     }
 
     // Tests the behavior of invalid filename detection
@@ -224,18 +224,96 @@ private slots:
         QCOMPARE(fakeFolder.currentLocalState(), fakeFolder.currentRemoteState());
 
         auto cap = TestUtils::testCapabilities();
-        cap.insert({ { "files", QVariantMap { { "blacklisted_files", QVariantList { ".foo", "bar" } } } } });
-        fakeFolder.account()->setCapabilities(cap);
+        cap.insert(
+            {{QStringLiteral("files"), QVariantMap{{QStringLiteral("blacklisted_files"), QVariantList{QStringLiteral(".foo"), QStringLiteral("bar")}}}}});
+        fakeFolder.account()->setCapabilities({fakeFolder.account()->url(), cap});
         fakeFolder.localModifier().insert(QStringLiteral("C/.foo"));
         fakeFolder.localModifier().insert(QStringLiteral("C/bar"));
         fakeFolder.localModifier().insert(QStringLiteral("C/moo"));
         fakeFolder.localModifier().insert(QStringLiteral("C/.moo"));
 
         QVERIFY(fakeFolder.applyLocalModificationsAndSync());
-        QVERIFY(fakeFolder.currentRemoteState().find("C/moo"));
-        QVERIFY(fakeFolder.currentRemoteState().find("C/.moo"));
-        QVERIFY(!fakeFolder.currentRemoteState().find("C/.foo"));
-        QVERIFY(!fakeFolder.currentRemoteState().find("C/bar"));
+        QVERIFY(fakeFolder.currentRemoteState().find(QStringLiteral("C/moo")));
+        QVERIFY(fakeFolder.currentRemoteState().find(QStringLiteral("C/.moo")));
+        QVERIFY(!fakeFolder.currentRemoteState().find(QStringLiteral("C/.foo")));
+        QVERIFY(!fakeFolder.currentRemoteState().find(QStringLiteral("C/bar")));
+    }
+
+    void testNameNormalization_data()
+    {
+        QTest::addColumn<QString>("correct");
+        QTest::addColumn<QString>("incorrect");
+
+        const unsigned char a_umlaut_composed_bytes[] = {0xc3, 0xa4, 0x00};
+        const QString a_umlaut_composed = QString::fromUtf8(reinterpret_cast<const char *>(a_umlaut_composed_bytes));
+        const QString a_umlaut_decomposed = a_umlaut_composed.normalized(QString::NormalizationForm_D);
+
+        QTest::newRow("a_umlaut decomposed") << a_umlaut_decomposed << a_umlaut_composed;
+        QTest::newRow("a_umlaut composed") << a_umlaut_composed << a_umlaut_decomposed;
+    }
+
+    // Test that when a file/directory name on the remote is encoded in NFC, the local name is encoded
+    // in the same way, and that a subsequent sync does not change anything. And the same for NFD.
+    void testNameNormalization()
+    {
+        QFETCH_GLOBAL(Vfs::Mode, vfsMode);
+        QFETCH_GLOBAL(bool, filesAreDehydrated);
+
+        QFETCH(QString, correct);
+        QFETCH(QString, incorrect);
+
+        // Create an empty remote folder
+        FakeFolder fakeFolder({FileInfo{}}, vfsMode, filesAreDehydrated);
+        OperationCounter counter(fakeFolder);
+
+        // Create a file with an a-umlout in the "correct" normalization:
+        fakeFolder.remoteModifier().mkdir(QStringLiteral("P"));
+        fakeFolder.remoteModifier().mkdir(QStringLiteral("P/A"));
+        fakeFolder.remoteModifier().insert(QStringLiteral("P/A/") + correct);
+
+        // Same for a directory, holding a "normal" file:
+        fakeFolder.remoteModifier().mkdir(QStringLiteral("P/B") + correct);
+        fakeFolder.remoteModifier().insert(QStringLiteral("P/B") + correct + QStringLiteral("/b"));
+
+        LocalDiscoveryTracker tracker;
+        connect(&fakeFolder.syncEngine(), &SyncEngine::itemCompleted, &tracker, &LocalDiscoveryTracker::slotItemCompleted);
+        connect(&fakeFolder.syncEngine(), &SyncEngine::finished, &tracker, &LocalDiscoveryTracker::slotSyncFinished);
+
+        // First sync: discover that there are files/directories on the server that are not yet synced to the local end
+        QVERIFY(fakeFolder.applyLocalModificationsAndSync());
+
+        // Check that locally we have the file and the directory with the correct names:
+        {
+            auto localState = fakeFolder.currentLocalState();
+            QVERIFY(localState.find(QStringLiteral("P/A/") + correct) != nullptr); // check if the file exists
+            QVERIFY(localState.find(QStringLiteral("P/B") + correct + QStringLiteral("/b")) != nullptr); // check if the file exists
+        }
+
+        counter.reset();
+
+        qDebug() << "*** MARK"; // Log marker to check if a PUT/DELETE shows up in the second sync
+
+        // Force a full local discovery on the next sync, which forces a walk of the (local) file system, reading back names (and file sizes/mtimes/etc.)...
+        fakeFolder.syncEngine().setLocalDiscoveryOptions(LocalDiscoveryStyle::DatabaseAndFilesystem, {QStringLiteral("P")});
+        tracker.startSyncFullDiscovery();
+
+        // ... and start the second sync:
+        QVERIFY(fakeFolder.applyLocalModificationsAndSync());
+
+        // If the normalization of the file/directory name did not change, no rename/move/etc. should have been detected, so check that the client didn't issue
+        // any of these operations:
+        QCOMPARE(counter.nDELETE, 0);
+        QCOMPARE(counter.nMOVE, 0);
+        QCOMPARE(counter.nPUT, 0);
+
+        // Check that the remote names are unchanged, and that no "incorrect" names have been introduced:
+        FileInfo &remoteState = fakeFolder.currentRemoteState();
+        QVERIFY(remoteState.find(QStringLiteral("P/A/") + correct) != nullptr); // check if the file still exists in the original normalization
+        QVERIFY(remoteState.find(QStringLiteral("P/A/") + incorrect) == nullptr); // there should NOT be a file with another normalization
+        QVERIFY(remoteState.find(QStringLiteral("P/B") + correct + QStringLiteral("/b"))
+            != nullptr); // check if the directory still exists in the original normalization
+        QVERIFY(remoteState.find(QStringLiteral("P/B") + incorrect + QStringLiteral("/b"))
+            == nullptr); // there should NOT be a directory with another normalization
     }
 };
 
