@@ -18,7 +18,7 @@ namespace {
 auto uploadConflictFilesCapabilities(bool b)
 {
     auto cap = TestUtils::testCapabilities();
-    cap.insert({ { "uploadConflictFiles", b } });
+    cap.insert({{QStringLiteral("uploadConflictFiles"), b}});
     return cap;
 }
 
@@ -26,19 +26,14 @@ bool itemSuccessful(const ItemCompletedSpy &spy, const QString &path, const Sync
 {
     auto item = spy.findItem(path);
     Q_ASSERT(item);
-    return item->_status == SyncFileItem::Success && item->_instruction == instr;
+    return item->_status == SyncFileItem::Success && item->instruction() == instr;
 }
 
 bool itemConflict(const ItemCompletedSpy &spy, const QString &path)
 {
     auto item = spy.findItem(path);
     Q_ASSERT(item);
-    return item->_status == SyncFileItem::Conflict && item->_instruction == CSYNC_INSTRUCTION_CONFLICT;
-}
-
-bool itemSuccessfulMove(const ItemCompletedSpy &spy, const QString &path)
-{
-    return itemSuccessful(spy, path, CSYNC_INSTRUCTION_RENAME);
+    return item->_status == SyncFileItem::Conflict && item->instruction() == CSYNC_INSTRUCTION_CONFLICT;
 }
 
 QStringList findConflicts(const FileInfo &dir)
@@ -61,10 +56,10 @@ bool expectAndWipeConflict(FakeFolder &fFolder, const QString &path)
         return false;
     }
 
-    for (const auto &item : qAsConst(base->children)) {
+    for (const auto &item : std::as_const(base->children)) {
         if (item.name.startsWith(pathComponents.fileName()) && item.name.contains(QLatin1String("(conflicted copy"))) {
             fFolder.localModifier().remove(item.path());
-            fFolder.applyLocalModificationsWithoutSync();
+            OC_ASSERT(fFolder.applyLocalModificationsWithoutSync());
             return true;
         }
     }
@@ -85,7 +80,7 @@ class TestSyncConflict : public QObject
 {
     Q_OBJECT
 
-private slots:
+private Q_SLOTS:
     void initTestCase_data()
     {
         QTest::addColumn<Vfs::Mode>("vfsMode");
@@ -95,11 +90,9 @@ private slots:
 
         if (VfsPluginManager::instance().isVfsPluginAvailable(Vfs::WindowsCfApi)) {
             QTest::newRow("Vfs::WindowsCfApi dehydrated") << Vfs::WindowsCfApi << true;
-
-            // TODO: the hydrated version will fail due to an issue in the winvfs plugin, so leave it disabled for now.
-            // QTest::newRow("Vfs::WindowsCfApi hydrated") << Vfs::WindowsCfApi << false;
+            QTest::newRow("Vfs::WindowsCfApi hydrated") << Vfs::WindowsCfApi << false;
         } else if (Utility::isWindows()) {
-            QWARN("Skipping Vfs::WindowsCfApi");
+            qWarning("Skipping Vfs::WindowsCfApi");
         }
     }
 
@@ -129,8 +122,8 @@ private slots:
                 QVERIFY(!name.contains(fakeFolder.syncEngine().account()->davDisplayName()));
             }
 
-            QVERIFY(expectAndWipeConflict(fakeFolder, "A/a1"));
-            QVERIFY(expectAndWipeConflict(fakeFolder, "A/a2"));
+            QVERIFY(expectAndWipeConflict(fakeFolder, QStringLiteral("A/a1")));
+            QVERIFY(expectAndWipeConflict(fakeFolder, QStringLiteral("A/a2")));
             QCOMPARE(fakeFolder.currentLocalState(), fakeFolder.currentRemoteState());
         }
     }
@@ -141,7 +134,7 @@ private slots:
         QFETCH_GLOBAL(bool, filesAreDehydrated);
 
         FakeFolder fakeFolder(FileInfo::A12_B12_C12_S12(), vfsMode, filesAreDehydrated);
-        fakeFolder.account()->setCapabilities(uploadConflictFilesCapabilities(true));
+        fakeFolder.account()->setCapabilities({fakeFolder.account()->url(), uploadConflictFilesCapabilities(true)});
         QCOMPARE(fakeFolder.currentLocalState(), fakeFolder.currentRemoteState());
 
         QMap<QByteArray, QString> conflictMap;
@@ -149,8 +142,8 @@ private slots:
             if (op == QNetworkAccessManager::PutOperation) {
                 if (request.rawHeader("OC-Conflict") == "1") {
                     auto baseFileId = request.rawHeader("OC-ConflictBaseFileId");
-                    auto components = request.url().toString().split('/');
-                    QString conflictFile = components.mid(components.size() - 2).join('/');
+                    auto components = request.url().toString().split(QLatin1Char('/'));
+                    QString conflictFile = components.mid(components.size() - 2).join(QLatin1Char('/'));
                     conflictMap[baseFileId] = conflictFile;
                     [&] {
                         QVERIFY(!baseFileId.isEmpty());
@@ -177,21 +170,21 @@ private slots:
             auto remote = fakeFolder.currentRemoteState();
             QCOMPARE(local, remote);
 
-            auto a1FileId = fakeFolder.remoteModifier().find("A/a1")->fileId;
-            auto a2FileId = fakeFolder.remoteModifier().find("A/a2")->fileId;
+            auto a1FileId = fakeFolder.remoteModifier().find(QStringLiteral("A/a1"))->fileId;
+            auto a2FileId = fakeFolder.remoteModifier().find(QStringLiteral("A/a2"))->fileId;
             QVERIFY(conflictMap.contains(a1FileId));
             QVERIFY(conflictMap.contains(a2FileId));
             QCOMPARE(conflictMap.size(), 2);
             QCOMPARE(Utility::conflictFileBaseNameFromPattern(conflictMap[a1FileId].toUtf8()), QByteArray("A/a1"));
 
             // Check that the conflict file contains the username
-            QVERIFY(conflictMap[a1FileId].contains(QString("(conflicted copy %1 ").arg(fakeFolder.syncEngine().account()->davDisplayName())));
+            QVERIFY(conflictMap[a1FileId].contains(QString::fromLatin1("(conflicted copy %1 ").arg(fakeFolder.syncEngine().account()->davDisplayName())));
 
             QCOMPARE(remote.find(conflictMap[a1FileId])->contentChar, 'L');
-            QCOMPARE(remote.find("A/a1")->contentChar, 'R');
+            QCOMPARE(remote.find(QStringLiteral("A/a1"))->contentChar, 'R');
 
             QCOMPARE(remote.find(conflictMap[a2FileId])->contentSize, 5);
-            QCOMPARE(remote.find("A/a2")->contentSize, 6);
+            QCOMPARE(remote.find(QStringLiteral("A/a2"))->contentSize, 6);
         }
     }
 
@@ -201,7 +194,7 @@ private slots:
         QFETCH_GLOBAL(bool, filesAreDehydrated);
 
         FakeFolder fakeFolder(FileInfo::A12_B12_C12_S12(), vfsMode, filesAreDehydrated);
-        fakeFolder.account()->setCapabilities(uploadConflictFilesCapabilities(true));
+        fakeFolder.account()->setCapabilities({fakeFolder.account()->url(), uploadConflictFilesCapabilities(true)});
         QCOMPARE(fakeFolder.currentLocalState(), fakeFolder.currentRemoteState());
 
         QMap<QByteArray, QString> conflictMap;
@@ -209,8 +202,8 @@ private slots:
             if (op == QNetworkAccessManager::PutOperation) {
                 if (request.rawHeader("OC-Conflict") == "1") {
                     auto baseFileId = request.rawHeader("OC-ConflictBaseFileId");
-                    auto components = request.url().toString().split('/');
-                    QString conflictFile = components.mid(components.size() - 2).join('/');
+                    auto components = request.url().toString().split(QLatin1Char('/'));
+                    QString conflictFile = components.mid(components.size() - 2).join(QLatin1Char('/'));
                     conflictMap[baseFileId] = conflictFile;
                     [&] {
                         QVERIFY(!baseFileId.isEmpty());
@@ -225,7 +218,7 @@ private slots:
         // Explicitly add a conflict file to simulate the case where the upload of the
         // file didn't finish in the same sync run that the conflict was created.
         // To do that we need to create a mock conflict record.
-        auto a1FileId = fakeFolder.remoteModifier().find("A/a1")->fileId;
+        auto a1FileId = fakeFolder.remoteModifier().find(QStringLiteral("A/a1"))->fileId;
         QString conflictName = QStringLiteral("A/a1 (conflicted copy me 1234)");
         fakeFolder.localModifier().insert(conflictName, FileModifier::DefaultFileSize, 'L');
         ConflictRecord conflictRecord;
@@ -276,7 +269,7 @@ private slots:
         QFETCH_GLOBAL(bool, filesAreDehydrated);
 
         FakeFolder fakeFolder(FileInfo::A12_B12_C12_S12(), vfsMode, filesAreDehydrated);
-        fakeFolder.account()->setCapabilities(uploadConflictFilesCapabilities(true));
+        fakeFolder.account()->setCapabilities({fakeFolder.account()->url(), uploadConflictFilesCapabilities(true)});
         QCOMPARE(fakeFolder.currentLocalState(), fakeFolder.currentRemoteState());
 
         // With no headers from the server
@@ -285,12 +278,12 @@ private slots:
         QCOMPARE(fakeFolder.currentLocalState(), fakeFolder.currentRemoteState());
         auto conflictRecord = fakeFolder.syncJournal().conflictRecord("A/a1 (conflicted copy 1234)");
         QVERIFY(conflictRecord.isValid());
-        QCOMPARE(conflictRecord.baseFileId, fakeFolder.remoteModifier().find("A/a1")->fileId);
+        QCOMPARE(conflictRecord.baseFileId, fakeFolder.remoteModifier().find(QStringLiteral("A/a1"))->fileId);
         QCOMPARE(conflictRecord.initialBasePath, QByteArray("A/a1"));
 
         // Now with server headers
         QObject parent;
-        auto a2FileId = fakeFolder.remoteModifier().find("A/a2")->fileId;
+        auto a2FileId = fakeFolder.remoteModifier().find(QStringLiteral("A/a2"))->fileId;
         fakeFolder.setServerOverride([&](QNetworkAccessManager::Operation op, const QNetworkRequest &request, QIODevice *) -> QNetworkReply * {
             if (op == QNetworkAccessManager::GetOperation) {
                 auto reply = new FakeGetReply(fakeFolder.remoteModifier(), op, request, &parent);
@@ -328,7 +321,7 @@ private slots:
         QFETCH_GLOBAL(bool, filesAreDehydrated);
 
         FakeFolder fakeFolder(FileInfo::A12_B12_C12_S12(), vfsMode, filesAreDehydrated);
-        fakeFolder.account()->setCapabilities(uploadConflictFilesCapabilities(true));
+        fakeFolder.account()->setCapabilities({fakeFolder.account()->url(), uploadConflictFilesCapabilities(true)});
         QCOMPARE(fakeFolder.currentLocalState(), fakeFolder.currentRemoteState());
 
         // Make conflict records
@@ -359,7 +352,7 @@ private slots:
         QFETCH_GLOBAL(bool, filesAreDehydrated);
 
         FakeFolder fakeFolder(FileInfo::A12_B12_C12_S12(), vfsMode, filesAreDehydrated);
-        fakeFolder.account()->setCapabilities(uploadConflictFilesCapabilities(false));
+        fakeFolder.account()->setCapabilities({fakeFolder.account()->url(), uploadConflictFilesCapabilities(false)});
         QCOMPARE(fakeFolder.currentLocalState(), fakeFolder.currentRemoteState());
 
         // Create two conflicts
@@ -371,31 +364,31 @@ private slots:
         fakeFolder.remoteModifier().appendByte(QStringLiteral("A/a2"));
         QVERIFY(fakeFolder.applyLocalModificationsAndSync());
 
-        const auto &conflicts = findConflicts(fakeFolder.currentLocalState().children["A"]);
+        const auto &conflicts = findConflicts(fakeFolder.currentLocalState().children[QStringLiteral("A")]);
         if (filesAreDehydrated) {
             // There should be no conflicts: before a modification is done to a local file,
             // it will be downloaded from the remote first.
             QCOMPARE(conflicts.size(), 0);
         } else {
-            QByteArray a1conflict;
-            QByteArray a2conflict;
+            QString a1conflict;
+            QString a2conflict;
             for (const auto &conflict : conflicts) {
-                if (conflict.contains("a1"))
-                    a1conflict = conflict.toUtf8();
-                if (conflict.contains("a2"))
-                    a2conflict = conflict.toUtf8();
+                if (conflict.contains(QLatin1String("a1")))
+                    a1conflict = conflict;
+                if (conflict.contains(QLatin1String("a2")))
+                    a2conflict = conflict;
             }
 
             // A nothing-to-sync keeps them alive
             QVERIFY(fakeFolder.applyLocalModificationsAndSync());
-            QVERIFY(fakeFolder.syncJournal().conflictRecord(a1conflict).isValid());
-            QVERIFY(fakeFolder.syncJournal().conflictRecord(a2conflict).isValid());
+            QVERIFY(fakeFolder.syncJournal().conflictRecord(a1conflict.toUtf8()).isValid());
+            QVERIFY(fakeFolder.syncJournal().conflictRecord(a2conflict.toUtf8()).isValid());
 
             // When the file is removed, the record is removed too
             fakeFolder.localModifier().remove(a2conflict);
             QVERIFY(fakeFolder.applyLocalModificationsAndSync());
-            QVERIFY(fakeFolder.syncJournal().conflictRecord(a1conflict).isValid());
-            QVERIFY(!fakeFolder.syncJournal().conflictRecord(a2conflict).isValid());
+            QVERIFY(fakeFolder.syncJournal().conflictRecord(a1conflict.toUtf8()).isValid());
+            QVERIFY(!fakeFolder.syncJournal().conflictRecord(a2conflict.toUtf8()).isValid());
         }
     }
 
@@ -485,7 +478,7 @@ private slots:
         QFETCH_GLOBAL(bool, filesAreDehydrated);
 
         FakeFolder fakeFolder(FileInfo::A12_B12_C12_S12(), vfsMode, filesAreDehydrated);
-        fakeFolder.account()->setCapabilities(uploadConflictFilesCapabilities(true));
+        fakeFolder.account()->setCapabilities({fakeFolder.account()->url(), uploadConflictFilesCapabilities(true)});
         ItemCompletedSpy completeSpy(fakeFolder);
 
         auto cleanup = [&]() {
@@ -497,7 +490,7 @@ private slots:
         fakeFolder.localModifier().mkdir(QStringLiteral("Z"));
         fakeFolder.localModifier().mkdir(QStringLiteral("Z/subdir"));
         fakeFolder.localModifier().insert(QStringLiteral("Z/foo"));
-        fakeFolder.remoteModifier().insert(QStringLiteral("Z"), 63_b);
+        fakeFolder.remoteModifier().insert(QStringLiteral("Z"), 63_B);
 
         // 2) local file becomes a dir; remote file changes
         fakeFolder.localModifier().remove(QStringLiteral("A/a1"));
@@ -508,7 +501,7 @@ private slots:
         // 3) local dir gets a new file; remote dir becomes a file
         fakeFolder.localModifier().insert(QStringLiteral("B/zzz"));
         fakeFolder.remoteModifier().remove(QStringLiteral("B"));
-        fakeFolder.remoteModifier().insert(QStringLiteral("B"), 31_b);
+        fakeFolder.remoteModifier().insert(QStringLiteral("B"), 31_B);
 
         QVERIFY(fakeFolder.applyLocalModificationsAndSync());
 
@@ -522,41 +515,45 @@ private slots:
         std::sort(conflictRecords.begin(), conflictRecords.end());
 
         // 1)
-        QVERIFY(itemConflict(completeSpy, "Z"));
-        QCOMPARE(fakeFolder.currentLocalState().find("Z")->contentSize, 63);
-        QVERIFY(conflicts[2].contains("Z"));
+        QVERIFY(itemConflict(completeSpy, QStringLiteral("Z")));
+        QCOMPARE(fakeFolder.currentLocalState().find(QStringLiteral("Z"))->contentSize, 63);
+        QVERIFY(conflicts[2].contains(QStringLiteral("Z")));
         QCOMPARE(conflicts[2].toUtf8(), conflictRecords[2]);
         QVERIFY(QFileInfo(fakeFolder.localPath() + conflicts[2]).isDir());
-        QVERIFY(QFile::exists(fakeFolder.localPath() + conflicts[2] + "/foo"));
+        QVERIFY(QFile::exists(fakeFolder.localPath() + conflicts[2] + QStringLiteral("/foo")));
 
         // 2)
-        QVERIFY(itemConflict(completeSpy, "A/a1"));
-        QCOMPARE(fakeFolder.currentLocalState().find("A/a1")->contentSize, 5);
-        QVERIFY(conflicts[0].contains("A/a1"));
+        QVERIFY(itemConflict(completeSpy, QStringLiteral("A/a1")));
+        QCOMPARE(fakeFolder.currentLocalState().find(QStringLiteral("A/a1"))->contentSize, 5);
+        QVERIFY(conflicts[0].contains(QStringLiteral("A/a1")));
         QCOMPARE(conflicts[0].toUtf8(), conflictRecords[0]);
         QVERIFY(QFileInfo(fakeFolder.localPath() + conflicts[0]).isDir());
-        QVERIFY(QFile::exists(fakeFolder.localPath() + conflicts[0] + "/bar"));
+        QVERIFY(QFile::exists(fakeFolder.localPath() + conflicts[0] + QStringLiteral("/bar")));
 
         // 3)
-        QVERIFY(itemConflict(completeSpy, "B"));
-        QCOMPARE(fakeFolder.currentLocalState().find("B")->contentSize, 31);
-        QVERIFY(conflicts[1].contains("B"));
+        QVERIFY(itemConflict(completeSpy, QStringLiteral("B")));
+        if (filesAreDehydrated) {
+            QVERIFY(fakeFolder.vfs()->isDehydratedPlaceholder(fakeFolder.localPath() + QStringLiteral("B")));
+        } else {
+            QCOMPARE(fakeFolder.currentLocalState().find(QStringLiteral("B"))->contentSize, 31);
+        }
+        QVERIFY(conflicts[1].contains(QStringLiteral("B")));
         QCOMPARE(conflicts[1].toUtf8(), conflictRecords[1]);
         QVERIFY(QFileInfo(fakeFolder.localPath() + conflicts[1]).isDir());
-        QVERIFY(QFile::exists(fakeFolder.localPath() + conflicts[1] + "/zzz"));
+        QVERIFY(QFile::exists(fakeFolder.localPath() + conflicts[1] + QStringLiteral("/zzz")));
 
         // The contents of the conflict directories will only be uploaded after
         // another sync.
-        QVERIFY(fakeFolder.syncEngine().isAnotherSyncNeeded() == ImmediateFollowUp);
+        QCOMPARE(fakeFolder.syncEngine().isAnotherSyncNeeded(), true);
         cleanup();
         QVERIFY(fakeFolder.applyLocalModificationsAndSync());
 
         QVERIFY(itemSuccessful(completeSpy, conflicts[0], CSYNC_INSTRUCTION_NEW));
-        QVERIFY(itemSuccessful(completeSpy, conflicts[0] + "/bar", CSYNC_INSTRUCTION_NEW));
+        QVERIFY(itemSuccessful(completeSpy, conflicts[0] + QStringLiteral("/bar"), CSYNC_INSTRUCTION_NEW));
         QVERIFY(itemSuccessful(completeSpy, conflicts[1], CSYNC_INSTRUCTION_NEW));
-        QVERIFY(itemSuccessful(completeSpy, conflicts[1] + "/zzz", CSYNC_INSTRUCTION_NEW));
+        QVERIFY(itemSuccessful(completeSpy, conflicts[1] + QStringLiteral("/zzz"), CSYNC_INSTRUCTION_NEW));
         QVERIFY(itemSuccessful(completeSpy, conflicts[2], CSYNC_INSTRUCTION_NEW));
-        QVERIFY(itemSuccessful(completeSpy, conflicts[2] + "/foo", CSYNC_INSTRUCTION_NEW));
+        QVERIFY(itemSuccessful(completeSpy, conflicts[2] + QStringLiteral("/foo"), CSYNC_INSTRUCTION_NEW));
         QCOMPARE(fakeFolder.currentLocalState(), fakeFolder.currentRemoteState());
     }
 
@@ -566,7 +563,7 @@ private slots:
         QFETCH_GLOBAL(bool, filesAreDehydrated);
 
         FakeFolder fakeFolder(FileInfo::A12_B12_C12_S12(), vfsMode, filesAreDehydrated);
-        fakeFolder.account()->setCapabilities(uploadConflictFilesCapabilities(true));
+        fakeFolder.account()->setCapabilities({fakeFolder.account()->url(), uploadConflictFilesCapabilities(true)});
         ItemCompletedSpy completeSpy(fakeFolder);
 
         // 1) a NEW/NEW conflict
@@ -577,7 +574,7 @@ private slots:
 
         // 2) local dir becomes file: remote dir adds file
         fakeFolder.localModifier().remove(QStringLiteral("A"));
-        fakeFolder.localModifier().insert(QStringLiteral("A"), 63_b);
+        fakeFolder.localModifier().insert(QStringLiteral("A"), 63_B);
         fakeFolder.remoteModifier().insert(QStringLiteral("A/bar"));
 
         // 3) local file changes; remote file becomes dir
@@ -587,12 +584,18 @@ private slots:
         fakeFolder.remoteModifier().insert(QStringLiteral("B/b1/zzz"));
 
         if (filesAreDehydrated) {
-            QSKIP("Known bug: https://github.com/owncloud/client/issues/10223");
-            // The QSKIP below is due to operations order (verified on a local machine),
-            // but the sync below already always fails in the CI. So we will be skipping
-            // this entire test until the issue is fixed.
+            // the dehydrating the placeholder failed as the metadata is out of sync
+            QSignalSpy spy(fakeFolder.vfs().get(), &Vfs::needSync);
+            QVERIFY(!fakeFolder.applyLocalModificationsAndSync());
+            QVERIFY(spy.count() == 1);
+            QVERIFY(fakeFolder.syncOnce());
+            // reapply change and try again
+            fakeFolder.localModifier().appendByte(QStringLiteral("B/b1"));
+            // writing to a file fails!
+            QVERIFY(!fakeFolder.applyLocalModificationsAndSync());
+            // TODO: check error type
+            return;
         }
-
         QVERIFY(fakeFolder.applyLocalModificationsAndSync());
         auto conflicts = findConflicts(fakeFolder.currentLocalState());
         conflicts += findConflicts(fakeFolder.currentLocalState().children[QStringLiteral("B")]);
@@ -604,23 +607,20 @@ private slots:
         std::sort(conflictRecords.begin(), conflictRecords.end());
 
         // 1)
-        QVERIFY(itemConflict(completeSpy, "Z"));
-        QVERIFY(conflicts[2].contains("Z"));
+        QVERIFY(itemConflict(completeSpy, QStringLiteral("Z")));
+        QVERIFY(conflicts[2].contains(QStringLiteral("Z")));
         QCOMPARE(conflicts[2].toUtf8(), conflictRecords[2]);
 
         // 2)
-        QVERIFY(itemConflict(completeSpy, "A"));
-        QVERIFY(conflicts[0].contains("A"));
+        QVERIFY(itemConflict(completeSpy, QStringLiteral("A")));
+        QVERIFY(conflicts[0].contains(QStringLiteral("A")));
         QCOMPARE(conflicts[0].toUtf8(), conflictRecords[0]);
 
         // 3)
-        QVERIFY(itemConflict(completeSpy, "B/b1"));
-        QVERIFY(conflicts[1].contains("B/b1"));
+        QVERIFY(itemConflict(completeSpy, QStringLiteral("B/b1")));
+        QVERIFY(conflicts[1].contains(QStringLiteral("B/b1")));
         QCOMPARE(conflicts[1].toUtf8(), conflictRecords[1]);
 
-        if (filesAreDehydrated) {
-            QSKIP("Known bug: https://github.com/owncloud/client/issues/10223");
-        }
         // Also verifies that conflicts were uploaded
         QCOMPARE(fakeFolder.currentLocalState(), fakeFolder.currentRemoteState());
     }
@@ -646,15 +646,15 @@ private slots:
 
         QVERIFY(fakeFolder.applyLocalModificationsAndSync());
 
-        QVERIFY(itemSuccessful(completeSpy, "A", CSYNC_INSTRUCTION_TYPE_CHANGE));
-        QVERIFY(itemConflict(completeSpy, "B"));
+        QVERIFY(itemSuccessful(completeSpy, QStringLiteral("A"), CSYNC_INSTRUCTION_TYPE_CHANGE));
+        QVERIFY(itemConflict(completeSpy, QStringLiteral("B")));
 
         auto conflicts = findConflicts(fakeFolder.currentLocalState());
         std::sort(conflicts.begin(), conflicts.end());
         QVERIFY(conflicts.size() == 2);
-        QVERIFY(conflicts[0].contains("A (conflicted copy"));
-        QVERIFY(conflicts[1].contains("B (conflicted copy"));
-        for (const auto &conflict : qAsConst(conflicts))
+        QVERIFY(conflicts[0].contains(QStringLiteral("A (conflicted copy")));
+        QVERIFY(conflicts[1].contains(QStringLiteral("B (conflicted copy")));
+        for (const auto &conflict : std::as_const(conflicts))
             QDir(fakeFolder.localPath() + conflict).removeRecursively();
         QCOMPARE(fakeFolder.currentLocalState(), fakeFolder.currentRemoteState());
 
@@ -666,10 +666,6 @@ private slots:
         QFETCH_GLOBAL(Vfs::Mode, vfsMode);
         QFETCH_GLOBAL(bool, filesAreDehydrated);
 
-        if (filesAreDehydrated) {
-            QSKIP("Known bug: https://github.com/owncloud/client/issues/10223");
-        }
-
         FakeFolder fakeFolder(FileInfo::A12_B12_C12_S12(), vfsMode, filesAreDehydrated);
         ItemCompletedSpy completeSpy(fakeFolder);
 
@@ -689,25 +685,49 @@ private slots:
 
         QVERIFY(fakeFolder.applyLocalModificationsAndSync());
 
-        QVERIFY(itemSuccessful(completeSpy, "A", CSYNC_INSTRUCTION_TYPE_CHANGE));
-        QVERIFY(itemSuccessful(completeSpy, "B", CSYNC_INSTRUCTION_TYPE_CHANGE));
-        QVERIFY(itemSuccessful(completeSpy, "C/c1", CSYNC_INSTRUCTION_TYPE_CHANGE));
-        QVERIFY(itemSuccessful(completeSpy, "C/c2", CSYNC_INSTRUCTION_TYPE_CHANGE));
+        QVERIFY(itemSuccessful(completeSpy, QStringLiteral("A"), CSYNC_INSTRUCTION_TYPE_CHANGE));
+        QVERIFY(itemSuccessful(completeSpy, QStringLiteral("B"), CSYNC_INSTRUCTION_TYPE_CHANGE));
+        QVERIFY(itemSuccessful(completeSpy, QStringLiteral("C/c1"), CSYNC_INSTRUCTION_TYPE_CHANGE));
+        QVERIFY(itemSuccessful(completeSpy, QStringLiteral("C/c2"), CSYNC_INSTRUCTION_TYPE_CHANGE));
 
         // A becomes a conflict because we don't delete folders with files
         // inside of them!
         const auto &conflicts = findConflicts(fakeFolder.currentLocalState());
         QVERIFY(conflicts.size() == 1);
-        QVERIFY(conflicts[0].contains("A (conflicted copy"));
+        QVERIFY(conflicts[0].contains(QStringLiteral("A (conflicted copy")));
         for (const auto &conflict : conflicts) {
             QDir(fakeFolder.localPath() + conflict).removeRecursively();
         }
 
-        QVERIFY(fakeFolder.syncEngine().isAnotherSyncNeeded() == ImmediateFollowUp);
+        QCOMPARE(fakeFolder.syncEngine().isAnotherSyncNeeded(), true);
         QVERIFY(fakeFolder.applyLocalModificationsAndSync());
         QCOMPARE(fakeFolder.currentLocalState(), fakeFolder.currentRemoteState());
     }
 
+
+    void testTypeChangeEmptyFolderToFile()
+    {
+        QFETCH_GLOBAL(Vfs::Mode, vfsMode);
+        QFETCH_GLOBAL(bool, filesAreDehydrated);
+
+        FakeFolder fakeFolder({}, vfsMode, filesAreDehydrated);
+
+        fakeFolder.remoteModifier().mkdir(QStringLiteral("TestFolder"));
+        QVERIFY(fakeFolder.applyLocalModificationsAndSync());
+
+        ItemCompletedSpy completeSpy(fakeFolder);
+        fakeFolder.remoteModifier().remove(QStringLiteral("TestFolder"));
+        fakeFolder.remoteModifier().insert(QStringLiteral("TestFolder"));
+        QVERIFY(fakeFolder.applyLocalModificationsAndSync());
+
+        // TestFolder should now be a file, if we are using placeholders, this file must not be hydrated
+        if (filesAreDehydrated) {
+            QVERIFY(fakeFolder.vfs()->isDehydratedPlaceholder(fakeFolder.localPath() + QStringLiteral("TestFolder")));
+        }
+        QVERIFY(itemSuccessful(completeSpy, QStringLiteral("TestFolder"), CSYNC_INSTRUCTION_TYPE_CHANGE));
+        QCOMPARE(fakeFolder.currentLocalState(), fakeFolder.currentRemoteState());
+    }
+
     // Test what happens if we remove entries both on the server, and locally
     void testRemoveRemove()
     {
@@ -727,11 +747,11 @@ private slots:
         QCOMPARE(fakeFolder.currentLocalState(), expectedState);
         QCOMPARE(fakeFolder.currentRemoteState(), expectedState);
 
-        QVERIFY(dbRecord(fakeFolder, "B/b2").isValid());
+        QVERIFY(dbRecord(fakeFolder, QStringLiteral("B/b2")).isValid());
 
-        QVERIFY(!dbRecord(fakeFolder, "B/b1").isValid());
-        QVERIFY(!dbRecord(fakeFolder, "A/a1").isValid());
-        QVERIFY(!dbRecord(fakeFolder, "A").isValid());
+        QVERIFY(!dbRecord(fakeFolder, QStringLiteral("B/b1")).isValid());
+        QVERIFY(!dbRecord(fakeFolder, QStringLiteral("A/a1")).isValid());
+        QVERIFY(!dbRecord(fakeFolder, QStringLiteral("A")).isValid());
     }
 };
 
