@@ -8,17 +8,23 @@
 #include "syncenginetestutils.h"
 #include "testutils.h"
 
-#include "httplogger.h"
 #include "accessmanager.h"
+#include "gui/accountmanager.h"
+#include "httplogger.h"
 #include "libsync/configfile.h"
+#include "libsync/syncresult.h"
 
 #include <thread>
+#include <vio/csync_vio_local.h>
 
 using namespace std::chrono_literals;
+using namespace std::chrono;
 
-PathComponents::PathComponents(const char *path)
-    : PathComponents { QString::fromUtf8(path) }
+namespace {
+inline auto chunkingNgUploadPathC()
 {
+    return QStringLiteral("/owncloud/remote.php/dav/uploads/admin/");
+}
 }
 
 PathComponents::PathComponents(const QString &path)
@@ -43,7 +49,7 @@ PathComponents PathComponents::subComponents() const &
 
 void DiskFileModifier::remove(const QString &relativePath)
 {
-    _processArguments.append({ "remove", relativePath });
+    _processArguments.append({QStringLiteral("remove"), relativePath});
 }
 
 void DiskFileModifier::insert(const QString &relativePath, quint64 size, char contentChar)
@@ -53,32 +59,32 @@ void DiskFileModifier::insert(const QString &relativePath, quint64 size, char co
     // up with a negative int when converting from quint64 -> int.
     Q_ASSERT(size <= std::numeric_limits<int>::max());
 
-    _processArguments.append({ "insert", relativePath, QString::number(size), QChar::fromLatin1(contentChar) });
+    _processArguments.append({QStringLiteral("insert"), relativePath, QString::number(size), QChar::fromLatin1(contentChar)});
 }
 
 void DiskFileModifier::setContents(const QString &relativePath, quint64 newSize, char contentChar)
 {
-    _processArguments.append({ "contents", relativePath, QString::number(newSize), QChar::fromLatin1(contentChar) });
+    _processArguments.append({QStringLiteral("contents"), relativePath, QString::number(newSize), QChar::fromLatin1(contentChar)});
 }
 
 void DiskFileModifier::appendByte(const QString &relativePath, char contentChar)
 {
-    _processArguments.append({ "appendbyte", relativePath, QChar::fromLatin1(contentChar) });
+    _processArguments.append({QStringLiteral("appendbyte"), relativePath, QChar::fromLatin1(contentChar)});
 }
 
 void DiskFileModifier::mkdir(const QString &relativePath)
 {
-    _processArguments.append({ "mkdir", relativePath });
+    _processArguments.append({QStringLiteral("mkdir"), relativePath});
 }
 
 void DiskFileModifier::rename(const QString &from, const QString &to)
 {
-    _processArguments.append({ "rename", from, to });
+    _processArguments.append({QStringLiteral("rename"), from, to});
 }
 
 void DiskFileModifier::setModTime(const QString &relativePath, const QDateTime &modTime)
 {
-    _processArguments.append({ "mtime", relativePath, QString::number(modTime.toSecsSinceEpoch()) });
+    _processArguments.append({QStringLiteral("mtime"), relativePath, QString::number(modTime.toSecsSinceEpoch())});
 }
 
 class HelperProcess : public QProcess
@@ -107,7 +113,7 @@ public:
             });
 
         qDebug() << "Starting helper:" << TEST_HELPER_EXE << processArguments;
-        start(TEST_HELPER_EXE, processArguments);
+        start(QStringLiteral(TEST_HELPER_EXE), processArguments);
     }
 
 public:
@@ -128,6 +134,9 @@ bool DiskFileModifier::applyModifications()
         std::this_thread::sleep_for(50ms);
         QThread::currentThread()->eventDispatcher()->processEvents(QEventLoop::AllEvents);
     } while (!helper.finished);
+    if (!helper.succeeded) {
+        qWarning() << Q_FUNC_INFO << "failed";
+    }
     return helper.succeeded;
 }
 
@@ -357,10 +366,10 @@ FakePropfindReply::FakePropfindReply(FileInfo &remoteRootFileInfo, QNetworkAcces
     Q_ASSERT(!fileName.isNull()); // for root, it should be empty
     const FileInfo *fileInfo = remoteRootFileInfo.find(fileName);
     if (!fileInfo) {
-        QMetaObject::invokeMethod(this, "respond404", Qt::QueuedConnection);
+        QMetaObject::invokeMethod(this, &FakePropfindReply::respond404, Qt::QueuedConnection);
         return;
     }
-    const QString prefix = request.url().path().left(request.url().path().size() - fileName.size());
+    const auto prefix = QUrl(request.url().path().left(request.url().path().size() - fileName.size()));
 
     // Don't care about the request and just return a full propfind
     const QString davUri { QStringLiteral("DAV:") };
@@ -387,10 +396,9 @@ FakePropfindReply::FakePropfindReply(FileInfo &remoteRootFileInfo, QNetworkAcces
             xml.writeEmptyElement(davUri, QStringLiteral("resourcetype"));
 
         auto gmtDate = fileInfo.lastModifiedInUtc();
-        auto stringDate = QLocale::c().toString(gmtDate, QStringLiteral("ddd, dd MMM yyyy HH:mm:ss 'GMT'"));
-        xml.writeTextElement(davUri, QStringLiteral("getlastmodified"), stringDate);
+        xml.writeTextElement(davUri, QStringLiteral("getlastmodified"), OCC::Utility::formatRFC1123Date(gmtDate));
         xml.writeTextElement(davUri, QStringLiteral("getcontentlength"), QString::number(fileInfo.contentSize));
-        xml.writeTextElement(davUri, QStringLiteral("getetag"), QStringLiteral("\"%1\"").arg(QString::fromLatin1(fileInfo.etag)));
+        xml.writeTextElement(davUri, QStringLiteral("getetag"), QStringLiteral("\"%1\"").arg(QString::fromUtf8(fileInfo.etag)));
         xml.writeTextElement(ocUri, QStringLiteral("permissions"), !fileInfo.permissions.isNull() ? QString(fileInfo.permissions.toString()) : fileInfo.isShared ? QStringLiteral("SRDNVCKW")
                                                                                                                                                                  : QStringLiteral("RDNVCKW"));
         xml.writeTextElement(ocUri, QStringLiteral("id"), QString::fromUtf8(fileInfo.fileId));
@@ -413,7 +421,7 @@ FakePropfindReply::FakePropfindReply(FileInfo &remoteRootFileInfo, QNetworkAcces
     xml.writeEndElement(); // multistatus
     xml.writeEndDocument();
 
-    QMetaObject::invokeMethod(this, "respond", Qt::QueuedConnection);
+    QMetaObject::invokeMethod(this, &FakePropfindReply::respond, Qt::QueuedConnection);
 }
 
 void FakePropfindReply::respond()
@@ -421,19 +429,22 @@ void FakePropfindReply::respond()
     setHeader(QNetworkRequest::ContentLengthHeader, payload.size());
     setHeader(QNetworkRequest::ContentTypeHeader, QByteArrayLiteral("application/xml; charset=utf-8"));
     setAttribute(QNetworkRequest::HttpStatusCodeAttribute, 207);
-    setFinished(true);
-    emit metaDataChanged();
+    Q_EMIT metaDataChanged();
     if (bytesAvailable())
-        emit readyRead();
-    emit finished();
+        Q_EMIT readyRead();
+    checkedFinished();
 }
 
 void FakePropfindReply::respond404()
 {
+    if (isFinished()) {
+        return;
+    }
+
     setAttribute(QNetworkRequest::HttpStatusCodeAttribute, 404);
     setError(InternalServerError, QStringLiteral("Not Found"));
-    emit metaDataChanged();
-    emit finished();
+    Q_EMIT metaDataChanged();
+    checkedFinished();
 }
 
 qint64 FakePropfindReply::bytesAvailable() const
@@ -457,7 +468,7 @@ FakePutReply::FakePutReply(FileInfo &remoteRootFileInfo, QNetworkAccessManager::
     setOperation(op);
     open(QIODevice::ReadOnly);
     fileInfo = perform(remoteRootFileInfo, request, putPayload);
-    QMetaObject::invokeMethod(this, "respond", Qt::QueuedConnection);
+    QMetaObject::invokeMethod(this, &FakePutReply::respond, Qt::QueuedConnection);
 }
 
 FileInfo *FakePutReply::perform(FileInfo &remoteRootFileInfo, const QNetworkRequest &request, const QByteArray &putPayload)
@@ -480,20 +491,14 @@ FileInfo *FakePutReply::perform(FileInfo &remoteRootFileInfo, const QNetworkRequ
 
 void FakePutReply::respond()
 {
-    emit uploadProgress(fileInfo->contentSize, fileInfo->contentSize);
+    Q_EMIT uploadProgress(fileInfo->contentSize, fileInfo->contentSize);
     setRawHeader("OC-ETag", fileInfo->etag);
     setRawHeader("ETag", fileInfo->etag);
     setRawHeader("OC-FileID", fileInfo->fileId);
     setRawHeader("X-OC-MTime", "accepted"); // Prevents Q_ASSERT(!_runningNow) since we'll call PropagateItemJob::done twice in that case.
     setAttribute(QNetworkRequest::HttpStatusCodeAttribute, 200);
-    emit metaDataChanged();
-    emit finished();
-}
-
-void FakePutReply::abort()
-{
-    setError(OperationCanceledError, QStringLiteral("abort"));
-    emit finished();
+    Q_EMIT metaDataChanged();
+    checkedFinished();
 }
 
 FakeMkcolReply::FakeMkcolReply(FileInfo &remoteRootFileInfo, QNetworkAccessManager::Operation op, const QNetworkRequest &request, QObject *parent)
@@ -512,15 +517,18 @@ FakeMkcolReply::FakeMkcolReply(FileInfo &remoteRootFileInfo, QNetworkAccessManag
         abort();
         return;
     }
-    QMetaObject::invokeMethod(this, "respond", Qt::QueuedConnection);
+    QMetaObject::invokeMethod(this, &FakeMkcolReply::respond, Qt::QueuedConnection);
 }
 
 void FakeMkcolReply::respond()
 {
-    setRawHeader("OC-FileId", fileInfo->fileId);
-    setAttribute(QNetworkRequest::HttpStatusCodeAttribute, 201);
-    emit metaDataChanged();
-    emit finished();
+    if (error() == QNetworkReply::NoError) {
+        setRawHeader("OC-FileId", fileInfo->fileId);
+        setAttribute(QNetworkRequest::HttpStatusCodeAttribute, 201);
+        Q_EMIT metaDataChanged();
+    }
+
+    checkedFinished();
 }
 
 FakeDeleteReply::FakeDeleteReply(FileInfo &remoteRootFileInfo, QNetworkAccessManager::Operation op, const QNetworkRequest &request, QObject *parent)
@@ -534,14 +542,15 @@ FakeDeleteReply::FakeDeleteReply(FileInfo &remoteRootFileInfo, QNetworkAccessMan
     QString fileName = getFilePathFromUrl(request.url());
     Q_ASSERT(!fileName.isEmpty());
     remoteRootFileInfo.remove(fileName);
-    QMetaObject::invokeMethod(this, "respond", Qt::QueuedConnection);
+    QMetaObject::invokeMethod(this, &FakeDeleteReply::respond, Qt::QueuedConnection);
 }
 
 void FakeDeleteReply::respond()
 {
     setAttribute(QNetworkRequest::HttpStatusCodeAttribute, 204);
-    emit metaDataChanged();
-    emit finished();
+    Q_EMIT metaDataChanged();
+
+    checkedFinished();
 }
 
 FakeMoveReply::FakeMoveReply(FileInfo &remoteRootFileInfo, QNetworkAccessManager::Operation op, const QNetworkRequest &request, QObject *parent)
@@ -557,14 +566,17 @@ FakeMoveReply::FakeMoveReply(FileInfo &remoteRootFileInfo, QNetworkAccessManager
     QString dest = getFilePathFromUrl(QUrl::fromEncoded(request.rawHeader("Destination")));
     Q_ASSERT(!dest.isEmpty());
     remoteRootFileInfo.rename(fileName, dest);
-    QMetaObject::invokeMethod(this, "respond", Qt::QueuedConnection);
+    QMetaObject::invokeMethod(this, &FakeMoveReply::respond, Qt::QueuedConnection);
 }
 
 void FakeMoveReply::respond()
 {
-    setAttribute(QNetworkRequest::HttpStatusCodeAttribute, 201);
-    emit metaDataChanged();
-    emit finished();
+    if (error() == QNetworkReply::NoError) {
+        setAttribute(QNetworkRequest::HttpStatusCodeAttribute, 201);
+        Q_EMIT metaDataChanged();
+    }
+
+    checkedFinished();
 }
 
 FakeGetReply::FakeGetReply(FileInfo &remoteRootFileInfo, QNetworkAccessManager::Operation op, const QNetworkRequest &request, QObject *parent)
@@ -609,47 +621,48 @@ std::pair<qint64, qint64> FakeGetReply::parseRange(const QNetworkRequest &reques
 
 void FakeGetReply::respond()
 {
-    switch (state) {
-    case State::Aborted:
-        setError(OperationCanceledError, QStringLiteral("Operation Canceled"));
-        emit metaDataChanged();
-        break;
-    case State::FileNotFound:
-        setAttribute(QNetworkRequest::HttpStatusCodeAttribute, 404);
-        setError(ContentNotFoundError, QStringLiteral("File Not Found"));
-        emit metaDataChanged();
-        break;
-    case State::Ok:
-        payload = fileInfo->contentChar;
-        if (_range.second != 0) {
-            if (_range.second == -1) {
-                size = fileInfo->contentSize - _range.first;
+    qDebug() << this << state;
+    if (state != State::Aborted) {
+        switch (state) {
+        case State::Aborted:
+            Q_UNREACHABLE();
+        case State::FileNotFound:
+            setAttribute(QNetworkRequest::HttpStatusCodeAttribute, 404);
+            setError(ContentNotFoundError, QStringLiteral("File Not Found"));
+            Q_EMIT metaDataChanged();
+            break;
+        case State::Ok:
+            payload = fileInfo->contentChar;
+            if (_range.second != 0) {
+                if (_range.second == -1) {
+                    size = fileInfo->contentSize - _range.first;
+                } else {
+                    size = _range.second - _range.first;
+                }
+                setRawHeader("Content-Range", QByteArrayLiteral("bytes ") + QByteArray::number(_range.first) + '-');
             } else {
-                size = _range.second - _range.first;
+                size = fileInfo->contentSize;
             }
-            setRawHeader("Content-Range", QByteArrayLiteral("bytes ") + QByteArray::number(_range.first) + '-');
-        } else {
-            size = fileInfo->contentSize;
-        }
-        setHeader(QNetworkRequest::ContentLengthHeader, size);
-        setAttribute(QNetworkRequest::HttpStatusCodeAttribute, 200);
-        setRawHeader("OC-ETag", fileInfo->etag);
-        setRawHeader("ETag", fileInfo->etag);
-        setRawHeader("OC-FileId", fileInfo->fileId);
-        setRawHeader("X-OC-Mtime", QByteArray::number(fileInfo->lastModifiedInSecondsUTC()));
+            setHeader(QNetworkRequest::ContentLengthHeader, size);
+            setAttribute(QNetworkRequest::HttpStatusCodeAttribute, 200);
+            setRawHeader("OC-ETag", fileInfo->etag);
+            setRawHeader("ETag", fileInfo->etag);
+            setRawHeader("OC-FileId", fileInfo->fileId);
+            setRawHeader("X-OC-Mtime", QByteArray::number(fileInfo->lastModifiedInSecondsUTC()));
 
-        emit metaDataChanged();
-        if (bytesAvailable()) {
-            emit readyRead();
+            Q_EMIT metaDataChanged();
+            if (bytesAvailable()) {
+                Q_EMIT readyRead();
+            }
         }
+        checkedFinished();
     }
-    emit finished();
 }
 
 void FakeGetReply::abort()
 {
-    setError(OperationCanceledError, QStringLiteral("Operation Canceled"));
     state = State::Aborted;
+    FakeReply::abort();
 }
 
 qint64 FakeGetReply::bytesAvailable() const
@@ -756,22 +769,16 @@ void FakeChunkMoveReply::respond()
     setRawHeader("OC-ETag", fileInfo->etag);
     setRawHeader("ETag", fileInfo->etag);
     setRawHeader("OC-FileId", fileInfo->fileId);
-    emit metaDataChanged();
-    emit finished();
+    Q_EMIT metaDataChanged();
+    checkedFinished();
 }
 
 void FakeChunkMoveReply::respondPreconditionFailed()
 {
     setAttribute(QNetworkRequest::HttpStatusCodeAttribute, 412);
     setError(InternalServerError, QStringLiteral("Precondition Failed"));
-    emit metaDataChanged();
-    emit finished();
-}
-
-void FakeChunkMoveReply::abort()
-{
-    setError(OperationCanceledError, QStringLiteral("abort"));
-    emit finished();
+    Q_EMIT metaDataChanged();
+    checkedFinished();
 }
 
 FakePayloadReply::FakePayloadReply(QNetworkAccessManager::Operation op, const QNetworkRequest &request, const QByteArray &body, QObject *parent)
@@ -783,16 +790,17 @@ FakePayloadReply::FakePayloadReply(QNetworkAccessManager::Operation op, const QN
     setOperation(op);
     open(QIODevice::ReadOnly);
     QTimer::singleShot(10ms, this, &FakePayloadReply::respond);
+    setAttribute(QNetworkRequest::HttpStatusCodeAttribute, 200);
 }
 
 void FakePayloadReply::respond()
 {
-    setAttribute(QNetworkRequest::HttpStatusCodeAttribute, 200);
-    setHeader(QNetworkRequest::ContentLengthHeader, _body.size());
-    emit metaDataChanged();
-    emit readyRead();
-    setFinished(true);
-    emit finished();
+    if (error() == QNetworkReply::NoError) {
+        setHeader(QNetworkRequest::ContentLengthHeader, _body.size());
+        Q_EMIT metaDataChanged();
+        Q_EMIT readyRead();
+        checkedFinished();
+    }
 }
 
 qint64 FakePayloadReply::readData(char *buf, qint64 max)
@@ -817,22 +825,30 @@ FakeErrorReply::FakeErrorReply(QNetworkAccessManager::Operation op, const QNetwo
     setOperation(op);
     open(QIODevice::ReadOnly);
     setAttribute(QNetworkRequest::HttpStatusCodeAttribute, httpErrorCode);
-    setError(InternalServerError, QStringLiteral("Internal Server Fake Error"));
+    switch (httpErrorCode) {
+    case 401:
+        setError(AuthenticationRequiredError, QStringLiteral("Fake credentials error"));
+        break;
+    case 403:
+        setError(ContentAccessDenied, QStringLiteral("Fake access denied error"));
+        break;
+    default:
+        setError(InternalServerError, QStringLiteral("Internal Server Fake Error"));
+    }
     QMetaObject::invokeMethod(this, &FakeErrorReply::respond, Qt::QueuedConnection);
 }
 
 void FakeErrorReply::respond()
 {
-    emit metaDataChanged();
-    emit readyRead();
+    Q_EMIT metaDataChanged();
+    Q_EMIT readyRead();
     // finishing can come strictly after readyRead was called
     QTimer::singleShot(5ms, this, &FakeErrorReply::slotSetFinished);
 }
 
 void FakeErrorReply::slotSetFinished()
 {
-    setFinished(true);
-    emit finished();
+    checkedFinished();
 }
 
 qint64 FakeErrorReply::readData(char *buf, qint64 max)
@@ -857,18 +873,9 @@ FakeHangingReply::FakeHangingReply(QNetworkAccessManager::Operation op, const QN
     open(QIODevice::ReadOnly);
 }
 
-void FakeHangingReply::abort()
-{
-    // Follow more or less the implementation of QNetworkReplyImpl::abort
-    close();
-    setError(OperationCanceledError, tr("Operation canceled"));
-    emit error(OperationCanceledError);
-    setFinished(true);
-    emit finished();
-}
-
-FakeAM::FakeAM(FileInfo initialRoot)
-    : _remoteRootFileInfo { std::move(initialRoot) }
+FakeAM::FakeAM(FileInfo initialRoot, QObject *parent)
+    : OCC::AccessManager(parent)
+    , _remoteRootFileInfo{std::move(initialRoot)}
 {
     setCookieJar(new OCC::CookieJar);
 }
@@ -885,30 +892,35 @@ QNetworkReply *FakeAM::createRequest(QNetworkAccessManager::Operation op, const
     }
     if (!reply) {
         const QString fileName = getFilePathFromUrl(newRequest.url());
-        Q_ASSERT(!fileName.isNull());
-        if (_errorPaths.contains(fileName)) {
+        if (fileName.isNull()) {
+            // we don't actually handle this case but thats ok for now
+            newRequest.setTransferTimeout(duration_cast<milliseconds>(5min).count());
+            qWarning() << "Ignoring request to" << newRequest.url();
+            reply = new FakeHangingReply(op, newRequest, this);
+        } else if (_errorPaths.contains(fileName)) {
             reply = new FakeErrorReply { op, newRequest, this, _errorPaths[fileName] };
         }
     }
     if (!reply) {
-        const bool isUpload = newRequest.url().path().startsWith(sUploadUrl.path());
+        // detect chunking ng upload
+        const bool isUpload = newRequest.url().path().startsWith(chunkingNgUploadPathC());
         FileInfo &info = isUpload ? _uploadFileInfo : _remoteRootFileInfo;
 
-        auto verb = newRequest.attribute(QNetworkRequest::CustomVerbAttribute);
-        if (verb == QLatin1String("PROPFIND"))
+        const auto verb = newRequest.attribute(QNetworkRequest::CustomVerbAttribute).toByteArray();
+        if (verb == QByteArrayLiteral("PROPFIND"))
             // Ignore outgoingData always returning somethign good enough, works for now.
             reply = new FakePropfindReply { info, op, newRequest, this };
-        else if (verb == QLatin1String("GET") || op == QNetworkAccessManager::GetOperation)
+        else if (verb == QByteArrayLiteral("GET") || op == QNetworkAccessManager::GetOperation)
             reply = new FakeGetReply { info, op, newRequest, this };
-        else if (verb == QLatin1String("PUT") || op == QNetworkAccessManager::PutOperation)
+        else if (verb == QByteArrayLiteral("PUT") || op == QNetworkAccessManager::PutOperation)
             reply = new FakePutReply { info, op, newRequest, outgoingData->readAll(), this };
-        else if (verb == QLatin1String("MKCOL"))
+        else if (verb == QByteArrayLiteral("MKCOL"))
             reply = new FakeMkcolReply { info, op, newRequest, this };
-        else if (verb == QLatin1String("DELETE") || op == QNetworkAccessManager::DeleteOperation)
+        else if (verb == QByteArrayLiteral("DELETE") || op == QNetworkAccessManager::DeleteOperation)
             reply = new FakeDeleteReply { info, op, newRequest, this };
-        else if (verb == QLatin1String("MOVE") && !isUpload)
+        else if (verb == QByteArrayLiteral("MOVE") && !isUpload)
             reply = new FakeMoveReply { info, op, newRequest, this };
-        else if (verb == QLatin1String("MOVE") && isUpload)
+        else if (verb == QByteArrayLiteral("MOVE") && isUpload)
             reply = new FakeChunkMoveReply { info, _remoteRootFileInfo, op, newRequest, this };
         else {
             qDebug() << verb << outgoingData;
@@ -929,31 +941,24 @@ FakeFolder::FakeFolder(const FileInfo &fileTemplate, OCC::Vfs::Mode vfsMode, boo
     : _localModifier(_tempDir.path())
 {
     // Needs to be done once
-    OCC::SyncEngine::minimumFileAgeForUpload = std::chrono::milliseconds(0);
+    OCC::SyncEngine::minimumFileAgeForUpload = 0s;
 
     QDir rootDir { _tempDir.path() };
     qDebug() << "FakeFolder operating on" << rootDir;
     toDisk(rootDir, filesAreDehydrated ? FileInfo() : fileTemplate);
 
-    _fakeAm = new FakeAM(fileTemplate);
-    _account = OCC::TestUtils::createDummyAccount();
-    _account->setCredentials(new FakeCredentials { _fakeAm });
+    _fakeAm = new FakeAM(fileTemplate, this);
+    _accountState = std::move(OCC::TestUtils::createDummyAccount());
+    account()->setCredentials(new FakeCredentials{_fakeAm});
 
     _journalDb.reset(new OCC::SyncJournalDb(localPath() + QStringLiteral(".sync_test.db")));
     // TODO: davUrl
 
-    _syncEngine.reset(new OCC::SyncEngine(_account, _account->davUrl(), localPath(), QString(), _journalDb.get()));
+    _syncEngine.reset(new OCC::SyncEngine(account(), account()->davUrl(), localPath(), QString(), _journalDb.get()));
     _syncEngine->setSyncOptions(OCC::SyncOptions { QSharedPointer<OCC::Vfs>(OCC::VfsPluginManager::instance().createVfsFromPlugin(vfsMode).release()) });
 
     // Ignore temporary files from the download. (This is in the default exclude list, but we don't load it)
-    _syncEngine->excludedFiles().addManualExclude(QStringLiteral("]*.~*"));
-
-    // handle aboutToRemoveAllFiles with a timeout in case our test does not handle it
-    QObject::connect(_syncEngine.get(), &OCC::SyncEngine::aboutToRemoveAllFiles, _syncEngine.get(), [this](OCC::SyncFileItem::Direction, const std::function<void(bool)> &callback) {
-        QTimer::singleShot(1s, _syncEngine.get(), [callback] {
-            callback(false);
-        });
-    });
+    _syncEngine->addManualExclude(QStringLiteral("]*.~*"));
 
     auto vfs = _syncEngine->syncOptions()._vfs;
     if (vfsMode != vfs->mode()) {
@@ -967,7 +972,7 @@ FakeFolder::FakeFolder(const FileInfo &fileTemplate, OCC::Vfs::Mode vfsMode, boo
     if (vfsMode != OCC::Vfs::Off) {
         const auto pinState = filesAreDehydrated ? OCC::PinState::OnlineOnly : OCC::PinState::AlwaysLocal;
         syncJournal().internalPinStates().setForPath("", pinState);
-        OC_ENFORCE(vfs->setPinState("", pinState));
+        OC_ENFORCE(vfs->setPinState(QString(), pinState));
     }
 
     // A new folder will update the local file state database on first sync.
@@ -976,6 +981,8 @@ FakeFolder::FakeFolder(const FileInfo &fileTemplate, OCC::Vfs::Mode vfsMode, boo
     OC_ENFORCE(syncOnce())
 }
 
+FakeFolder::~FakeFolder() { }
+
 void FakeFolder::switchToVfs(QSharedPointer<OCC::Vfs> vfs)
 {
     auto opts = _syncEngine->syncOptions();
@@ -986,7 +993,7 @@ void FakeFolder::switchToVfs(QSharedPointer<OCC::Vfs> vfs)
     opts._vfs = vfs;
     _syncEngine->setSyncOptions(opts);
 
-    OCC::VfsSetupParams vfsParams(_account, _account->davUrl(), false);
+    OCC::VfsSetupParams vfsParams(account(), account()->davUrl(), false, &syncEngine());
     vfsParams.filesystemPath = localPath();
     vfsParams.remotePath = QLatin1Char('/');
     vfsParams.journal = _journalDb.get();
@@ -1089,34 +1096,41 @@ void FakeFolder::toDisk(QDir &dir, const FileInfo &templateFi)
 
 void FakeFolder::fromDisk(QDir &dir, FileInfo &templateFi)
 {
-    const auto infoList = dir.entryInfoList(QDir::AllEntries | QDir::NoDotAndDotDot);
-    for (const auto &diskChild : infoList) {
-        if (diskChild.isHidden() || diskChild.fileName().startsWith(QStringLiteral(".sync_"))) {
-            // Skip system files, sqlite db files, sync log, etc.
+    auto dh = csync_vio_local_opendir(dir.absolutePath());
+    if (!dh) {
+        return;
+    }
+    while (true) {
+        auto dirent = csync_vio_local_readdir(dh, nullptr);
+        if (!dirent)
+            break;
+        if (dirent->type == ItemTypeSkip)
+            continue;
+        if (dirent->is_hidden || dirent->path.startsWith(QStringLiteral(".sync_")))
             continue;
-        }
 
-        if (diskChild.isDir()) {
+        QString absolutePathItem = dir.absolutePath() + QDir::separator() + dirent->path;
+        if (dirent->type == ItemTypeDirectory) {
             QDir subDir = dir;
-            subDir.cd(diskChild.fileName());
-            FileInfo &subFi = templateFi.children[diskChild.fileName()] = FileInfo { diskChild.fileName() };
-            subFi.setLastModified(diskChild.lastModified());
+            subDir.cd(dirent->path);
+            FileInfo &subFi = templateFi.children[dirent->path] = FileInfo{dirent->path};
+            subFi.setLastModified(QDateTime::fromSecsSinceEpoch(dirent->modtime, QTimeZone::utc()));
             fromDisk(subDir, subFi);
         } else {
-            FileInfo fi(diskChild.fileName());
+            FileInfo fi(dirent->path);
             fi.isDir = false;
-            fi.fileSize = diskChild.size();
-            fi.isDehydratedPlaceholder = isDehydratedPlaceholder(diskChild.absoluteFilePath());
-            fi.setLastModified(diskChild.lastModified());
+            fi.fileSize = dirent->size;
+            fi.isDehydratedPlaceholder = isDehydratedPlaceholder(absolutePathItem);
+            fi.setLastModified(QDateTime::fromSecsSinceEpoch(dirent->modtime, QTimeZone::utc()));
             if (fi.isDehydratedPlaceholder) {
                 fi.contentChar = '\0';
                 fi.contentSize = 0;
             } else {
-                QFile f { diskChild.filePath() };
+                QFile f{absolutePathItem};
                 OC_ENFORCE(f.open(QFile::ReadOnly));
                 auto content = f.read(1);
                 if (content.size() == 0) {
-                    qWarning() << "Empty file at:" << diskChild.filePath();
+                    qWarning() << "Empty file at:" << dirent->path;
                     fi.contentChar = FileInfo::DefaultContentChar;
                 } else {
                     fi.contentChar = content.at(0);
@@ -1127,6 +1141,7 @@ void FakeFolder::fromDisk(QDir &dir, FileInfo &templateFi)
             templateFi.children.insert(fi.name, fi);
         }
     }
+    csync_vio_local_closedir(dh);
 }
 
 FileInfo &findOrCreateDirs(FileInfo &base, const PathComponents &components)
@@ -1165,6 +1180,31 @@ FileInfo FakeFolder::dbState() const
     return result;
 }
 
+bool FakeFolder::execUntilFinished()
+{
+    QSignalSpy spy(_syncEngine.get(), &OCC::SyncEngine::finished);
+    bool ok = spy.wait(3600000);
+    Q_ASSERT(ok && "Sync timed out");
+    return spy[0][0].toBool();
+}
+
+bool FakeFolder::syncOnce()
+{
+    QObject connectScope;
+    QList<QPair<QString, OCC::ErrorCategory>> errors;
+    connect(_syncEngine.get(), &OCC::SyncEngine::syncError, &connectScope,
+        [&errors](const QString &message, OCC::ErrorCategory category) { errors << qMakePair(message, category); });
+    OCC::SyncResult result;
+    connect(
+        _syncEngine.get(), &OCC::SyncEngine::itemCompleted, &connectScope, [&result](const OCC::SyncFileItemPtr &item) { result.processCompletedItem(item); });
+    scheduleSync();
+    const bool ok = execUntilFinished();
+    if (!ok) {
+        qWarning() << Q_FUNC_INFO << "failed. Errors:" << errors << "Another sync needed:" << _syncEngine->isAnotherSyncNeeded() << result.errorStrings();
+    }
+    return ok;
+}
+
 OCC::SyncFileItemPtr ItemCompletedSpy::findItem(const QString &path) const
 {
     for (const QList<QVariant> &args : *this) {
@@ -1178,9 +1218,52 @@ OCC::SyncFileItemPtr ItemCompletedSpy::findItem(const QString &path) const
 FakeReply::FakeReply(QObject *parent)
     : QNetworkReply(parent)
 {
-    setRawHeader(QByteArrayLiteral("Date"), QDateTime::currentDateTimeUtc().toString(Qt::RFC2822Date).toUtf8());
+    setRawHeader(QByteArrayLiteral("Date"), OCC::Utility::formatRFC1123Date(QDateTime::currentDateTimeUtc()).toUtf8());
+    QTimer::singleShot(0, this, [this] {
+        // emulate the real world
+        // don't Q_EMIT if we where already aborted
+        if (!isFinished() && error() != QNetworkReply::OperationCanceledError) {
+            Q_EMIT requestSent();
+        }
+    });
 }
 
-FakeReply::~FakeReply()
+FakeReply::~FakeReply() { }
+
+void FakeReply::checkedFinished()
 {
+    // It can happen that this job is cancelled before the full data is read, and the job is aborted. This being the test framework, processing happens
+    // synchronously, so between the signals above and here, the job can get aborted, which emits the finished signal. So do NOT emit it again if
+    // this is the case.
+    if (!isFinished()) {
+        setFinished(true);
+        Q_EMIT finished();
+    }
+}
+
+void FakeReply::abort()
+{
+    if (!isFinished()) {
+        setError(OperationCanceledError, QStringLiteral("Operation Canceled"));
+        Q_EMIT metaDataChanged();
+        checkedFinished();
+    }
+}
+
+QString getFilePathFromUrl(const QUrl &url)
+{
+    const QString path = url.path();
+    // old school dav url
+    const QString sRootUrl = QStringLiteral("/owncloud/remote.php/webdav/");
+    // more modern dav url including user name
+    const QString sRootUrl2 = QStringLiteral("/owncloud/remote.php/dav/files/admin/");
+
+    if (path.startsWith(sRootUrl)) {
+        return path.mid(sRootUrl.length());
+    } else if (path.startsWith(sRootUrl2)) {
+        return path.mid(sRootUrl2.length());
+    } else if (path.startsWith(chunkingNgUploadPathC())) {
+        return path.mid(chunkingNgUploadPathC().length());
+    }
+    return {};
 }
