@@ -35,22 +35,7 @@ using namespace OCC::FileSystem::SizeLiterals;
  * only works when directly called from a QTest :-(
  */
 
-
-static const QUrl sRootUrl = QUrl::fromEncoded("owncloud://somehost/owncloud/remote.php/webdav/");
-static const QUrl sRootUrl2 = QUrl::fromEncoded("owncloud://somehost/owncloud/remote.php/dav/files/admin/");
-static const QUrl sUploadUrl = QUrl::fromEncoded("owncloud://somehost/owncloud/remote.php/dav/uploads/admin/");
-
-inline QString getFilePathFromUrl(const QUrl &url)
-{
-    QString path = url.path();
-    if (path.startsWith(sRootUrl.path()))
-        return path.mid(sRootUrl.path().length());
-    if (path.startsWith(sRootUrl2.path()))
-        return path.mid(sRootUrl2.path().length());
-    if (path.startsWith(sUploadUrl.path()))
-        return path.mid(sUploadUrl.path().length());
-    return {};
-}
+QString getFilePathFromUrl(const QUrl &url);
 
 
 inline QByteArray generateEtag()
@@ -65,7 +50,6 @@ inline QByteArray generateFileId()
 class PathComponents : public QStringList
 {
 public:
-    PathComponents(const char *path);
     PathComponents(const QString &path);
     PathComponents(const QStringList &pathComponents);
 
@@ -87,7 +71,7 @@ public:
 class FileModifier
 {
 public:
-    static constexpr auto DefaultFileSize = 64_b;
+    static constexpr auto DefaultFileSize = 64_B;
     static constexpr char DefaultContentChar = 'X';
 
     virtual ~FileModifier() { }
@@ -215,6 +199,7 @@ public:
 
     /// Return a pointer to the FileInfo, or a nullptr if it doesn't exist
     FileInfo *find(PathComponents pathComponents, const bool invalidateEtags = false);
+    FileInfo *find(const QString &pathComponents, const bool invalidateEtags = false) { return find(PathComponents{pathComponents}, invalidateEtags); }
 
     FileInfo *createDir(const QString &relativePath);
 
@@ -321,10 +306,16 @@ public:
     virtual ~FakeReply() override;
 
     // useful to be public for testing
+    using QNetworkReply::setAttribute;
     using QNetworkReply::setRawHeader;
+
+    void checkedFinished();
+
+    virtual void abort() override;
 };
 
 class FakePropfindReply : public FakeReply
+
 {
     Q_OBJECT
 
@@ -337,8 +328,6 @@ public:
 
     Q_INVOKABLE void respond404();
 
-    void abort() override { }
-
     qint64 bytesAvailable() const override;
     qint64 readData(char *data, qint64 maxlen) override;
 };
@@ -354,7 +343,6 @@ public:
 
     Q_INVOKABLE virtual void respond();
 
-    void abort() override;
     qint64 readData(char *, qint64) override { return 0; }
 
 private:
@@ -370,7 +358,6 @@ public:
 
     Q_INVOKABLE void respond();
 
-    void abort() override { }
     qint64 readData(char *, qint64) override { return 0; }
 
 private:
@@ -386,7 +373,6 @@ public:
 
     Q_INVOKABLE void respond();
 
-    void abort() override { }
     qint64 readData(char *, qint64) override { return 0; }
 };
 
@@ -399,7 +385,6 @@ public:
 
     Q_INVOKABLE void respond();
 
-    void abort() override { }
     qint64 readData(char *, qint64) override { return 0; }
 };
 
@@ -413,6 +398,7 @@ public:
         Aborted,
         FileNotFound,
     };
+    Q_ENUM(State);
 
     const FileInfo *fileInfo;
     char payload;
@@ -422,7 +408,7 @@ public:
 
     FakeGetReply(FileInfo &remoteRootFileInfo, QNetworkAccessManager::Operation op, const QNetworkRequest &request, QObject *parent);
 
-    Q_INVOKABLE void respond();
+    virtual void respond();
 
     void abort() override;
     virtual qint64 bytesAvailable() const override;
@@ -448,8 +434,6 @@ public:
 
     Q_INVOKABLE void respondPreconditionFailed();
 
-    void abort() override;
-
     qint64 readData(char *, qint64) override { return 0; }
 };
 
@@ -460,9 +444,8 @@ public:
     FakePayloadReply(QNetworkAccessManager::Operation op, const QNetworkRequest &request,
         const QByteArray &body, QObject *parent);
 
-    void respond();
+    virtual void respond();
 
-    void abort() override { }
     qint64 readData(char *buf, qint64 max) override;
     qint64 bytesAvailable() const override;
     QByteArray _body;
@@ -482,11 +465,10 @@ public:
     using QNetworkReply::setAttribute;
     using QNetworkReply::setError;
 
-public slots:
+public Q_SLOTS:
     void slotSetFinished();
 
 public:
-    void abort() override { }
     qint64 readData(char *buf, qint64 max) override;
     qint64 bytesAvailable() const override;
 
@@ -500,7 +482,6 @@ class FakeHangingReply : public FakeReply
 public:
     FakeHangingReply(QNetworkAccessManager::Operation op, const QNetworkRequest &request, QObject *parent);
 
-    void abort() override;
     qint64 readData(char *, qint64) override { return 0; }
 };
 
@@ -540,7 +521,7 @@ private:
     Override _override;
 
 public:
-    FakeAM(FileInfo initialRoot);
+    FakeAM(FileInfo initialRoot, QObject *parent);
     FileInfo &currentRemoteState() { return _remoteRootFileInfo; }
     FileInfo &uploadState() { return _uploadFileInfo; }
 
@@ -576,22 +557,24 @@ private:
     OCC::AccessManager *_am;
 };
 
-class FakeFolder
+class FakeFolder : public QObject
 {
+    Q_OBJECT
     const QTemporaryDir _tempDir = OCC::TestUtils::createTempDir();
     DiskFileModifier _localModifier;
-    // FIXME: Clarify ownership, double delete
     FakeAM *_fakeAm;
-    OCC::AccountPtr _account;
+    OCC::TestUtils::TestUtilsPrivate::AccountStateRaii _accountState =
+        OCC::TestUtils::TestUtilsPrivate::AccountStateRaii{nullptr, &OCC::TestUtils::TestUtilsPrivate::accountStateDeleter};
     std::unique_ptr<OCC::SyncJournalDb> _journalDb;
     std::unique_ptr<OCC::SyncEngine> _syncEngine;
 
 public:
     FakeFolder(const FileInfo &fileTemplate, OCC::Vfs::Mode vfsMode = OCC::Vfs::Off, bool filesAreDehydrated = false);
+    ~FakeFolder();
 
     void switchToVfs(QSharedPointer<OCC::Vfs> vfs);
 
-    OCC::AccountPtr account() const { return _account; }
+    OCC::AccountPtr account() const { return _accountState->account(); }
     OCC::SyncEngine &syncEngine() const { return *_syncEngine; }
     OCC::SyncJournalDb &syncJournal() const { return *_journalDb; }
 
@@ -623,19 +606,9 @@ public:
 
     void execUntilItemCompleted(const QString &relativePath);
 
-    bool execUntilFinished()
-    {
-        QSignalSpy spy(_syncEngine.get(), &OCC::SyncEngine::finished);
-        bool ok = spy.wait(3600000);
-        Q_ASSERT(ok && "Sync timed out");
-        return spy[0][0].toBool();
-    }
+    bool execUntilFinished();
 
-    bool syncOnce()
-    {
-        scheduleSync();
-        return execUntilFinished();
-    }
+    bool syncOnce();
 
     Q_REQUIRED_RESULT bool applyLocalModificationsAndSync()
     {
@@ -698,8 +671,8 @@ inline void addFiles(QStringList &dest, const FileInfo &fi)
 {
     if (fi.isDir) {
         dest += QStringLiteral("%1 - dir").arg(fi.path());
-        for (const auto &fi : fi.children)
-            addFiles(dest, fi);
+        for (const auto &child : fi.children)
+            addFiles(dest, child);
     } else {
         dest += QStringLiteral("%1 - %2 %3-bytes (%4)").arg(fi.path(), QString::number(fi.contentSize), QChar::fromLatin1(fi.contentChar), fi.lastModifiedInUtc().toString());
     }
@@ -708,8 +681,9 @@ inline void addFiles(QStringList &dest, const FileInfo &fi)
 inline QString toStringNoElide(const FileInfo &fi)
 {
     QStringList files;
-    for (const auto &fi : fi.children)
-        addFiles(files, fi);
+    for (const auto &childInfo : fi.children) {
+        addFiles(files, childInfo);
+    }
     files.sort();
     return QStringLiteral("FileInfo with %1 files(\n\t%2\n)").arg(files.size()).arg(files.join(QStringLiteral("\n\t")));
 }
@@ -724,8 +698,8 @@ inline void addFilesDbData(QStringList &dest, const FileInfo &fi)
     // could include etag, permissions etc, but would need extra work
     if (fi.isDir) {
         dest += QStringLiteral("%1 - %2 %3 %4").arg(fi.name, fi.isDir ? QStringLiteral("dir") : QStringLiteral("file"), QString::number(fi.lastModifiedInSecondsUTC()), QString::fromUtf8(fi.fileId));
-        for (const auto &fi : fi.children)
-            addFilesDbData(dest, fi);
+        for (const auto &child : fi.children)
+            addFilesDbData(dest, child);
     } else {
         dest += QStringLiteral("%1 - %2 %3 %4 %5").arg(fi.name, fi.isDir ? QStringLiteral("dir") : QStringLiteral("file"), QString::number(fi.contentSize), QString::number(fi.lastModifiedInSecondsUTC()), QString::fromUtf8(fi.fileId));
     }
@@ -734,11 +708,16 @@ inline void addFilesDbData(QStringList &dest, const FileInfo &fi)
 inline char *printDbData(const FileInfo &fi)
 {
     QStringList files;
-    for (const auto &fi : fi.children)
-        addFilesDbData(files, fi);
+    for (const auto &child : fi.children)
+        addFilesDbData(files, child);
     return QTest::toString(QStringLiteral("FileInfo with %1 files(%2)").arg(files.size()).arg(files.join(QStringLiteral(", "))));
 }
 
+/**
+ * @brief Utility class that count the number of GET/PUT/MOVE/DELETE operations during a sync.
+ *
+ * This can be used for subsequent syncs, but the counters need to be reset.
+ */
 struct OperationCounter
 {
     int nGET = 0;
@@ -746,7 +725,7 @@ struct OperationCounter
     int nMOVE = 0;
     int nDELETE = 0;
 
-    OperationCounter() {};
+    OperationCounter() { }
     OperationCounter(const OperationCounter &) = delete;
     OperationCounter(OperationCounter &&) = delete;
     void operator=(OperationCounter const &) = delete;
@@ -768,7 +747,7 @@ struct OperationCounter
             ++nPUT;
         } else if (op == QNetworkAccessManager::DeleteOperation) {
             ++nDELETE;
-        } else if (req.attribute(QNetworkRequest::CustomVerbAttribute) == QLatin1String("MOVE")) {
+        } else if (req.attribute(QNetworkRequest::CustomVerbAttribute).toByteArray() == QByteArrayLiteral("MOVE")) {
             ++nMOVE;
         }
         return nullptr;
